"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const react_1 = require("react");
const locale_1 = require("@joplin/lib/locale");
const Setting_1 = require("@joplin/lib/models/Setting");
const focusHandler_1 = require("@joplin/lib/utils/focusHandler");
const FontSearch = (props) => {
    const { type, style, value, availableFonts, onChange, subtype } = props;
    const [filteredAvailableFonts, setFilteredAvailableFonts] = (0, react_1.useState)(availableFonts);
    const [inputText, setInputText] = (0, react_1.useState)(value);
    const [showList, setShowList] = (0, react_1.useState)(false);
    const [isListHovered, setIsListHovered] = (0, react_1.useState)(false);
    const [isFontSelected, setIsFontSelected] = (0, react_1.useState)(value !== '');
    const [visibleFonts, setVisibleFonts] = (0, react_1.useState)([]);
    const [isMonoBoxChecked, setIsMonoBoxChecked] = (0, react_1.useState)(false);
    const isLoadingFonts = filteredAvailableFonts.length === 0;
    const fontInputRef = (0, react_1.useRef)(null);
    (0, react_1.useEffect)(() => {
        if (subtype === Setting_1.SettingItemSubType.MonospaceFontFamily) {
            setIsMonoBoxChecked(true);
        }
    }, [subtype]);
    (0, react_1.useEffect)(() => {
        if (!isMonoBoxChecked)
            return setFilteredAvailableFonts(availableFonts);
        const localMonospacedFonts = availableFonts.filter((font) => monospaceKeywords.some((word) => font.toLowerCase().includes(word)) ||
            knownMonospacedFonts.includes(font.toLowerCase()));
        setFilteredAvailableFonts(localMonospacedFonts);
    }, [isMonoBoxChecked, availableFonts]);
    const displayedFonts = (0, react_1.useMemo)(() => {
        if (isFontSelected)
            return filteredAvailableFonts;
        return filteredAvailableFonts.filter((font) => font.toLowerCase().startsWith(inputText.toLowerCase()));
    }, [filteredAvailableFonts, inputText, isFontSelected]);
    (0, react_1.useEffect)(() => {
        setVisibleFonts(displayedFonts.slice(0, 20));
    }, [displayedFonts]);
    // Lazy loading
    const handleListScroll = (0, react_1.useCallback)((event) => {
        const scrollTop = event.target.scrollTop;
        const scrollHeight = event.target.scrollHeight;
        const clientHeight = event.target.clientHeight;
        // Check if the user has scrolled to the bottom of the container
        // A small buffer of 20 pixels is subtracted from the total scrollHeight to ensure new content starts loading slightly before the user reaches the absolute bottom, providing a smoother experience.
        if (scrollTop + clientHeight >= scrollHeight - 20) {
            // Load the next 20 fonts
            const remainingFonts = displayedFonts.slice(visibleFonts.length, visibleFonts.length + 20);
            setVisibleFonts([...visibleFonts, ...remainingFonts]);
        }
    }, [displayedFonts, visibleFonts]);
    const handleTextChange = (0, react_1.useCallback)((event) => {
        setIsFontSelected(false);
        setInputText(event.target.value);
        onChange(event.target.value);
    }, [onChange]);
    const handleFocus = (0, react_1.useCallback)(() => setShowList(true), []);
    const handleBlur = (0, react_1.useCallback)(() => {
        if (!isListHovered) {
            setShowList(false);
        }
    }, [isListHovered]);
    const handleFontClick = (0, react_1.useCallback)((event) => {
        const font = event.target.innerText;
        setInputText(font);
        setShowList(false);
        onChange(font);
        setIsFontSelected(true);
    }, [onChange]);
    const handleListHover = (0, react_1.useCallback)(() => setIsListHovered(true), []);
    const handleListLeave = (0, react_1.useCallback)(() => setIsListHovered(false), []);
    const handleMonoBoxCheck = (0, react_1.useCallback)(() => {
        setIsMonoBoxChecked(!isMonoBoxChecked);
        (0, focusHandler_1.focus)('FontSearch::fontInputRef', fontInputRef.current);
    }, [isMonoBoxChecked]);
    return (React.createElement(React.Fragment, null,
        React.createElement("input", { type: type, style: style, value: inputText, onChange: handleTextChange, onFocus: handleFocus, onBlur: handleBlur, spellCheck: false, id: props.inputId, ref: fontInputRef }),
        React.createElement("div", { className: 'font-search-list', style: { display: showList ? 'block' : 'none' }, onMouseEnter: handleListHover, onMouseLeave: handleListLeave, onScroll: handleListScroll }, isLoadingFonts ? React.createElement("div", null, (0, locale_1._)('Loading...')) :
            React.createElement(React.Fragment, null,
                React.createElement("div", { className: 'monospace-checkbox' },
                    React.createElement("input", { type: 'checkbox', checked: isMonoBoxChecked, onChange: handleMonoBoxCheck, id: `show-monospace-fonts_${subtype}` }),
                    React.createElement("label", { htmlFor: `show-monospace-fonts_${subtype}` }, (0, locale_1._)('Show monospace fonts only.'))),
                visibleFonts.map((font) => React.createElement("div", { key: font, style: { fontFamily: `"${font}"` }, onClick: handleFontClick, className: 'font-search-item' }, font))))));
};
exports.default = FontSearch;
// Known monospaced fonts from wikipedia
// https://en.wikipedia.org/wiki/List_of_monospaced_typefaces
// https://en.wikipedia.org/wiki/Category:Monospaced_typefaces
// Make sure to add the fonts in lower case
// cSpell:disable
const knownMonospacedFonts = [
    'andalé mono',
    'anonymous pro',
    'bitstream vera sans mono',
    'cascadia code',
    'century schoolbook monospace',
    'comic mono',
    'computer modern mono/typewriter',
    'consolas',
    'courier',
    'courier final draft',
    'courier new',
    'courier prime',
    'courier screenplay',
    'cousine',
    'dejavu sans mono',
    'droid sans mono',
    'envy code r',
    'everson mono',
    'fantasque sans mono',
    'fira code',
    'fira mono',
    'fixed',
    'fixedsys',
    'freemono',
    'go mono',
    'hack',
    'hyperfont',
    'ibm courier',
    'ibm plex mono',
    'inconsolata',
    'input',
    'iosevka',
    'jetbrains mono',
    'juliamono',
    'letter gothic',
    'liberation mono',
    'lucida console',
    'menlo',
    'monaco',
    'monofur',
    'monospace (unicode)',
    'nimbus mono l',
    'nk57 monospace',
    'noto mono',
    'ocr-a',
    'ocr-b',
    'operator mono',
    'overpass mono',
    'oxygen mono',
    'pragmatapro',
    'profont',
    'pt mono',
    'recursive mono',
    'roboto mono',
    'sf mono',
    'source code pro',
    'spleen',
    'terminal',
    'terminus',
    'tex gyre cursor',
    'ubuntu mono',
    'victor mono',
    'wumpus mono',
];
const monospaceKeywords = [
    'mono',
    'code',
    'courier',
    'console',
    'source code',
    'terminal',
    'fixed',
];
//# sourceMappingURL=FontSearch.js.map