"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const react_1 = require("react");
const locale_1 = require("@joplin/lib/locale");
const theme_1 = require("@joplin/lib/theme");
const useKeyboardHandler_1 = require("./DialogButtonRow/useKeyboardHandler");
function DialogButtonRow(props) {
    const theme = (0, theme_1.themeStyle)(props.themeId);
    const buttonStyle = (0, react_1.useMemo)(() => {
        return Object.assign(Object.assign({}, theme.buttonStyle), { marginLeft: 10 });
    }, [theme.buttonStyle]);
    const onOkButtonClick = (0, react_1.useCallback)(() => {
        if (props.onClick && !props.okButtonDisabled)
            props.onClick({ buttonName: 'ok' });
    }, [props.onClick, props.okButtonDisabled]);
    const onCancelButtonClick = (0, react_1.useCallback)(() => {
        if (props.onClick && !props.cancelButtonDisabled)
            props.onClick({ buttonName: 'cancel' });
    }, [props.onClick, props.cancelButtonDisabled]);
    const onCustomButtonClick = (0, react_1.useCallback)((event) => {
        if (props.onClick)
            props.onClick(event);
    }, [props.onClick]);
    const onKeyDown = (0, useKeyboardHandler_1.default)({ onOkButtonClick, onCancelButtonClick });
    const buttonComps = [];
    if (props.customButtons) {
        for (const b of props.customButtons) {
            buttonComps.push(React.createElement("button", { key: b.name, style: buttonStyle, onClick: () => onCustomButtonClick({ buttonName: b.name }), onKeyDown: onKeyDown }, b.label));
        }
    }
    if (props.okButtonShow !== false) {
        buttonComps.push(React.createElement("button", { disabled: props.okButtonDisabled, key: "ok", style: buttonStyle, onClick: onOkButtonClick, ref: props.okButtonRef, onKeyDown: onKeyDown }, props.okButtonLabel ? props.okButtonLabel : (0, locale_1._)('OK')));
    }
    if (props.cancelButtonShow !== false) {
        buttonComps.push(React.createElement("button", { disabled: props.cancelButtonDisabled, key: "cancel", style: Object.assign({}, buttonStyle), onClick: onCancelButtonClick }, props.cancelButtonLabel ? props.cancelButtonLabel : (0, locale_1._)('Cancel')));
    }
    return React.createElement("div", { style: { textAlign: 'right', marginTop: 10 } }, buttonComps);
}
exports.default = DialogButtonRow;
//# sourceMappingURL=DialogButtonRow.js.map