"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = require("react");
const fontSizeCache_ = {};
exports.default = (props) => {
    const containerRef = (0, react_1.useRef)(null);
    const [containerReady, setContainerReady] = (0, react_1.useState)(false);
    const fontSize = (0, react_1.useMemo)(() => {
        if (!containerReady)
            return props.height;
        const cacheKey = [props.width, props.height, props.emoji].join('-');
        if (fontSizeCache_[cacheKey]) {
            return fontSizeCache_[cacheKey];
        }
        // Set the emoji font size so that it fits within the specified width
        // and height. In fact, currently it only looks at the height.
        let spanFontSize = props.height;
        const span = document.createElement('span');
        span.innerText = props.emoji;
        span.style.fontSize = `${spanFontSize}px`;
        containerRef.current.appendChild(span);
        let rect = span.getBoundingClientRect();
        while (rect.height > props.height) {
            spanFontSize -= .5;
            span.style.fontSize = `${spanFontSize}px`;
            rect = span.getBoundingClientRect();
        }
        span.remove();
        fontSizeCache_[cacheKey] = spanFontSize;
        return spanFontSize;
    }, [props.width, props.height, props.emoji, containerReady, containerRef]);
    return React.createElement("div", { className: "emoji-box", ref: el => { containerRef.current = el; setContainerReady(true); }, style: { display: 'flex', alignItems: 'center', justifyContent: 'center', width: props.width, height: props.height, fontSize } }, props.emoji);
};
//# sourceMappingURL=EmojiBox.js.map