"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const React = require('react');
const EncryptionService_1 = require("@joplin/lib/services/e2ee/EncryptionService");
const theme_1 = require("@joplin/lib/theme");
const locale_1 = require("@joplin/lib/locale");
const time_1 = require("@joplin/lib/time");
const shim_1 = require("@joplin/lib/shim");
const dialogs_1 = require("../dialogs");
const utils_1 = require("@joplin/lib/components/EncryptionConfigScreen/utils");
const syncInfoUtils_1 = require("@joplin/lib/services/synchronizer/syncInfoUtils");
const utils_2 = require("@joplin/lib/services/e2ee/utils");
const Button_1 = require("../Button/Button");
const react_1 = require("react");
const react_redux_1 = require("react-redux");
const Setting_1 = require("@joplin/lib/models/Setting");
const CommandService_1 = require("@joplin/lib/services/CommandService");
const ToggleAdvancedSettingsButton_1 = require("../ConfigScreen/controls/ToggleAdvancedSettingsButton");
const MissingPasswordHelpLink_1 = require("../ConfigScreen/controls/MissingPasswordHelpLink");
const EncryptionConfigScreen = (props) => {
    const { inputPasswords, onInputPasswordChange } = (0, utils_1.useInputPasswords)(props.passwords);
    const theme = (0, react_1.useMemo)(() => {
        return (0, theme_1.themeStyle)(props.themeId);
    }, [props.themeId]);
    const stats = (0, utils_1.useStats)();
    const { passwordChecks, masterPasswordKeys, masterPasswordStatus } = (0, utils_1.usePasswordChecker)(props.masterKeys, props.activeMasterKeyId, props.masterPassword, props.passwords);
    const { showDisabledMasterKeys, toggleShowDisabledMasterKeys } = (0, utils_1.useToggleShowDisabledMasterKeys)();
    const needMasterPassword = (0, utils_1.useNeedMasterPassword)(passwordChecks, props.masterKeys);
    const onUpgradeMasterKey = (0, react_1.useCallback)(async (mk) => {
        const password = (0, utils_1.determineKeyPassword)(mk.id, masterPasswordKeys, props.masterPassword, props.passwords);
        const result = await (0, utils_1.upgradeMasterKey)(mk, password);
        await shim_1.default.showMessageBox(result, { type: shim_1.MessageBoxType.Info });
    }, [props.passwords, masterPasswordKeys, props.masterPassword]);
    const renderNeedUpgradeSection = () => {
        if (!shim_1.default.isElectron())
            return null;
        const needUpgradeMasterKeys = EncryptionService_1.default.instance().masterKeysThatNeedUpgrading(props.masterKeys);
        if (!needUpgradeMasterKeys.length)
            return null;
        const theme = (0, theme_1.themeStyle)(props.themeId);
        const rows = [];
        for (const mk of needUpgradeMasterKeys) {
            rows.push(React.createElement("tr", { key: mk.id },
                React.createElement("td", { style: theme.textStyle }, mk.id),
                React.createElement("td", null,
                    React.createElement("button", { onClick: () => onUpgradeMasterKey(mk), style: theme.buttonStyle }, "Upgrade"))));
        }
        return (React.createElement("div", null,
            React.createElement("h2", null, (0, locale_1._)('Keys that need upgrading')),
            React.createElement("p", null, (0, locale_1._)('The following keys use an out-dated encryption algorithm and it is recommended to upgrade them. The upgraded key will still be able to decrypt and encrypt your data as usual.')),
            React.createElement("table", null,
                React.createElement("tbody", null,
                    React.createElement("tr", null,
                        React.createElement("th", { style: theme.textStyle }, (0, locale_1._)('ID')),
                        React.createElement("th", { style: theme.textStyle }, (0, locale_1._)('Upgrade'))),
                    rows))));
    };
    const renderMasterKey = (mk) => {
        const theme = (0, theme_1.themeStyle)(props.themeId);
        const passwordStyle = {
            color: theme.color,
            backgroundColor: theme.backgroundColor,
            border: '1px solid',
            borderColor: theme.dividerColor,
        };
        const missingPasswordCellStyle = Object.assign(Object.assign({}, theme.textStyle), { border: '3px solid', borderColor: theme.colorError });
        const password = inputPasswords[mk.id] ? inputPasswords[mk.id] : '';
        const isActive = props.activeMasterKeyId === mk.id;
        const activeIcon = isActive ? '✔' : '';
        const passwordOk = passwordChecks[mk.id] === true ? '✔' : '❌';
        const renderPasswordInput = (masterKeyId) => {
            if (masterPasswordKeys[masterKeyId] || !passwordChecks['master']) {
                return (React.createElement("td", { style: Object.assign(Object.assign({}, theme.textStyle), { color: theme.colorFaded, fontStyle: 'italic' }) },
                    "(",
                    (0, locale_1._)('Master password'),
                    ")"));
            }
            else {
                return (React.createElement("td", { style: passwordChecks[masterKeyId] ? theme.textStyle : missingPasswordCellStyle },
                    React.createElement("input", { type: "password", placeholder: (0, locale_1._)('Enter password'), style: passwordStyle, value: password, onChange: event => onInputPasswordChange(mk, event.target.value) }),
                    ' ',
                    React.createElement("button", { style: theme.buttonStyle, onClick: () => (0, utils_1.onSavePasswordClick)(mk, Object.assign(Object.assign({}, props.passwords), inputPasswords)) }, (0, locale_1._)('Save'))));
            }
        };
        return (React.createElement("tr", { key: mk.id },
            React.createElement("td", { style: theme.textStyle }, activeIcon),
            React.createElement("td", { style: theme.textStyle },
                mk.id,
                React.createElement("br", null),
                (0, locale_1._)('Source: '),
                mk.source_application),
            React.createElement("td", { style: theme.textStyle },
                (0, locale_1._)('Created: '),
                time_1.default.formatMsToLocal(mk.created_time),
                React.createElement("br", null),
                (0, locale_1._)('Updated: '),
                time_1.default.formatMsToLocal(mk.updated_time)),
            renderPasswordInput(mk.id),
            React.createElement("td", { style: theme.textStyle }, passwordOk),
            React.createElement("td", { style: theme.textStyle },
                React.createElement("button", { style: theme.buttonStyle, onClick: () => (0, utils_1.onToggleEnabledClick)(mk) }, (0, syncInfoUtils_1.masterKeyEnabled)(mk) ? (0, locale_1._)('Disable') : (0, locale_1._)('Enable')))));
    };
    const renderMasterKeySection = (masterKeys, isEnabledMasterKeys) => {
        const theme = (0, theme_1.themeStyle)(props.themeId);
        const mkComps = [];
        const showTable = isEnabledMasterKeys || showDisabledMasterKeys;
        for (let i = 0; i < masterKeys.length; i++) {
            const mk = masterKeys[i];
            mkComps.push(renderMasterKey(mk));
        }
        const headerComp = isEnabledMasterKeys ? React.createElement("h2", null, (0, locale_1._)('Encryption keys')) : React.createElement("a", { onClick: () => toggleShowDisabledMasterKeys(), style: Object.assign(Object.assign({}, theme.urlStyle), { display: 'inline-block', marginBottom: 10 }), href: "#" }, showTable ? (0, locale_1._)('Hide disabled keys') : (0, locale_1._)('Show disabled keys'));
        // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
        const infoComp = null; // isEnabledMasterKeys ? <p>{'Note: Only one key is going to be used for encryption (the one marked as "active"). Any of the keys might be used for decryption, depending on how the notes or notebooks were originally encrypted.'}</p> : null;
        const tableComp = !showTable ? null : (React.createElement("table", null,
            React.createElement("tbody", null,
                React.createElement("tr", null,
                    React.createElement("th", { style: theme.textStyle }, (0, locale_1._)('Active')),
                    React.createElement("th", { style: theme.textStyle }, (0, locale_1._)('ID')),
                    React.createElement("th", { style: theme.textStyle }, (0, locale_1._)('Date')),
                    React.createElement("th", { style: theme.textStyle }, (0, locale_1._)('Password')),
                    React.createElement("th", { style: theme.textStyle }, (0, locale_1._)('Valid')),
                    React.createElement("th", { style: theme.textStyle }, (0, locale_1._)('Actions'))),
                mkComps)));
        if (mkComps.length) {
            return (React.createElement("div", { className: "section" },
                headerComp,
                tableComp,
                infoComp));
        }
        return null;
    };
    const onToggleButtonClick = (0, react_1.useCallback)(async () => {
        const isEnabled = (0, syncInfoUtils_1.getEncryptionEnabled)();
        const newEnabled = !isEnabled;
        const masterKey = (0, utils_2.getDefaultMasterKey)();
        const hasMasterPassword = !!props.masterPassword;
        let newPassword = '';
        if (isEnabled) {
            const answer = await dialogs_1.default.confirm((0, locale_1._)('Disabling encryption means *all* your notes and attachments are going to be re-synchronised and sent unencrypted to the sync target. Do you wish to continue?'));
            if (!answer)
                return;
        }
        else {
            const msg = (0, utils_1.enableEncryptionConfirmationMessages)(masterKey, hasMasterPassword);
            newPassword = await dialogs_1.default.prompt(msg.join('\n\n'), '', '', { type: 'password' });
        }
        if (hasMasterPassword && newEnabled) {
            if (!(await (0, utils_2.masterPasswordIsValid)(newPassword))) {
                await dialogs_1.default.alert('Invalid password. Please try again. If you have forgotten your password you will need to reset it.');
                return;
            }
        }
        try {
            await (0, utils_2.toggleAndSetupEncryption)(EncryptionService_1.default.instance(), newEnabled, masterKey, newPassword);
        }
        catch (error) {
            await dialogs_1.default.alert(error.message);
        }
    }, [props.masterPassword]);
    const renderEncryptionSection = () => {
        const decryptedItemsInfo = React.createElement("p", null, (0, utils_1.decryptedStatText)(stats));
        const toggleButton = (React.createElement(Button_1.default, { onClick: onToggleButtonClick, title: props.encryptionEnabled ? (0, locale_1._)('Disable encryption') : (0, locale_1._)('Enable encryption'), level: Button_1.ButtonLevel.Secondary }));
        const needUpgradeSection = renderNeedUpgradeSection();
        return (React.createElement("div", { className: "section" },
            React.createElement("div", { className: "encryption-section" },
                React.createElement("h2", { className: "-no-top-margin" }, (0, locale_1._)('End-to-end encryption')),
                React.createElement("p", null,
                    (0, locale_1._)('Encryption:'),
                    " ",
                    React.createElement("strong", null, props.encryptionEnabled ? (0, locale_1._)('Enabled') : (0, locale_1._)('Disabled'))),
                React.createElement("p", null,
                    (0, locale_1._)('Public-private key pair:'),
                    " ",
                    React.createElement("strong", null, props.ppk ? (0, locale_1._)('Generated') : (0, locale_1._)('Not generated'))),
                decryptedItemsInfo,
                toggleButton,
                needUpgradeSection)));
    };
    const renderMasterPasswordSection = () => {
        const onManageMasterPassword = async () => {
            void CommandService_1.default.instance().execute('openMasterPasswordDialog');
        };
        const buttonTitle = CommandService_1.default.instance().label('openMasterPasswordDialog');
        const needPasswordMessage = !needMasterPassword ? null : (React.createElement("p", { className: "needpassword" },
            (0, locale_1._)('Your password is needed to decrypt some of your data.'),
            React.createElement("br", null),
            (0, locale_1._)('Please click on "%s" to proceed, or set the passwords in the "%s" list below.', buttonTitle, (0, locale_1._)('Encryption keys')),
            React.createElement("br", null),
            React.createElement(MissingPasswordHelpLink_1.default, { theme: theme, text: (0, locale_1._)('%s: Missing password.', (0, locale_1._)('Help')) })));
        return (React.createElement("div", { className: "section" },
            React.createElement("div", { className: "manage-password-section" },
                React.createElement("h2", null, (0, locale_1._)('Master password')),
                React.createElement("p", { className: "status" },
                    React.createElement("span", null, (0, locale_1._)('Master password:')),
                    "\u00A0",
                    React.createElement("span", { className: "bold" }, (0, utils_2.getMasterPasswordStatusMessage)(masterPasswordStatus))),
                needPasswordMessage,
                React.createElement(Button_1.default, { className: "managebutton", level: needMasterPassword ? Button_1.ButtonLevel.Primary : Button_1.ButtonLevel.Secondary, onClick: onManageMasterPassword, title: buttonTitle }))));
    };
    const onClearMasterPassword = (0, react_1.useCallback)(() => {
        Setting_1.default.setValue('encryption.masterPassword', '');
    }, []);
    const renderDebugSection = () => {
        if (Setting_1.default.value('env') !== 'dev')
            return null;
        return (React.createElement("div", { style: { paddingBottom: '20px' } },
            React.createElement(Button_1.default, { level: Button_1.ButtonLevel.Secondary, onClick: onClearMasterPassword, title: "Clear master password" })));
    };
    const renderNonExistingMasterKeysSection = () => {
        let nonExistingMasterKeySection = null;
        const nonExistingMasterKeyIds = props.notLoadedMasterKeys.slice();
        for (let i = 0; i < props.masterKeys.length; i++) {
            const mk = props.masterKeys[i];
            const idx = nonExistingMasterKeyIds.indexOf(mk.id);
            if (idx >= 0)
                nonExistingMasterKeyIds.splice(idx, 1);
        }
        if (nonExistingMasterKeyIds.length) {
            const rows = [];
            for (let i = 0; i < nonExistingMasterKeyIds.length; i++) {
                const id = nonExistingMasterKeyIds[i];
                rows.push(React.createElement("tr", { key: id },
                    React.createElement("td", { style: theme.textStyle }, id)));
            }
            nonExistingMasterKeySection = (React.createElement("div", { className: "section" },
                React.createElement("h2", null, (0, locale_1._)('Missing keys')),
                React.createElement("p", null, (0, locale_1._)('The keys with these IDs are used to encrypt some of your items, however the application does not currently have access to them. It is likely they will eventually be downloaded via synchronisation.')),
                React.createElement("table", null,
                    React.createElement("tbody", null,
                        React.createElement("tr", null,
                            React.createElement("th", { style: theme.textStyle }, (0, locale_1._)('ID'))),
                        rows))));
        }
        return nonExistingMasterKeySection;
    };
    const renderReencryptData = () => {
        if (!shim_1.default.isElectron())
            return null;
        if (!props.encryptionEnabled)
            return null;
        const theme = (0, theme_1.themeStyle)(props.themeId);
        const buttonLabel = (0, locale_1._)('Re-encrypt data');
        const intro = props.shouldReencrypt ? (0, locale_1._)('The default encryption method has been changed to a more secure one and it is recommended that you apply it to your data.') : (0, locale_1._)('You may use the tool below to re-encrypt your data, for example if you know that some of your notes are encrypted with an obsolete encryption method.');
        let t = `${intro}\n\n${(0, locale_1._)('In order to do so, your entire data set will have to be encrypted and synchronised, so it is best to run it overnight.\n\nTo start, please follow these instructions:\n\n1. Synchronise all your devices.\n2. Click "%s".\n3. Let it run to completion. While it runs, avoid changing any note on your other devices, to avoid conflicts.\n4. Once sync is done on this device, sync all your other devices and let it run to completion.\n\nImportant: you only need to run this ONCE on one device.', buttonLabel)}`;
        t = t.replace(/\n\n/g, '</p><p>');
        t = t.replace(/\n/g, '<br>');
        t = `<p>${t}</p>`;
        return (React.createElement(React.Fragment, null,
            React.createElement("h2", null, (0, locale_1._)('Re-encryption')),
            React.createElement("p", { style: theme.textStyle, dangerouslySetInnerHTML: { __html: t } }),
            React.createElement("span", { style: { marginRight: 10 } },
                React.createElement("button", { onClick: () => void (0, utils_1.reencryptData)(), style: theme.buttonStyle }, buttonLabel)),
            !props.shouldReencrypt ? null : React.createElement("button", { onClick: () => (0, utils_1.dontReencryptData)(), style: theme.buttonStyle }, (0, locale_1._)('Ignore'))));
    };
    // If the user should re-encrypt, ensure that the section is visible initially.
    const [showAdvanced, setShowAdvanced] = (0, react_1.useState)(props.shouldReencrypt);
    const toggleAdvanced = (0, react_1.useCallback)(() => {
        setShowAdvanced(!showAdvanced);
    }, [showAdvanced]);
    const advancedSettingsId = (0, react_1.useId)();
    const renderAdvancedSection = () => {
        const reEncryptSection = renderReencryptData();
        if (!reEncryptSection)
            return null;
        return (React.createElement("div", null,
            React.createElement(ToggleAdvancedSettingsButton_1.default, { onClick: toggleAdvanced, advancedSettingsVisible: showAdvanced, "aria-controls": advancedSettingsId }),
            React.createElement("div", { id: advancedSettingsId }, showAdvanced ? reEncryptSection : null)));
    };
    return (React.createElement("div", { className: "config-screen-content" },
        renderDebugSection(),
        renderEncryptionSection(),
        renderMasterPasswordSection(),
        renderMasterKeySection(props.masterKeys.filter(mk => (0, syncInfoUtils_1.masterKeyEnabled)(mk)), true),
        renderMasterKeySection(props.masterKeys.filter(mk => !(0, syncInfoUtils_1.masterKeyEnabled)(mk)), false),
        renderNonExistingMasterKeysSection(),
        renderAdvancedSection()));
};
const mapStateToProps = (state) => {
    const syncInfo = new syncInfoUtils_1.SyncInfo(state.settings['syncInfoCache']);
    return {
        themeId: state.settings.theme,
        masterKeys: syncInfo.masterKeys,
        passwords: state.settings['encryption.passwordCache'],
        encryptionEnabled: syncInfo.e2ee,
        activeMasterKeyId: syncInfo.activeMasterKeyId,
        shouldReencrypt: state.settings['encryption.shouldReencrypt'] >= Setting_1.default.SHOULD_REENCRYPT_YES,
        notLoadedMasterKeys: state.notLoadedMasterKeys,
        masterPassword: state.settings['encryption.masterPassword'],
        ppk: syncInfo.ppk,
    };
};
exports.default = (0, react_redux_1.connect)(mapStateToProps)(EncryptionConfigScreen);
//# sourceMappingURL=EncryptionConfigScreen.js.map