"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const bridge_1 = require("../services/bridge");
const locale_1 = require("@joplin/lib/locale");
const theme_1 = require("@joplin/lib/theme");
const { createSelector } = require('reselect');
// eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
const themeSelector = (_state, props) => (0, theme_1.themeStyle)(props.themeId);
const styleSelector = createSelector(themeSelector, (theme) => {
    const output = {
        root: {
            width: 220,
            height: 60,
            borderRadius: 4,
            border: '1px solid',
            borderColor: theme.dividerColor,
            backgroundColor: theme.backgroundColor,
            paddingLeft: 14,
            paddingRight: 14,
            paddingTop: 8,
            paddingBottom: 8,
            boxSizing: 'border-box',
            display: 'flex',
            flexDirection: 'row',
            boxShadow: '0px 1px 1px rgba(0,0,0,0.3)',
        },
        logo: {
            width: 42,
            height: 42,
        },
        labelGroup: {
            display: 'flex',
            flexDirection: 'column',
            justifyContent: 'center',
            marginLeft: 14,
            fontFamily: theme.fontFamily,
            color: theme.color,
            fontSize: theme.fontSize,
        },
        locationLabel: {
            fontSize: theme.fontSize * 1.2,
            fontWeight: 'bold',
        },
    };
    return output;
});
function platformAssets(type) {
    if (type === 'firefox') {
        return {
            logoImage: `${(0, bridge_1.default)().buildDir()}/images/firefox-logo.svg`,
            locationLabel: (0, locale_1._)('Firefox Extension'),
        };
    }
    if (type === 'chrome') {
        return {
            logoImage: `${(0, bridge_1.default)().buildDir()}/images/chrome-logo.svg`,
            locationLabel: (0, locale_1._)('Chrome Web Store'),
        };
    }
    throw new Error(`Invalid type:${type}`);
}
function ExtensionBadge(props) {
    const style = styleSelector(null, props);
    const assets = platformAssets(props.type);
    const onClick = () => {
        void (0, bridge_1.default)().openExternal(props.url);
    };
    const rootStyle = props.style ? Object.assign(Object.assign({}, style.root), props.style) : style.root;
    return (React.createElement("a", { style: rootStyle, onClick: onClick, href: "#" },
        React.createElement("img", { style: style.logo, src: assets.logoImage }),
        React.createElement("div", { style: style.labelGroup },
            React.createElement("div", null, (0, locale_1._)('Get it now:')),
            React.createElement("div", { style: style.locationLabel }, assets.locationLabel))));
}
exports.default = ExtensionBadge;
//# sourceMappingURL=ExtensionBadge.js.map