"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = require("react");
const ButtonBar_1 = require("./ConfigScreen/ButtonBar");
const locale_1 = require("@joplin/lib/locale");
const electron_1 = require("electron");
const Button_1 = require("./Button/Button");
const bridge = require('@electron/remote').require('./bridge').default;
const uuid_1 = require("@joplin/lib/uuid");
const joplinCloudUtils_1 = require("@joplin/lib/services/joplinCloudUtils");
const Logger_1 = require("@joplin/utils/Logger");
const registry_1 = require("@joplin/lib/registry");
const logger = Logger_1.default.create('JoplinCloudLoginScreen');
const { connect } = require('react-redux');
const JoplinCloudScreenComponent = (props) => {
    const confirmUrl = (applicationAuthId) => `${props.joplinCloudWebsite}/applications/${applicationAuthId}/confirm`;
    const applicationAuthUrl = (applicationAuthId) => `${props.joplinCloudApi}/api/application_auth/${applicationAuthId}`;
    const [intervalIdentifier, setIntervalIdentifier] = (0, react_1.useState)(undefined);
    const [state, dispatch] = (0, react_1.useReducer)(joplinCloudUtils_1.reducer, joplinCloudUtils_1.defaultState);
    const applicationAuthId = (0, react_1.useMemo)(() => (0, uuid_1.uuidgen)(), []);
    const periodicallyCheckForCredentials = () => {
        if (intervalIdentifier)
            return;
        const interval = setInterval(async () => {
            try {
                const response = await (0, joplinCloudUtils_1.checkIfLoginWasSuccessful)(applicationAuthUrl(applicationAuthId));
                if (response && response.success) {
                    dispatch({ type: 'COMPLETED' });
                    clearInterval(interval);
                    void registry_1.reg.scheduleSync(0);
                }
            }
            catch (error) {
                logger.error(error);
                dispatch({ type: 'ERROR', payload: error.message });
                clearInterval(interval);
            }
        }, 2 * 1000);
        setIntervalIdentifier(interval);
    };
    const onButtonUsed = () => {
        if (state.next === 'LINK_USED') {
            dispatch({ type: 'LINK_USED' });
        }
        periodicallyCheckForCredentials();
    };
    const onAuthorizeClicked = async () => {
        const url = await (0, joplinCloudUtils_1.generateApplicationConfirmUrl)(confirmUrl(applicationAuthId));
        bridge().openExternal(url);
        onButtonUsed();
    };
    const onCopyToClipboardClicked = async () => {
        const url = await (0, joplinCloudUtils_1.generateApplicationConfirmUrl)(confirmUrl(applicationAuthId));
        electron_1.clipboard.writeText(url);
        onButtonUsed();
    };
    (0, react_1.useEffect)(() => {
        return () => {
            clearInterval(intervalIdentifier);
        };
    }, [intervalIdentifier]);
    return (React.createElement("div", { className: "login-page" },
        React.createElement("div", { className: "page-container" },
            state.active !== 'COMPLETED' ? (React.createElement(react_1.Fragment, null,
                React.createElement("p", { className: "text" }, (0, locale_1._)('To allow Joplin to synchronise with Joplin Cloud, please login using this URL:')),
                React.createElement("div", { className: "buttons-container" },
                    React.createElement(Button_1.default, { onClick: onAuthorizeClicked, title: (0, locale_1._)('Authorise'), iconName: 'fa fa-external-link-alt', level: Button_1.ButtonLevel.Primary }),
                    React.createElement(Button_1.default, { onClick: onCopyToClipboardClicked, title: (0, locale_1._)('Copy link to website'), iconName: 'fa fa-clone', level: Button_1.ButtonLevel.Secondary })))) : null,
            React.createElement("p", { className: state.className },
                state.message(),
                state.active === 'ERROR' ? (React.createElement("span", { className: state.className }, state.errorMessage)) : null),
            state.active === 'LINK_USED' ? React.createElement("div", { id: "loading-animation" }) : null),
        React.createElement(ButtonBar_1.default, { onCancelClick: () => props.dispatch({ type: 'NAV_BACK' }) })));
};
const mapStateToProps = (state) => {
    return {
        joplinCloudWebsite: state.settings['sync.10.website'],
        joplinCloudApi: state.settings['sync.10.path'],
    };
};
exports.default = connect(mapStateToProps)(JoplinCloudScreenComponent);
//# sourceMappingURL=JoplinCloudLoginScreen.js.map