"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ShortcutRecorder = void 0;
const React = require("react");
const react_1 = require("react");
const KeymapService_1 = require("@joplin/lib/services/KeymapService");
const styles_1 = require("./styles");
const locale_1 = require("@joplin/lib/locale");
const keymapService = KeymapService_1.default.instance();
const ShortcutRecorder = ({ onSave, onReset, onCancel, onError, initialAccelerator, commandName, themeId }) => {
    const styles = (0, styles_1.default)(themeId);
    const [accelerator, setAccelerator] = (0, react_1.useState)(initialAccelerator);
    const [saveAllowed, setSaveAllowed] = (0, react_1.useState)(true);
    (0, react_1.useEffect)(() => {
        try {
            // Only perform validations if there's an accelerator provided
            // Otherwise performing a save means that it's going to be disabled
            if (accelerator) {
                keymapService.validateAccelerator(accelerator);
                keymapService.validateKeymap({ accelerator, command: commandName });
            }
            // Discard previous errors
            onError({ recorderError: null });
            setSaveAllowed(true);
        }
        catch (recorderError) {
            onError({ recorderError });
            setSaveAllowed(false);
        }
        // eslint-disable-next-line @seiyab/react-hooks/exhaustive-deps -- Old code before rule was applied
    }, [accelerator]);
    const handleKeyDown = (event) => {
        // Shift-tab and tab are needed for navigating the shortcuts screen with the keyboard. Do not
        // .preventDefault.
        if (event.code === 'Tab' && !event.metaKey && !event.altKey && !event.ctrlKey) {
            return;
        }
        event.preventDefault();
        const newAccelerator = keymapService.domToElectronAccelerator(event);
        switch (newAccelerator) {
            case 'Enter':
                if (saveAllowed)
                    return onSave({ commandName, accelerator });
                break;
            case 'Escape':
                return onCancel({ commandName });
            case 'Backspace':
            case 'Delete':
                return setAccelerator('');
            default:
                setAccelerator(newAccelerator);
        }
    };
    const hintText = (0, locale_1._)('Press the shortcut and then press ENTER. Or, press BACKSPACE to clear the shortcut.');
    const placeholderText = (0, locale_1._)('Press the shortcut');
    return (React.createElement("div", { className: 'shortcut-recorder', style: styles.recorderContainer },
        React.createElement("input", { className: 'shortcut text-input', value: accelerator, "aria-label": accelerator ? accelerator : placeholderText, placeholder: placeholderText, title: hintText, "aria-description": hintText, "aria-invalid": accelerator && !saveAllowed, "aria-live": 'polite', onKeyDown: handleKeyDown, readOnly: true, autoFocus: true }),
        React.createElement("button", { style: styles.inlineButton, disabled: !saveAllowed, onClick: () => onSave({ commandName, accelerator }) }, (0, locale_1._)('Save')),
        React.createElement("button", { style: styles.inlineButton, onClick: () => onReset({ commandName }) }, (0, locale_1._)('Restore')),
        React.createElement("button", { style: styles.inlineButton, onClick: () => onCancel({ commandName }) }, (0, locale_1._)('Cancel'))));
};
exports.ShortcutRecorder = ShortcutRecorder;
//# sourceMappingURL=ShortcutRecorder.js.map