"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const ResizableLayout_1 = require("./ResizableLayout/ResizableLayout");
const findItemByKey_1 = require("./ResizableLayout/utils/findItemByKey");
const movements_1 = require("./ResizableLayout/utils/movements");
const CommandService_1 = require("@joplin/lib/services/CommandService");
const reducer_1 = require("@joplin/lib/services/plugins/reducer");
const Sidebar_1 = require("./Sidebar/Sidebar");
const UserWebview_1 = require("../services/plugins/UserWebview");
const UserWebviewDialog_1 = require("../services/plugins/UserWebviewDialog");
const WebviewController_1 = require("@joplin/lib/services/plugins/WebviewController");
const reducer_2 = require("@joplin/lib/reducer");
const locale_1 = require("@joplin/lib/locale");
const NoteListWrapper_1 = require("./NoteListWrapper/NoteListWrapper");
const persist_1 = require("./ResizableLayout/utils/persist");
const Setting_1 = require("@joplin/lib/models/Setting");
const shouldShowMissingPasswordWarning_1 = require("@joplin/lib/components/shared/config/shouldShowMissingPasswordWarning");
const immer_1 = require("immer");
const shim_1 = require("@joplin/lib/shim");
const bridge_1 = require("../services/bridge");
const styled_components_1 = require("styled-components");
const theme_1 = require("@joplin/lib/theme");
const validateLayout_1 = require("./ResizableLayout/utils/validateLayout");
const iterateItems_1 = require("./ResizableLayout/utils/iterateItems");
const removeItem_1 = require("./ResizableLayout/utils/removeItem");
const EncryptionService_1 = require("@joplin/lib/services/e2ee/EncryptionService");
const removeKeylessItems_1 = require("./ResizableLayout/utils/removeKeylessItems");
const syncInfoUtils_1 = require("@joplin/lib/services/synchronizer/syncInfoUtils");
const callbackUrlUtils_1 = require("@joplin/lib/callbackUrlUtils");
const utils_1 = require("@joplin/lib/services/e2ee/utils");
const invitationRespond_1 = require("@joplin/lib/services/share/invitationRespond");
const restart_1 = require("../services/restart");
const react_redux_1 = require("react-redux");
const validateColumns_1 = require("./NoteListHeader/utils/validateColumns");
const TrashNotification_1 = require("./TrashNotification/TrashNotification");
const UpdateNotification_1 = require("./UpdateNotification/UpdateNotification");
const NoteEditor_1 = require("./NoteEditor/NoteEditor");
const PluginNotification_1 = require("./PluginNotification/PluginNotification");
const ipcRenderer = require('electron').ipcRenderer;
const StyledUserWebviewDialogContainer = styled_components_1.default.div `
	display: flex;
	position: absolute;
	top: 0;
	left: 0;
	width: 100%;
	height: 100%;
	z-index: 1000;
	box-sizing: border-box;
`;
const defaultLayout = {
    key: 'root',
    children: [
        { key: 'sideBar', width: 250 },
        { key: 'noteList', width: 250 },
        { key: 'editor' },
    ],
};
class MainScreenComponent extends React.Component {
    constructor(props) {
        super(props);
        this.state = {
            promptOptions: null,
            notePropertiesDialogOptions: {},
            noteContentPropertiesDialogOptions: {},
            shareNoteDialogOptions: {},
            shareFolderDialogOptions: {
                visible: false,
                folderId: '',
            },
        };
        this.updateMainLayout(this.buildLayout(props.plugins));
        this.setupAppCloseHandling();
        this.resizableLayout_resize = this.resizableLayout_resize.bind(this);
        this.resizableLayout_renderItem = this.resizableLayout_renderItem.bind(this);
        this.resizableLayout_moveButtonClick = this.resizableLayout_moveButtonClick.bind(this);
        this.window_resize = this.window_resize.bind(this);
        this.rowHeight = this.rowHeight.bind(this);
        this.layoutModeListenerKeyDown = this.layoutModeListenerKeyDown.bind(this);
        window.addEventListener('resize', this.window_resize);
        // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
        ipcRenderer.on('asynchronous-message', (_event, message, args) => {
            if (message === 'openCallbackUrl') {
                this.openCallbackUrl(args.url);
            }
        });
        const initialCallbackUrl = (0, bridge_1.default)().electronApp().initialCallbackUrl();
        if (initialCallbackUrl) {
            this.openCallbackUrl(initialCallbackUrl);
        }
    }
    openCallbackUrl(url) {
        if (!(0, callbackUrlUtils_1.isCallbackUrl)(url))
            throw new Error(`Invalid callback URL: ${url}`);
        const { command, params } = (0, callbackUrlUtils_1.parseCallbackUrl)(url);
        void CommandService_1.default.instance().execute(command.toString(), params.id);
    }
    updateLayoutPluginViews(layout, plugins) {
        const infos = reducer_1.utils.viewInfosByType(plugins, 'webview');
        let newLayout = (0, immer_1.default)(layout, (draftLayout) => {
            for (const info of infos) {
                if (info.view.containerType !== WebviewController_1.ContainerType.Panel)
                    continue;
                const viewId = info.view.id;
                const existingItem = (0, findItemByKey_1.default)(draftLayout, viewId);
                if (!existingItem) {
                    draftLayout.children.push({
                        key: viewId,
                        context: {
                            pluginId: info.plugin.id,
                        },
                    });
                }
            }
        });
        // Remove layout items that belong to plugins that are no longer
        // active.
        const pluginIds = Object.keys(plugins);
        const itemsToRemove = [];
        (0, iterateItems_1.default)(newLayout, (_itemIndex, item, _parent) => {
            if (item.context && item.context.pluginId && !pluginIds.includes(item.context.pluginId)) {
                itemsToRemove.push(item.key);
            }
            return true;
        });
        for (const itemKey of itemsToRemove) {
            newLayout = (0, removeItem_1.default)(newLayout, itemKey);
        }
        return newLayout !== layout ? (0, validateLayout_1.default)(newLayout) : layout;
    }
    showShareInvitationNotification(props) {
        if (props.processingShareInvitationResponse)
            return false;
        return !!props.shareInvitations.find(i => i.status === 0);
    }
    buildLayout(plugins) {
        const rootLayoutSize = this.rootLayoutSize();
        const userLayout = Setting_1.default.value('ui.layout');
        let output = null;
        try {
            output = (0, persist_1.loadLayout)(Object.keys(userLayout).length ? userLayout : null, defaultLayout, rootLayoutSize);
            // For unclear reasons, layout items sometimes end up without a key.
            // In that case, we can't do anything with them, so remove them
            // here. It could be due to the deprecated plugin API, which allowed
            // creating panel without a key, although in this case it should
            // have been set automatically.
            // https://github.com/laurent22/joplin/issues/4926
            output = (0, removeKeylessItems_1.default)(output);
            if (!(0, findItemByKey_1.default)(output, 'sideBar') || !(0, findItemByKey_1.default)(output, 'noteList') || !(0, findItemByKey_1.default)(output, 'editor')) {
                throw new Error('"sideBar", "noteList" and "editor" must be present in the layout');
            }
        }
        catch (error) {
            console.warn('Could not load layout - restoring default layout:', error);
            console.warn('Layout was:', userLayout);
            output = (0, persist_1.loadLayout)(null, defaultLayout, rootLayoutSize);
        }
        return this.updateLayoutPluginViews(output, plugins);
    }
    window_resize() {
        this.updateRootLayoutSize();
    }
    setupAppCloseHandling() {
        this.waitForNotesSavedIID_ = null;
        // This event is dispatched from the main process when the app is about
        // to close. The renderer process must respond with the "appCloseReply"
        // and tell the main process whether the app can really be closed or not.
        // For example, it cannot be closed right away if a note is being saved.
        // If a note is being saved, we wait till it is saved and then call
        // "appCloseReply" again.
        ipcRenderer.on('appClose', async () => {
            if (this.waitForNotesSavedIID_)
                shim_1.default.clearInterval(this.waitForNotesSavedIID_);
            this.waitForNotesSavedIID_ = null;
            const sendCanClose = async (canClose) => {
                if (canClose) {
                    Setting_1.default.setValue('wasClosedSuccessfully', true);
                    await Setting_1.default.saveAll();
                }
                ipcRenderer.send('asynchronous-message', 'appCloseReply', { canClose });
            };
            await sendCanClose(!this.props.hasNotesBeingSaved);
            if (this.props.hasNotesBeingSaved) {
                this.waitForNotesSavedIID_ = shim_1.default.setInterval(() => {
                    if (!this.props.hasNotesBeingSaved) {
                        shim_1.default.clearInterval(this.waitForNotesSavedIID_);
                        this.waitForNotesSavedIID_ = null;
                        void sendCanClose(true);
                    }
                }, 50);
            }
        });
    }
    updateMainLayout(layout) {
        this.props.dispatch({
            type: 'MAIN_LAYOUT_SET',
            value: layout,
        });
    }
    updateRootLayoutSize() {
        // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
        this.updateMainLayout((0, immer_1.default)(this.props.mainLayout, (draft) => {
            const s = this.rootLayoutSize();
            draft.width = s.width;
            draft.height = s.height;
        }));
    }
    componentDidUpdate(prevProps, prevState) {
        if (prevProps.style.width !== this.props.style.width ||
            prevProps.style.height !== this.props.style.height ||
            this.messageBoxVisible(prevProps) !== this.messageBoxVisible(this.props)) {
            this.updateRootLayoutSize();
        }
        if (prevProps.plugins !== this.props.plugins) {
            this.updateMainLayout(this.updateLayoutPluginViews(this.props.mainLayout, this.props.plugins));
            // this.setState({ layout: this.buildLayout(this.props.plugins) });
        }
        if (this.props.mainLayout !== prevProps.mainLayout) {
            const toSave = (0, persist_1.saveLayout)(this.props.mainLayout);
            Setting_1.default.setValue('ui.layout', toSave);
        }
        if (prevState.promptOptions !== this.state.promptOptions) {
            this.props.dispatch({
                type: !prevState.promptOptions ? 'VISIBLE_DIALOGS_ADD' : 'VISIBLE_DIALOGS_REMOVE',
                name: 'promptDialog',
            });
        }
        if (this.props.isResettingLayout) {
            Setting_1.default.setValue('ui.layout', null);
            this.updateMainLayout(this.buildLayout(this.props.plugins));
            this.props.dispatch({
                type: 'RESET_LAYOUT',
                value: false,
            });
        }
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    layoutModeListenerKeyDown(event) {
        if (event.key !== 'Escape')
            return;
        if (!this.props.layoutMoveMode)
            return;
        void CommandService_1.default.instance().execute('toggleLayoutMoveMode');
    }
    componentDidMount() {
        window.addEventListener('keydown', this.layoutModeListenerKeyDown);
    }
    componentWillUnmount() {
        window.removeEventListener('resize', this.window_resize);
        window.removeEventListener('keydown', this.layoutModeListenerKeyDown);
    }
    rootLayoutSize() {
        return {
            width: window.innerWidth,
            height: this.rowHeight(),
        };
    }
    rowHeight() {
        if (!this.props)
            return 0;
        return this.props.style.height - (this.messageBoxVisible() ? this.messageBoxHeight() : 0);
    }
    messageBoxHeight() {
        return 50;
    }
    styles(themeId, width, height, messageBoxVisible) {
        const styleKey = [themeId, width, height, messageBoxVisible].join('_');
        if (styleKey === this.styleKey_)
            return this.styles_;
        const theme = (0, theme_1.themeStyle)(themeId);
        this.styleKey_ = styleKey;
        this.styles_ = {};
        this.styles_.header = {
            width: width,
        };
        this.styles_.messageBox = {
            width: width,
            height: this.messageBoxHeight(),
            display: 'flex',
            alignItems: 'center',
            paddingLeft: 10,
            backgroundColor: theme.warningBackgroundColor,
        };
        const rowHeight = height - (messageBoxVisible ? this.styles_.messageBox.height : 0);
        this.styles_.rowHeight = rowHeight;
        this.styles_.resizableLayout = {
            height: rowHeight,
        };
        this.styles_.prompt = {
            width: width,
            height: height,
        };
        return this.styles_;
    }
    // eslint-disable-next-line @typescript-eslint/ban-types -- Old code before rule was applied
    renderNotificationMessage(message, callForAction = null, callForActionHandler = null, callForAction2 = null, callForActionHandler2 = null) {
        const theme = (0, theme_1.themeStyle)(this.props.themeId);
        // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
        const urlStyle = { color: theme.colorWarnUrl, textDecoration: 'underline' };
        if (!callForAction)
            return React.createElement("span", null, message);
        const cfa = (React.createElement("a", { href: "#", style: urlStyle, onClick: () => callForActionHandler() }, callForAction));
        const cfa2 = !callForAction2 ? null : (React.createElement("a", { href: "#", style: urlStyle, onClick: () => callForActionHandler2() }, callForAction2));
        return (React.createElement("span", null,
            message,
            callForAction ? ' ' : '',
            cfa,
            callForAction2 ? ' / ' : '',
            cfa2));
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    renderNotification(theme, styles) {
        if (!this.messageBoxVisible())
            return null;
        const onViewStatusScreen = () => {
            this.props.dispatch({
                type: 'NAV_GO',
                routeName: 'Status',
            });
        };
        const onViewEncryptionConfigScreen = () => {
            this.props.dispatch({
                type: 'NAV_GO',
                routeName: 'Config',
                props: {
                    defaultSection: 'encryption',
                },
            });
        };
        const onViewJoplinCloudLoginScreen = () => {
            this.props.dispatch({
                type: 'NAV_GO',
                routeName: 'JoplinCloudLogin',
            });
        };
        const onViewSyncSettingsScreen = () => {
            this.props.dispatch({
                type: 'NAV_GO',
                routeName: 'Config',
                props: {
                    defaultSection: 'sync',
                },
            });
        };
        const onRestartAndUpgrade = async () => {
            Setting_1.default.setValue('sync.upgradeState', Setting_1.default.SYNC_UPGRADE_STATE_MUST_DO);
            await Setting_1.default.saveAll();
            await (0, restart_1.default)();
        };
        const onDisableSafeModeAndRestart = async () => {
            Setting_1.default.setValue('isSafeMode', false);
            await Setting_1.default.saveAll();
            await (0, restart_1.default)();
        };
        const onInvitationRespond = async (shareUserId, folderId, masterKey, accept) => {
            await (0, invitationRespond_1.default)(shareUserId, folderId, masterKey, accept);
        };
        let msg = null;
        // When adding something here, don't forget to update the condition in
        // this.messageBoxVisible()
        if (this.props.isSafeMode) {
            msg = this.renderNotificationMessage((0, locale_1._)('Safe mode is currently active. Note rendering and all plugins are temporarily disabled.'), (0, locale_1._)('Disable safe mode and restart'), onDisableSafeModeAndRestart);
        }
        else if (this.props.hasMissingSyncCredentials) {
            msg = this.renderNotificationMessage((0, locale_1._)('The synchronisation password is missing.'), (0, locale_1._)('Set the password'), onViewSyncSettingsScreen);
        }
        else if (this.props.shouldUpgradeSyncTarget) {
            msg = this.renderNotificationMessage((0, locale_1._)('The sync target needs to be upgraded before Joplin can sync. The operation may take a few minutes to complete and the app needs to be restarted. To proceed please click on the link.'), (0, locale_1._)('Restart and upgrade'), onRestartAndUpgrade);
        }
        else if (this.props.hasDisabledEncryptionItems) {
            msg = this.renderNotificationMessage((0, locale_1._)('Some items cannot be decrypted.'), (0, locale_1._)('View them now'), onViewStatusScreen);
        }
        else if (this.props.showNeedUpgradingMasterKeyMessage) {
            msg = this.renderNotificationMessage((0, locale_1._)('One of your master keys use an obsolete encryption method.'), (0, locale_1._)('View them now'), onViewEncryptionConfigScreen);
        }
        else if (this.props.showShouldReencryptMessage) {
            msg = this.renderNotificationMessage((0, locale_1._)('The default encryption method has been changed, you should re-encrypt your data.'), (0, locale_1._)('More info'), onViewEncryptionConfigScreen);
        }
        else if (this.showShareInvitationNotification(this.props)) {
            const invitation = this.props.shareInvitations.find(inv => inv.status === 0);
            const sharer = invitation.share.user;
            msg = this.renderNotificationMessage((0, locale_1._)('%s (%s) would like to share a notebook with you.', sharer.full_name, sharer.email), (0, locale_1._)('Accept'), () => onInvitationRespond(invitation.id, invitation.share.folder_id, invitation.master_key, true), (0, locale_1._)('Reject'), () => onInvitationRespond(invitation.id, invitation.share.folder_id, invitation.master_key, false));
        }
        else if (this.props.hasDisabledSyncItems) {
            msg = this.renderNotificationMessage((0, locale_1._)('Some items cannot be synchronised.'), (0, locale_1._)('View them now'), onViewStatusScreen);
        }
        else if (this.props.showMissingMasterKeyMessage) {
            msg = this.renderNotificationMessage((0, locale_1._)('One or more master keys need a password.'), (0, locale_1._)('Set the password'), onViewEncryptionConfigScreen);
        }
        else if (this.props.mustUpgradeAppMessage) {
            msg = this.renderNotificationMessage(this.props.mustUpgradeAppMessage);
        }
        else if (this.props.showInvalidJoplinCloudCredential) {
            msg = this.renderNotificationMessage((0, locale_1._)('Your Joplin Cloud credentials are invalid, please login.'), (0, locale_1._)('Login to Joplin Cloud.'), onViewJoplinCloudLoginScreen);
        }
        return (React.createElement("div", { style: styles.messageBox },
            React.createElement("span", { style: theme.textStyle }, msg)));
    }
    messageBoxVisible(props = null) {
        if (!props)
            props = this.props;
        return props.hasDisabledSyncItems ||
            props.showMissingMasterKeyMessage ||
            props.hasMissingSyncCredentials ||
            props.showNeedUpgradingMasterKeyMessage ||
            props.showShouldReencryptMessage ||
            props.hasDisabledEncryptionItems ||
            this.props.shouldUpgradeSyncTarget ||
            props.isSafeMode ||
            this.showShareInvitationNotification(props) ||
            this.props.needApiAuth ||
            !!this.props.mustUpgradeAppMessage ||
            props.showInvalidJoplinCloudCredential;
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    resizableLayout_resize(event) {
        this.updateMainLayout(event.layout);
    }
    resizableLayout_moveButtonClick(event) {
        const newLayout = (0, movements_1.move)(this.props.mainLayout, event.itemKey, event.direction);
        this.updateMainLayout(newLayout);
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    resizableLayout_renderItem(key, event) {
        var _a, _b;
        // Key should never be undefined but somehow it can happen, also not
        // clear how. For now in this case render nothing so that the app
        // doesn't crash.
        // https://discourse.joplinapp.org/t/rearranging-the-pannels-crushed-the-app-and-generated-fatal-error/14373?u=laurent
        if (!key) {
            console.error('resizableLayout_renderItem: Trying to render an item using an empty key. Full layout is:', this.props.mainLayout);
            return null;
        }
        const eventEmitter = event.eventEmitter;
        // const viewsToRemove:string[] = [];
        // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
        const components = {
            sideBar: () => {
                return React.createElement(Sidebar_1.default, { key: key });
            },
            noteList: () => {
                return React.createElement(NoteListWrapper_1.default, { key: key, resizableLayoutEventEmitter: eventEmitter, visible: event.visible, size: event.size, themeId: this.props.themeId, listRendererId: this.props.listRendererId, startupPluginsLoaded: this.props.startupPluginsLoaded, notesSortOrderField: this.props.notesSortOrderField, notesSortOrderReverse: this.props.notesSortOrderReverse, columns: this.props.notesColumns, selectedFolderId: this.props.selectedFolderId });
            },
            editor: () => {
                return React.createElement("div", { className: 'note-editor-wrapper', role: 'main', "aria-label": (0, locale_1._)('Note') },
                    React.createElement(NoteEditor_1.default, { windowId: reducer_2.defaultWindowId, key: key, startupPluginsLoaded: this.props.startupPluginsLoaded }));
            },
        };
        if (components[key])
            return components[key]();
        const viewsToRemove = [];
        if (key.indexOf('plugin-view') === 0) {
            const viewInfo = reducer_1.utils.viewInfoByViewId(this.props.plugins, event.item.key);
            if (!viewInfo) {
                // Once all startup plugins have loaded, we know that all the
                // views are ready so we can remove the orphans ones.
                //
                // Before they are loaded, there might be views that don't match
                // any plugins, but that's only because it hasn't loaded yet.
                if (this.props.startupPluginsLoaded) {
                    console.warn(`Could not find plugin associated with view: ${event.item.key}`);
                    viewsToRemove.push(event.item.key);
                }
            }
            else {
                const { view, plugin } = viewInfo;
                const html = (_b = (_a = this.props.pluginHtmlContents[plugin.id]) === null || _a === void 0 ? void 0 : _a[view.id]) !== null && _b !== void 0 ? _b : '';
                return React.createElement(UserWebview_1.default, { key: view.id, viewId: view.id, themeId: this.props.themeId, html: html, scripts: view.scripts, pluginId: plugin.id, borderBottom: true, fitToContent: false });
            }
        }
        else {
            throw new Error(`Invalid layout component: ${key}`);
        }
        if (viewsToRemove.length) {
            window.requestAnimationFrame(() => {
                let newLayout = this.props.mainLayout;
                for (const itemKey of viewsToRemove) {
                    newLayout = (0, removeItem_1.default)(newLayout, itemKey);
                }
                if (newLayout !== this.props.mainLayout) {
                    console.warn('Removed invalid views:', viewsToRemove);
                    this.updateMainLayout(newLayout);
                }
            });
        }
    }
    renderPluginDialogs() {
        var _a, _b;
        const output = [];
        const infos = reducer_1.utils.viewInfosByType(this.props.plugins, 'webview');
        for (const info of infos) {
            const { plugin, view } = info;
            if (view.containerType !== WebviewController_1.ContainerType.Dialog)
                continue;
            if (!view.opened)
                continue;
            const html = (_b = (_a = this.props.pluginHtmlContents[plugin.id]) === null || _a === void 0 ? void 0 : _a[view.id]) !== null && _b !== void 0 ? _b : '';
            output.push(React.createElement(UserWebviewDialog_1.default, { key: view.id, viewId: view.id, themeId: this.props.themeId, html: html, scripts: view.scripts, pluginId: plugin.id, buttons: view.buttons, fitToContent: view.fitToContent }));
        }
        if (!output.length)
            return null;
        return (React.createElement(StyledUserWebviewDialogContainer, null, output));
    }
    render() {
        const theme = (0, theme_1.themeStyle)(this.props.themeId);
        const style = Object.assign({ color: theme.color, backgroundColor: theme.backgroundColor }, this.props.style);
        const styles = this.styles(this.props.themeId, style.width, style.height, this.messageBoxVisible());
        const messageComp = this.renderNotification(theme, styles);
        const layoutComp = this.props.mainLayout ? (React.createElement(ResizableLayout_1.default, { height: styles.rowHeight, layout: this.props.mainLayout, onResize: this.resizableLayout_resize, onMoveButtonClick: this.resizableLayout_moveButtonClick, renderItem: this.resizableLayout_renderItem, moveMode: this.props.layoutMoveMode, moveModeMessage: (0, locale_1._)('Use the arrows to move the layout items. Press "Escape" to exit.') })) : null;
        return (React.createElement("div", { style: style },
            React.createElement(TrashNotification_1.default, { lastDeletion: this.props.lastDeletion, lastDeletionNotificationTime: this.props.lastDeletionNotificationTime, themeId: this.props.themeId, 
                // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
                dispatch: this.props.dispatch }),
            React.createElement(UpdateNotification_1.default, { themeId: this.props.themeId }),
            React.createElement(PluginNotification_1.default, { themeId: this.props.themeId, toast: this.props.toast }),
            messageComp,
            layoutComp));
    }
}
const mapStateToProps = (state) => {
    const syncInfo = (0, syncInfoUtils_1.localSyncInfoFromState)(state);
    const showNeedUpgradingEnabledMasterKeyMessage = !!EncryptionService_1.default.instance().masterKeysThatNeedUpgrading(syncInfo.masterKeys.filter((k) => !!k.enabled)).length;
    const windowState = reducer_2.stateUtils.windowStateById(state, reducer_2.defaultWindowId);
    return {
        themeId: state.settings.theme,
        hasDisabledSyncItems: state.hasDisabledSyncItems,
        hasDisabledEncryptionItems: state.hasDisabledEncryptionItems,
        showMissingMasterKeyMessage: (0, utils_1.showMissingMasterKeyMessage)(syncInfo, state.notLoadedMasterKeys),
        showNeedUpgradingMasterKeyMessage: showNeedUpgradingEnabledMasterKeyMessage,
        showShouldReencryptMessage: state.settings['encryption.shouldReencrypt'] >= Setting_1.default.SHOULD_REENCRYPT_YES,
        shouldUpgradeSyncTarget: state.settings['sync.upgradeState'] === Setting_1.default.SYNC_UPGRADE_STATE_SHOULD_DO,
        hasMissingSyncCredentials: (0, shouldShowMissingPasswordWarning_1.default)(state.settings['sync.target'], state.settings),
        plugins: state.pluginService.plugins,
        pluginHtmlContents: state.pluginService.pluginHtmlContents,
        hasNotesBeingSaved: reducer_2.stateUtils.hasNotesBeingSaved(state),
        layoutMoveMode: state.layoutMoveMode,
        mainLayout: state.mainLayout,
        startupPluginsLoaded: state.startupPluginsLoaded,
        shareInvitations: state.shareService.shareInvitations,
        processingShareInvitationResponse: state.shareService.processingShareInvitationResponse,
        isSafeMode: state.settings.isSafeMode,
        enableLegacyMarkdownEditor: state.settings['editor.legacyMarkdown'],
        needApiAuth: state.needApiAuth,
        isResettingLayout: state.isResettingLayout,
        listRendererId: state.settings['notes.listRendererId'],
        lastDeletion: state.lastDeletion,
        lastDeletionNotificationTime: state.lastDeletionNotificationTime,
        selectedFolderId: windowState.selectedFolderId,
        mustUpgradeAppMessage: state.mustUpgradeAppMessage,
        notesSortOrderField: state.settings['notes.sortOrder.field'],
        notesSortOrderReverse: state.settings['notes.sortOrder.reverse'],
        notesColumns: (0, validateColumns_1.default)(state.settings['notes.columns']),
        showInvalidJoplinCloudCredential: state.settings['sync.target'] === 10 && state.mustAuthenticate,
        toast: state.toast,
    };
};
exports.default = (0, react_redux_1.connect)(mapStateToProps)(MainScreenComponent);
//# sourceMappingURL=MainScreen.js.map