"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.WindowMode = exports.WindowIdContext = void 0;
const reducer_1 = require("@joplin/lib/reducer");
const shim_1 = require("@joplin/lib/shim");
const React = require("react");
const react_1 = require("react");
const react_dom_1 = require("react-dom");
// This component uses react-dom's Portals to render its children in a different HTML
// document. As children are rendered in a different Window/Document, they should avoid
// referencing the `window` and `document` globals. Instead, HTMLElement.ownerDocument
// and refs can be used to access the child component's DOM.
exports.WindowIdContext = (0, react_1.createContext)(reducer_1.defaultWindowId);
var WindowMode;
(function (WindowMode) {
    WindowMode[WindowMode["Iframe"] = 0] = "Iframe";
    WindowMode[WindowMode["NewWindow"] = 1] = "NewWindow";
})(WindowMode || (exports.WindowMode = WindowMode = {}));
const useDocument = (mode, iframeElement, onClose) => {
    const [doc, setDoc] = (0, react_1.useState)(null);
    const onCloseRef = (0, react_1.useRef)(onClose);
    onCloseRef.current = onClose;
    (0, react_1.useEffect)(() => {
        var _a;
        let openedWindow = null;
        const unmounted = false;
        if (iframeElement) {
            setDoc((_a = iframeElement === null || iframeElement === void 0 ? void 0 : iframeElement.contentWindow) === null || _a === void 0 ? void 0 : _a.document);
        }
        else if (mode === WindowMode.NewWindow) {
            openedWindow = window.open('about:blank');
            setDoc(openedWindow.document);
            // .onbeforeunload and .onclose events don't seem to fire when closed by a user -- rely on polling
            // instead:
            void (async () => {
                var _a;
                while (!unmounted) {
                    await new Promise(resolve => {
                        shim_1.default.setTimeout(() => resolve(), 2000);
                    });
                    if (openedWindow === null || openedWindow === void 0 ? void 0 : openedWindow.closed) {
                        (_a = onCloseRef.current) === null || _a === void 0 ? void 0 : _a.call(onCloseRef);
                        openedWindow = null;
                        break;
                    }
                }
            })();
        }
        return () => {
            var _a;
            // Delay: Closing immediately causes Electron to crash
            setTimeout(() => {
                var _a;
                if (!(openedWindow === null || openedWindow === void 0 ? void 0 : openedWindow.closed)) {
                    openedWindow === null || openedWindow === void 0 ? void 0 : openedWindow.close();
                    (_a = onCloseRef.current) === null || _a === void 0 ? void 0 : _a.call(onCloseRef);
                    openedWindow = null;
                }
            }, 200);
            if (iframeElement && !openedWindow) {
                (_a = onCloseRef.current) === null || _a === void 0 ? void 0 : _a.call(onCloseRef);
            }
        };
    }, [iframeElement, mode]);
    return doc;
};
const useDocumentSetup = (doc, setLoaded, onFocus) => {
    const onFocusRef = (0, react_1.useRef)(onFocus);
    onFocusRef.current = onFocus;
    (0, react_1.useEffect)(() => {
        var _a;
        if (!doc)
            return;
        doc.open();
        doc.write('<!DOCTYPE html><html><head></head><body></body></html>');
        doc.close();
        const cssUrls = [
            'style.min.css',
        ];
        for (const url of cssUrls) {
            const style = doc.createElement('link');
            style.rel = 'stylesheet';
            style.href = url;
            doc.head.appendChild(style);
        }
        const jsUrls = [
            'vendor/lib/smalltalk/dist/smalltalk.min.js',
            './utils/window/eventHandlerOverrides.js',
        ];
        for (const url of jsUrls) {
            const script = doc.createElement('script');
            script.src = url;
            doc.head.appendChild(script);
        }
        doc.body.style.height = '100vh';
        const containerWindow = doc.defaultView;
        containerWindow.addEventListener('focus', () => {
            var _a;
            (_a = onFocusRef.current) === null || _a === void 0 ? void 0 : _a.call(onFocusRef);
        });
        if (doc.hasFocus()) {
            (_a = onFocusRef.current) === null || _a === void 0 ? void 0 : _a.call(onFocusRef);
        }
        setLoaded(true);
    }, [doc, setLoaded]);
};
const NewWindowOrIFrame = props => {
    const [iframeRef, setIframeRef] = (0, react_1.useState)(null);
    const [loaded, setLoaded] = (0, react_1.useState)(false);
    const doc = useDocument(props.mode, iframeRef, props.onClose);
    useDocumentSetup(doc, setLoaded, props.onFocus);
    (0, react_1.useEffect)(() => {
        if (!doc)
            return;
        doc.title = props.title;
    }, [doc, props.title]);
    (0, react_1.useEffect)(() => {
        const win = doc === null || doc === void 0 ? void 0 : doc.defaultView;
        if (win && 'electronWindow' in win && typeof win.electronWindow === 'object') {
            const electronWindow = win.electronWindow;
            electronWindow.onSetWindowId(props.windowId);
        }
    }, [doc, props.windowId]);
    const parentNode = loaded ? doc === null || doc === void 0 ? void 0 : doc.body : null;
    const wrappedChildren = React.createElement(exports.WindowIdContext.Provider, { value: props.windowId }, props.children);
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Needed to allow adding the portal to the DOM
    const contentPortal = parentNode && (0, react_dom_1.createPortal)(wrappedChildren, parentNode);
    if (props.mode === WindowMode.NewWindow) {
        return React.createElement("div", { style: { display: 'none' } }, contentPortal);
    }
    else {
        return React.createElement("iframe", { ref: setIframeRef, style: { flexGrow: 1, width: '100%', height: '100%', border: 'none' } }, contentPortal);
    }
};
exports.default = NewWindowOrIFrame;
//# sourceMappingURL=NewWindowOrIFrame.js.map