"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const react_1 = require("react");
const locale_1 = require("@joplin/lib/locale");
const DialogButtonRow_1 = require("./DialogButtonRow");
const { themeStyle } = require('@joplin/lib/theme');
const Countable = require('@joplin/lib/countable/Countable');
const markupLanguageUtils_1 = require("@joplin/lib/utils/markupLanguageUtils");
const Dialog_1 = require("./Dialog");
// eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
let markupToHtml_ = null;
function markupToHtml() {
    if (markupToHtml_)
        return markupToHtml_;
    markupToHtml_ = markupLanguageUtils_1.default.newMarkupToHtml();
    return markupToHtml_;
}
// eslint-disable-next-line @typescript-eslint/ban-types -- Old code before rule was applied
function countElements(text, wordSetter, characterSetter, characterNoSpaceSetter, lineSetter) {
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    Countable.count(text, (counter) => {
        wordSetter(counter.words);
        characterSetter(counter.all);
        characterNoSpaceSetter(counter.characters);
    });
    lineSetter(text === '' ? 0 : text.split('\n').length);
}
function formatReadTime(readTimeMinutes) {
    if (readTimeMinutes < 1) {
        return '< 1';
    }
    return Math.ceil(readTimeMinutes).toString();
}
function NoteContentPropertiesDialog(props) {
    const theme = themeStyle(props.themeId);
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    const tableBodyComps = [];
    // For the source Markdown
    const [lines, setLines] = (0, react_1.useState)(0);
    const [words, setWords] = (0, react_1.useState)(0);
    const [characters, setCharacters] = (0, react_1.useState)(0);
    const [charactersNoSpace, setCharactersNoSpace] = (0, react_1.useState)(0);
    // For source with Markdown syntax stripped out
    const [strippedLines, setStrippedLines] = (0, react_1.useState)(0);
    const [strippedWords, setStrippedWords] = (0, react_1.useState)(0);
    const [strippedCharacters, setStrippedCharacters] = (0, react_1.useState)(0);
    const [strippedCharactersNoSpace, setStrippedCharactersNoSpace] = (0, react_1.useState)(0);
    const [strippedReadTime, setStrippedReadTime] = (0, react_1.useState)(0);
    // This amount based on the following paper:
    // https://www.researchgate.net/publication/332380784_How_many_words_do_we_read_per_minute_A_review_and_meta-analysis_of_reading_rate
    const wordsPerMinute = 250;
    (0, react_1.useEffect)(() => {
        countElements(props.text, setWords, setCharacters, setCharactersNoSpace, setLines);
    }, [props.text]);
    (0, react_1.useEffect)(() => {
        const strippedText = markupToHtml().stripMarkup(props.markupLanguage, props.text);
        countElements(strippedText, setStrippedWords, setStrippedCharacters, setStrippedCharactersNoSpace, setStrippedLines);
        // eslint-disable-next-line @seiyab/react-hooks/exhaustive-deps -- Old code before rule was applied
    }, [props.text]);
    (0, react_1.useEffect)(() => {
        const readTimeMinutes = strippedWords / wordsPerMinute;
        setStrippedReadTime(readTimeMinutes);
    }, [strippedWords]);
    const textProperties = {
        lines: lines,
        words: words,
        characters: characters,
        charactersNoSpace: charactersNoSpace,
    };
    const strippedTextProperties = {
        // The function stripMarkup() currently removes all new lines so we can't use the
        // strippedLines property. Instead we simply use the lines property which should
        // be a good approximation anyway.
        // Also dummy check to silence TypeScript warning
        lines: strippedLines === -5000 ? strippedLines : lines,
        words: strippedWords,
        characters: strippedCharacters,
        charactersNoSpace: strippedCharactersNoSpace,
    };
    const keyToLabel = {
        words: (0, locale_1._)('Words'),
        characters: (0, locale_1._)('Characters'),
        charactersNoSpace: (0, locale_1._)('Characters excluding spaces'),
        lines: (0, locale_1._)('Lines'),
    };
    const buttonRow_click = () => {
        props.onClose();
    };
    const labelCompStyle = Object.assign(Object.assign({}, theme.textStyle), { fontWeight: 'bold', width: '10em' });
    const controlCompStyle = Object.assign(Object.assign({}, theme.textStyle), { textAlign: 'center' });
    const createTableBodyRow = (key, value, strippedValue) => {
        const labelComp = React.createElement("td", { style: labelCompStyle }, keyToLabel[key]);
        const controlComp = React.createElement("td", { style: controlCompStyle }, value);
        const strippedControlComp = React.createElement("td", { style: controlCompStyle }, strippedValue);
        return (React.createElement("tr", { key: key },
            labelComp,
            controlComp,
            strippedControlComp));
    };
    const tableHeaderStyle = Object.assign(Object.assign({}, theme.textStyle), { textAlign: 'center' });
    const tableHeader = (React.createElement("tr", null,
        React.createElement("th", { style: tableHeaderStyle }),
        React.createElement("th", { style: tableHeaderStyle }, (0, locale_1._)('Editor')),
        React.createElement("th", { style: tableHeaderStyle }, (0, locale_1._)('Viewer'))));
    for (const key in textProperties) {
        const comp = createTableBodyRow(key, textProperties[key], strippedTextProperties[key]);
        tableBodyComps.push(comp);
    }
    const dialogBoxHeadingStyle = Object.assign(Object.assign({}, theme.dialogTitle), { textAlign: 'center' });
    const readTimeLabel = (0, locale_1._)('Read time: %s min', formatReadTime(strippedReadTime));
    return (React.createElement(Dialog_1.default, { onCancel: props.onClose },
        React.createElement("div", { style: dialogBoxHeadingStyle }, (0, locale_1._)('Statistics')),
        React.createElement("table", null,
            React.createElement("thead", null, tableHeader),
            React.createElement("tbody", null, tableBodyComps)),
        React.createElement("div", { style: Object.assign(Object.assign({}, labelCompStyle), { marginTop: 10 }) }, readTimeLabel),
        React.createElement(DialogButtonRow_1.default, { themeId: props.themeId, onClick: buttonRow_click, okButtonShow: false, cancelButtonLabel: (0, locale_1._)('Close') })));
}
exports.default = NoteContentPropertiesDialog;
//# sourceMappingURL=NoteContentPropertiesDialog.js.map