"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const react_1 = require("react");
const NoteEditor_1 = require("./NoteEditor");
const StyleSheetContainer_1 = require("../StyleSheets/StyleSheetContainer");
const react_redux_1 = require("react-redux");
const NewWindowOrIFrame_1 = require("../NewWindowOrIFrame");
const WindowCommandsAndDialogs_1 = require("../WindowCommandsAndDialogs/WindowCommandsAndDialogs");
const { StyleSheetManager } = require('styled-components');
// Note: Transitive dependencies used only by react-select. Remove if react-select is removed.
const cache_1 = require("@emotion/cache");
const react_2 = require("@emotion/react");
const reducer_1 = require("@joplin/lib/reducer");
const emptyCallback = () => { };
const useWindowTitle = (isNewWindow) => {
    const [title, setTitle] = (0, react_1.useState)('Untitled');
    if (!isNewWindow) {
        return {
            windowTitle: 'Editor',
            onNoteTitleChange: emptyCallback,
        };
    }
    return { windowTitle: `Joplin - ${title}`, onNoteTitleChange: setTitle };
};
const SecondaryWindow = props => {
    const containerRef = (0, react_1.useRef)(null);
    const { windowTitle, onNoteTitleChange } = useWindowTitle(props.newWindow);
    const editor = React.createElement("div", { className: 'note-editor-wrapper', ref: containerRef },
        React.createElement(NoteEditor_1.default, { windowId: props.windowId, onTitleChange: onNoteTitleChange, startupPluginsLoaded: props.startupPluginsLoaded }));
    const newWindow = props.newWindow;
    const onWindowClose = (0, react_1.useCallback)(() => {
        if (newWindow) {
            props.dispatch({ type: 'WINDOW_CLOSE', windowId: props.windowId });
        }
    }, [props.dispatch, props.windowId, newWindow]);
    const onWindowFocus = (0, react_1.useCallback)(() => {
        var _a;
        // Verify that the window still has focus (e.g. to handle the case where the event was delayed).
        if ((_a = containerRef.current) === null || _a === void 0 ? void 0 : _a.ownerDocument.hasFocus()) {
            props.dispatch({
                type: 'WINDOW_FOCUS',
                windowId: props.windowId,
                lastWindowId: props.activeWindowId,
            });
        }
    }, [props.dispatch, props.windowId, props.activeWindowId]);
    return React.createElement(NewWindowOrIFrame_1.default, { mode: newWindow ? NewWindowOrIFrame_1.WindowMode.NewWindow : NewWindowOrIFrame_1.WindowMode.Iframe, windowId: props.windowId, onClose: onWindowClose, onFocus: onWindowFocus, title: windowTitle },
        React.createElement(LibraryStyleRoot, null,
            React.createElement(WindowCommandsAndDialogs_1.default, { windowId: props.windowId }),
            editor),
        React.createElement(StyleSheetContainer_1.default, null));
};
// Sets the root style container for libraries. At present, this is needed by react-select (which uses @emotion/...)
// and styled-components.
// See: https://github.com/JedWatson/react-select/issues/3680 and https://github.com/styled-components/styled-components/issues/659
const LibraryStyleRoot = props => {
    const [dependencyStyleContainer, setDependencyStyleContainer] = (0, react_1.useState)(null);
    const cache = (0, react_1.useMemo)(() => {
        return (0, cache_1.default)({
            key: 'new-window-cache',
            container: dependencyStyleContainer,
        });
    }, [dependencyStyleContainer]);
    return React.createElement(React.Fragment, null,
        React.createElement("div", { ref: setDependencyStyleContainer }),
        React.createElement(StyleSheetManager, { target: dependencyStyleContainer },
            React.createElement(react_2.CacheProvider, { value: cache }, props.children)));
};
exports.default = (0, react_redux_1.connect)((state, ownProps) => {
    var _a;
    // May be undefined if the window hasn't opened
    const windowState = reducer_1.stateUtils.windowStateById(state, ownProps.windowId);
    return {
        themeId: state.settings.theme,
        isSafeMode: state.settings.isSafeMode,
        codeView: (_a = windowState === null || windowState === void 0 ? void 0 : windowState.editorCodeView) !== null && _a !== void 0 ? _a : state.settings['editor.codeView'],
        legacyMarkdown: state.settings['editor.legacyMarkdown'],
        activeWindowId: reducer_1.stateUtils.activeWindowId(state),
        startupPluginsLoaded: state.startupPluginsLoaded,
    };
})(SecondaryWindow);
//# sourceMappingURL=EditorWindow.js.map