"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const react_1 = require("react");
const CodeMirror = require("codemirror");
require("codemirror/addon/comment/comment");
require("codemirror/addon/dialog/dialog");
require("codemirror/addon/edit/closebrackets");
require("codemirror/addon/edit/continuelist");
require("codemirror/addon/scroll/annotatescrollbar");
require("codemirror/addon/search/matchesonscrollbar");
require("codemirror/addon/search/searchcursor");
const useListIdent_1 = require("./utils/useListIdent");
const useScrollUtils_1 = require("./utils/useScrollUtils");
const useCursorUtils_1 = require("./utils/useCursorUtils");
const useLineSorting_1 = require("./utils/useLineSorting");
const useEditorSearchExtension_1 = require("../utils/useEditorSearchExtension");
const useJoplinMode_1 = require("./utils/useJoplinMode");
const useKeymap_1 = require("./utils/useKeymap");
const useExternalPlugins_1 = require("./utils/useExternalPlugins");
const useJoplinCommands_1 = require("./utils/useJoplinCommands");
require("codemirror/keymap/emacs");
require("codemirror/keymap/vim");
require("codemirror/keymap/sublime"); // Used for swapLineUp and swapLineDown
require("codemirror/mode/meta");
const Setting_1 = require("@joplin/lib/models/Setting");
// import eventManager from '@joplin/lib/eventManager';
const registry_1 = require("@joplin/lib/registry");
const focusHandler_1 = require("@joplin/lib/utils/focusHandler");
// Based on http://pypl.github.io/PYPL.html
const topLanguages = [
    'python',
    'clike',
    'javascript',
    'jsx',
    'php',
    'r',
    'swift',
    'go',
    'vb',
    'vbscript',
    'ruby',
    'rust',
    'dart',
    'lua',
    'groovy',
    'perl',
    'cobol',
    'julia',
    'haskell',
    'pascal',
    'css',
    // Additional languages, not in the PYPL list
    'xml', // For HTML too
    'markdown',
    'yaml',
    'shell',
    'dockerfile',
    'diff',
    'erlang',
    'sql',
];
// Load Top Modes
for (let i = 0; i < topLanguages.length; i++) {
    const mode = topLanguages[i];
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    if (CodeMirror.modeInfo.find((m) => m.mode === mode)) {
        require(`codemirror/mode/${mode}/${mode}`);
    }
    else {
        registry_1.reg.logger().error('Cannot find CodeMirror mode: ', mode);
    }
}
// eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
function Editor(props, ref) {
    const [editor, setEditor] = (0, react_1.useState)(null);
    const editorParent = (0, react_1.useRef)(null);
    const lastEditTime = (0, react_1.useRef)(NaN);
    // Codemirror plugins add new commands to codemirror (or change it's behavior)
    // This command adds the smartListIndent function which will be bound to tab
    (0, useListIdent_1.default)(CodeMirror);
    (0, useScrollUtils_1.default)(CodeMirror);
    (0, useCursorUtils_1.default)(CodeMirror);
    (0, useLineSorting_1.default)(CodeMirror);
    (0, useEditorSearchExtension_1.default)(CodeMirror);
    (0, useJoplinMode_1.default)(CodeMirror);
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    const pluginOptions = (0, useExternalPlugins_1.default)(CodeMirror, props.plugins);
    (0, useKeymap_1.default)(CodeMirror);
    (0, useJoplinCommands_1.default)(CodeMirror);
    (0, react_1.useImperativeHandle)(ref, () => {
        return editor;
    });
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    const editor_change = (0, react_1.useCallback)((cm, change) => {
        if (props.onChange && change.origin !== 'setValue') {
            props.onChange(cm.getValue());
            lastEditTime.current = Date.now();
        }
    }, [props.onChange]);
    // eslint-disable-next-line no-unused-vars, @typescript-eslint/no-unused-vars, @typescript-eslint/no-explicit-any -- Old code before rule was applied
    const editor_scroll = (0, react_1.useCallback)((_cm) => {
        props.onScroll();
    }, [props.onScroll]);
    // eslint-disable-next-line no-unused-vars, @typescript-eslint/no-unused-vars, @typescript-eslint/no-explicit-any -- Old code before rule was applied
    const editor_paste = (0, react_1.useCallback)((_cm, _event) => {
        props.onEditorPaste();
    }, [props.onEditorPaste]);
    // eslint-disable-next-line no-unused-vars, @typescript-eslint/no-unused-vars, @typescript-eslint/no-explicit-any -- Old code before rule was applied
    const editor_drop = (0, react_1.useCallback)((cm, _event) => {
        (0, focusHandler_1.focus)('v5/Editor::editor_drop', cm);
    }, []);
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    const editor_drag = (0, react_1.useCallback)((cm, event) => {
        // This is the type for all drag and drops that are external to codemirror
        // setting the cursor allows us to drop them in the right place
        if (event.dataTransfer.effectAllowed === 'all') {
            const coords = cm.coordsChar({ left: event.x, top: event.y });
            cm.setCursor(coords);
        }
        event.dataTransfer.dropEffect = 'copy';
    }, []);
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    const editor_resize = (0, react_1.useCallback)((cm) => {
        props.onResize(cm);
    }, [props.onResize]);
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    const editor_update = (0, react_1.useCallback)((cm) => {
        const edited = Date.now() - lastEditTime.current <= 100;
        props.onUpdate(cm, edited);
    }, [props.onUpdate]);
    (0, react_1.useEffect)(() => {
        if (!editorParent.current)
            return () => { };
        const userOptions = {};
        // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
        const safeOptions = {
            value: props.value,
            readOnly: props.readOnly,
        };
        // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
        const unsafeOptions = {
            screenReaderLabel: props.value,
            theme: props.codeMirrorTheme,
            mode: props.mode,
            readOnly: props.readOnly,
            autoCloseBrackets: props.autoMatchBraces,
            inputStyle: Setting_1.default.value('editor.spellcheckBeta') ? 'contenteditable' : 'textarea',
            lineWrapping: true,
            lineNumbers: false,
            indentWithTabs: true,
            indentUnit: 4,
            spellcheck: true,
            allowDropFileTypes: [''], // disable codemirror drop handling
            keyMap: props.keyMap ? props.keyMap : 'default',
        };
        // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
        let cmOptions = Object.assign({}, safeOptions);
        if (!props.isSafeMode) {
            cmOptions = Object.assign(Object.assign(Object.assign({}, cmOptions), unsafeOptions), userOptions);
        }
        const cm = CodeMirror(editorParent.current, cmOptions);
        setEditor(cm);
        cm.on('change', editor_change);
        cm.on('scroll', editor_scroll);
        cm.on('paste', editor_paste);
        cm.on('drop', editor_drop);
        cm.on('dragover', editor_drag);
        cm.on('refresh', editor_resize);
        cm.on('update', editor_update);
        // It's possible for searchMarkers to be available before the editor
        // In these cases we set the markers asap so the user can see them as
        // soon as the editor is ready
        if (props.searchMarkers) {
            cm.setMarkers(props.searchMarkers.keywords, props.searchMarkers.options);
        }
        return () => {
            // Clean up codemirror
            cm.off('change', editor_change);
            cm.off('scroll', editor_scroll);
            cm.off('paste', editor_paste);
            cm.off('drop', editor_drop);
            cm.off('dragover', editor_drag);
            cm.off('refresh', editor_resize);
            cm.off('update', editor_update);
            // eslint-disable-next-line @seiyab/react-hooks/exhaustive-deps -- Old code before rule was applied
            if (editorParent.current)
                editorParent.current.removeChild(cm.getWrapperElement());
            setEditor(null);
        };
        // eslint-disable-next-line @seiyab/react-hooks/exhaustive-deps -- Old code before rule was applied
    }, []);
    (0, react_1.useEffect)(() => {
        if (editor) {
            //  Value can also be changed by the editor itself so we need this guard
            //  to prevent loops
            if (props.value !== editor.getValue()) {
                editor.setValue(props.value);
                editor.clearHistory();
            }
            editor.setOption('screenReaderLabel', props.value);
        }
        // eslint-disable-next-line @seiyab/react-hooks/exhaustive-deps -- Old code before rule was applied
    }, [props.value]);
    (0, react_1.useEffect)(() => {
        if (editor) {
            editor.setOption('theme', props.codeMirrorTheme);
        }
        // eslint-disable-next-line @seiyab/react-hooks/exhaustive-deps -- Old code before rule was applied
    }, [props.codeMirrorTheme]);
    (0, react_1.useEffect)(() => {
        if (editor) {
            editor.setOption('mode', props.mode);
        }
        // eslint-disable-next-line @seiyab/react-hooks/exhaustive-deps -- Old code before rule was applied
    }, [props.mode]);
    (0, react_1.useEffect)(() => {
        if (editor) {
            editor.setOption('readOnly', props.readOnly);
        }
        // eslint-disable-next-line @seiyab/react-hooks/exhaustive-deps -- Old code before rule was applied
    }, [props.readOnly]);
    (0, react_1.useEffect)(() => {
        if (editor) {
            editor.setOption('autoCloseBrackets', props.autoMatchBraces);
        }
        // eslint-disable-next-line @seiyab/react-hooks/exhaustive-deps -- Old code before rule was applied
    }, [props.autoMatchBraces]);
    (0, react_1.useEffect)(() => {
        if (editor) {
            editor.setOption('keyMap', props.keyMap ? props.keyMap : 'default');
        }
        // eslint-disable-next-line @seiyab/react-hooks/exhaustive-deps -- Old code before rule was applied
    }, [props.keyMap]);
    (0, react_1.useEffect)(() => {
        if (editor) {
            for (const option in pluginOptions) {
                editor.setOption(option, pluginOptions[option]);
            }
        }
    }, [pluginOptions, editor]);
    return React.createElement("div", { className: 'codeMirrorEditor CodeMirror5', style: props.style, ref: editorParent });
}
exports.default = (0, react_1.forwardRef)(Editor);
//# sourceMappingURL=Editor.js.map