"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = require("react");
const loadContentScripts_1 = require("@joplin/lib/services/plugins/utils/loadContentScripts");
const path_1 = require("path");
const shim_1 = require("@joplin/lib/shim");
const uuid_1 = require("@joplin/lib/uuid");
const registry_1 = require("@joplin/lib/registry");
// eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
function useExternalPlugins(CodeMirror, plugins) {
    const [options, setOptions] = (0, react_1.useState)({});
    (0, react_1.useEffect)(() => {
        let newOptions = {};
        const contentScripts = (0, loadContentScripts_1.contentScriptsToCodeMirrorPlugin)(plugins);
        const addedElements = [];
        for (const contentScript of contentScripts) {
            try {
                const mod = contentScript.module;
                if (mod.codeMirrorResources) {
                    for (const asset of mod.codeMirrorResources) {
                        try {
                            require(`codemirror/${asset}`);
                        }
                        catch (error) {
                            error.message = `${asset} is not a valid CodeMirror asset, keymap or mode. You can find a list of valid assets here: https://codemirror.net/doc/manual.html#addons`;
                            throw error;
                        }
                    }
                }
                if (mod.codeMirrorOptions) {
                    newOptions = Object.assign(Object.assign({}, newOptions), mod.codeMirrorOptions);
                }
                if (mod.assets) {
                    const cssStrings = [];
                    for (const asset of mod.assets()) {
                        let mime = asset.mime;
                        if (!mime && asset.inline)
                            throw new Error('Mime type is required for inline assets');
                        if (!mime && asset.name) {
                            const ext = (0, path_1.extname)(asset.name).toLowerCase();
                            if (ext === '.css')
                                mime = 'text/css';
                        }
                        if (mime !== 'text/css')
                            throw new Error('Only css assets are supported for CodeMirror plugins');
                        if (asset.inline) {
                            cssStrings.push(asset.text);
                        }
                        else {
                            addedElements.push(addScript(shim_1.default.fsDriver().resolveRelativePathWithinDir(contentScript.assetPath, asset.name), contentScript.id));
                        }
                    }
                    if (cssStrings.length > 0) {
                        addedElements.push(addInlineCss(cssStrings, contentScript.id));
                    }
                }
                if (mod.plugin) {
                    mod.plugin(CodeMirror);
                }
            }
            catch (error) {
                registry_1.reg.logger().error(error.toString());
            }
        }
        setOptions(newOptions);
        return () => {
            for (const element of addedElements) {
                element.remove();
            }
        };
    }, [plugins, CodeMirror]);
    function addInlineCss(cssStrings, id) {
        const element = document.createElement('style');
        element.setAttribute('id', `content-script-${id}-inline-${uuid_1.default.createNano()}`);
        document.head.appendChild(element);
        element.appendChild(document.createTextNode(cssStrings.join('\n')));
        return element;
    }
    function addScript(path, id) {
        const element = document.createElement('link');
        element.setAttribute('id', `content-script-${id}-link-${uuid_1.default.createNano()}`);
        element.setAttribute('rel', 'stylesheet');
        element.setAttribute('href', path);
        document.head.appendChild(element);
        return element;
    }
    return options;
}
exports.default = useExternalPlugins;
//# sourceMappingURL=useExternalPlugins.js.map