"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const react_1 = require("react");
const createEditor_1 = require("@joplin/editor/CodeMirror/createEditor");
const types_1 = require("@joplin/lib/services/plugins/api/types");
const shim_1 = require("@joplin/lib/shim");
const PluginService_1 = require("@joplin/lib/services/plugins/PluginService");
const setupVim_1 = require("@joplin/editor/CodeMirror/utils/setupVim");
const path_1 = require("path");
const useKeymap_1 = require("./utils/useKeymap");
const useEditorSearchExtension_1 = require("../utils/useEditorSearchExtension");
const CommandService_1 = require("@joplin/lib/services/CommandService");
const localisation_1 = require("./utils/localisation");
const Editor = (props, ref) => {
    const editorContainerRef = (0, react_1.useRef)();
    const [editor, setEditor] = (0, react_1.useState)(null);
    // The editor will only be created once, so callbacks that could
    // change need to be stored as references.
    const onEventRef = (0, react_1.useRef)(props.onEvent);
    const onLogMessageRef = (0, react_1.useRef)(props.onLogMessage);
    (0, react_1.useEffect)(() => {
        onEventRef.current = props.onEvent;
        onLogMessageRef.current = props.onLogMessage;
    }, [props.onEvent, props.onLogMessage]);
    (0, useEditorSearchExtension_1.default)(editor);
    (0, react_1.useEffect)(() => {
        if (!editor) {
            return () => { };
        }
        // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
        const pasteEventHandler = (_editor, event) => {
            props.onEditorPaste(event);
        };
        editor.on('paste', pasteEventHandler);
        return () => {
            editor.off('paste', pasteEventHandler);
        };
    }, [editor, props.onEditorPaste]);
    (0, react_1.useImperativeHandle)(ref, () => {
        return editor;
    }, [editor]);
    (0, react_1.useEffect)(() => {
        var _a;
        if (!editor) {
            return;
        }
        const contentScripts = [];
        for (const pluginId in props.pluginStates) {
            const pluginState = props.pluginStates[pluginId];
            const codeMirrorContentScripts = (_a = pluginState.contentScripts[types_1.ContentScriptType.CodeMirrorPlugin]) !== null && _a !== void 0 ? _a : [];
            for (const contentScript of codeMirrorContentScripts) {
                contentScripts.push({
                    pluginId,
                    contentScriptId: contentScript.id,
                    contentScriptJs: () => shim_1.default.fsDriver().readFile(contentScript.path),
                    loadCssAsset: (name) => {
                        const assetPath = (0, path_1.dirname)(contentScript.path);
                        const path = shim_1.default.fsDriver().resolveRelativePathWithinDir(assetPath, name);
                        return shim_1.default.fsDriver().readFile(path, 'utf8');
                    },
                    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
                    postMessageHandler: (message) => {
                        const plugin = PluginService_1.default.instance().pluginById(pluginId);
                        return plugin.emitContentScriptMessage(contentScript.id, message);
                    },
                });
            }
        }
        void editor.setContentScripts(contentScripts);
    }, [editor, props.pluginStates]);
    (0, react_1.useEffect)(() => {
        if (!editorContainerRef.current)
            return () => { };
        const editorProps = Object.assign(Object.assign({}, props), { localisations: (0, localisation_1.default)(), onEvent: event => onEventRef.current(event), onLogMessage: message => onLogMessageRef.current(message) });
        const editor = (0, createEditor_1.default)(editorContainerRef.current, editorProps);
        editor.addStyles({
            '.cm-scroller': { overflow: 'auto' },
            '&.CodeMirror': {
                height: 'unset',
                background: 'unset',
                overflow: 'unset',
                direction: 'unset',
            },
        });
        setEditor(editor);
        return () => {
            editor.remove();
        };
        // eslint-disable-next-line @seiyab/react-hooks/exhaustive-deps -- Should run just once
    }, []);
    (0, react_1.useEffect)(() => {
        if (!editor) {
            return;
        }
        const searchState = editor.getSearchState();
        const externalSearchText = props.externalSearch.keywords.map(k => k.value).join(' ') || searchState.searchText;
        if (externalSearchText === searchState.searchText && searchState.dialogVisible === props.useLocalSearch) {
            return;
        }
        editor.setSearchState(Object.assign(Object.assign({}, searchState), { dialogVisible: props.useLocalSearch, searchText: externalSearchText }));
    }, [editor, props.externalSearch, props.useLocalSearch]);
    const theme = props.settings.themeData;
    (0, react_1.useEffect)(() => {
        if (!editor)
            return () => { };
        const styles = editor.addStyles({
            '& .cm-search-marker *, & .cm-search-marker': {
                color: theme.searchMarkerColor,
                backgroundColor: theme.searchMarkerBackgroundColor,
            },
            '& .cm-search-marker-selected *, & .cm-search-marker-selected': {
                background: `${theme.selectedColor2} !important`,
                color: `${theme.color2} !important`,
            },
        });
        return () => {
            styles.remove();
        };
    }, [editor, theme]);
    (0, react_1.useEffect)(() => {
        editor === null || editor === void 0 ? void 0 : editor.updateSettings(props.settings);
    }, [props.settings, editor]);
    (0, react_1.useEffect)(() => {
        if (!editor) {
            return;
        }
        (0, setupVim_1.default)(editor, {
            sync: () => {
                void CommandService_1.default.instance().execute('synchronize');
            },
        });
    }, [editor]);
    (0, useKeymap_1.default)(editor);
    return (React.createElement("div", { style: props.style, ref: editorContainerRef }));
};
exports.default = (0, react_1.forwardRef)(Editor);
//# sourceMappingURL=Editor.js.map