"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const types_1 = require("@joplin/lib/services/plugins/api/types");
const SpellCheckerService_1 = require("@joplin/lib/services/spellChecker/SpellCheckerService");
const react_1 = require("react");
const bridge_1 = require("../../../../../services/bridge");
const contextMenuUtils_1 = require("../../../utils/contextMenuUtils");
const contextMenu_1 = require("../../../utils/contextMenu");
const MenuUtils_1 = require("@joplin/lib/services/commands/MenuUtils");
const CommandService_1 = require("@joplin/lib/services/CommandService");
const Setting_1 = require("@joplin/lib/models/Setting");
const Resource_1 = require("@joplin/lib/models/Resource");
const Menu = (0, bridge_1.default)().Menu;
const MenuItem = (0, bridge_1.default)().MenuItem;
const menuUtils = new MenuUtils_1.default(CommandService_1.default.instance());
// x and y are the absolute coordinates, as returned by the context-menu event
// handler on the webContent. This function will return null if the point is
// not within the TinyMCE editor.
function contextMenuElement(editor, x, y) {
    if (!editor || !editor.getDoc())
        return null;
    const containerDoc = editor.getContainer().ownerDocument;
    const iframes = containerDoc.getElementsByClassName('tox-edit-area__iframe');
    if (!iframes.length)
        return null;
    const zoom = Setting_1.default.value('windowContentZoomFactor') / 100;
    const xScreen = x / zoom;
    const yScreen = y / zoom;
    // We use .elementFromPoint to handle the case where a dialog is covering
    // part of the editor.
    const targetElement = containerDoc.elementFromPoint(xScreen, yScreen);
    if (targetElement !== iframes[0]) {
        return null;
    }
    const iframeRect = iframes[0].getBoundingClientRect();
    const relativeX = xScreen - iframeRect.left;
    const relativeY = yScreen - iframeRect.top;
    return editor.getDoc().elementFromPoint(relativeX, relativeY);
}
const contextMenuActionOptions = { current: null };
// eslint-disable-next-line @typescript-eslint/ban-types, @typescript-eslint/no-explicit-any -- Old code before rule was applied, Old code before rule was applied
function default_1(editor, plugins, dispatch, htmlToMd, mdToHtml) {
    (0, react_1.useEffect)(() => {
        if (!editor)
            return () => { };
        const contextMenuItems = (0, contextMenu_1.menuItems)(dispatch, htmlToMd, mdToHtml);
        const targetWindow = (0, bridge_1.default)().activeWindow();
        // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
        function onContextMenu(event, params) {
            const element = contextMenuElement(editor, params.x, params.y);
            if (!element)
                return;
            event.preventDefault();
            const menu = new Menu();
            let itemType = contextMenuUtils_1.ContextMenuItemType.None;
            let resourceId = '';
            let linkToCopy = null;
            if (element.nodeName === 'IMG') {
                itemType = contextMenuUtils_1.ContextMenuItemType.Image;
                resourceId = Resource_1.default.pathToId(element.src);
            }
            else if (element.nodeName === 'A') {
                resourceId = Resource_1.default.pathToId(element.href);
                itemType = resourceId ? contextMenuUtils_1.ContextMenuItemType.Resource : contextMenuUtils_1.ContextMenuItemType.Link;
                linkToCopy = element.getAttribute('href') || '';
            }
            else {
                itemType = contextMenuUtils_1.ContextMenuItemType.Text;
            }
            contextMenuActionOptions.current = {
                itemType,
                resourceId,
                filename: null,
                mime: null,
                linkToCopy,
                textToCopy: null,
                htmlToCopy: editor.selection ? editor.selection.getContent() : '',
                insertContent: (content) => {
                    editor.insertContent(content);
                },
                isReadOnly: false,
                fireEditorEvent: (event) => {
                    editor.fire(event);
                },
                htmlToMd,
                mdToHtml,
            };
            for (const itemName in contextMenuItems) {
                const item = contextMenuItems[itemName];
                if (!item.isActive(itemType, contextMenuActionOptions.current))
                    continue;
                menu.append(new MenuItem({
                    label: item.label,
                    click: () => {
                        item.onAction(contextMenuActionOptions.current);
                    },
                }));
            }
            const spellCheckerMenuItems = SpellCheckerService_1.default.instance().contextMenuItems(params.misspelledWord, params.dictionarySuggestions);
            for (const item of spellCheckerMenuItems) {
                menu.append(new MenuItem(item));
            }
            for (const item of menuUtils.pluginContextMenuItems(plugins, types_1.MenuItemLocation.EditorContextMenu)) {
                menu.append(new MenuItem(item));
            }
            menu.popup({ window: targetWindow });
        }
        targetWindow.webContents.prependListener('context-menu', onContextMenu);
        return () => {
            var _a;
            if (!targetWindow.isDestroyed() && ((_a = targetWindow === null || targetWindow === void 0 ? void 0 : targetWindow.webContents) === null || _a === void 0 ? void 0 : _a.off)) {
                targetWindow.webContents.off('context-menu', onContextMenu);
            }
        };
    }, [editor, plugins, dispatch, htmlToMd, mdToHtml]);
}
exports.default = default_1;
//# sourceMappingURL=useContextMenu.js.map