"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const react_1 = require("react");
const TinyMCE_1 = require("./NoteBody/TinyMCE/TinyMCE");
const react_redux_1 = require("react-redux");
const MultiNoteActions_1 = require("../MultiNoteActions");
const utils_1 = require("./utils");
const useSearchMarkers_1 = require("./utils/useSearchMarkers");
const useNoteSearchBar_1 = require("./utils/useNoteSearchBar");
const useMessageHandler_1 = require("./utils/useMessageHandler");
const useWindowCommandHandler_1 = require("./utils/useWindowCommandHandler");
const useDropHandler_1 = require("./utils/useDropHandler");
const useMarkupToHtml_1 = require("../hooks/useMarkupToHtml");
const useFormNote_1 = require("./utils/useFormNote");
const useEffectiveNoteId_1 = require("./utils/useEffectiveNoteId");
const useFolder_1 = require("./utils/useFolder");
const styles_1 = require("./styles");
const CommandService_1 = require("@joplin/lib/services/CommandService");
const ToolbarButton_1 = require("../ToolbarButton/ToolbarButton");
const Button_1 = require("../Button/Button");
const eventManager_1 = require("@joplin/lib/eventManager");
const ToolbarButtonUtils_1 = require("@joplin/lib/services/commands/ToolbarButtonUtils");
const locale_1 = require("@joplin/lib/locale");
const TagList_1 = require("../TagList");
const NoteTitleBar_1 = require("./NoteTitle/NoteTitleBar");
const markupLanguageUtils_1 = require("@joplin/lib/utils/markupLanguageUtils");
const Setting_1 = require("@joplin/lib/models/Setting");
const stateToWhenClauseContext_1 = require("../../services/commands/stateToWhenClauseContext");
const ExternalEditWatcher_1 = require("@joplin/lib/services/ExternalEditWatcher");
const readOnly_1 = require("@joplin/lib/models/utils/readOnly");
const { themeStyle } = require('@joplin/lib/theme');
const { substrWithEllipsis } = require('@joplin/lib/string-utils');
const NoteSearchBar_1 = require("../NoteSearchBar");
const Note_1 = require("@joplin/lib/models/Note");
const Folder_1 = require("@joplin/lib/models/Folder");
const NoteRevisionViewer_1 = require("../NoteRevisionViewer");
const reducer_1 = require("@joplin/lib/services/share/reducer");
const useAsyncEffect_1 = require("@joplin/lib/hooks/useAsyncEffect");
const BaseModel_1 = require("@joplin/lib/BaseModel");
const BaseItem_1 = require("@joplin/lib/models/BaseItem");
const errors_1 = require("@joplin/lib/errors");
const ItemChange_1 = require("@joplin/lib/models/ItemChange");
const PlainEditor_1 = require("./NoteBody/PlainEditor/PlainEditor");
const CodeMirror_1 = require("./NoteBody/CodeMirror/v6/CodeMirror");
const CodeMirror_2 = require("./NoteBody/CodeMirror/v5/CodeMirror");
const contextMenu_1 = require("./utils/contextMenu");
const renderer_1 = require("@joplin/renderer");
const useScrollWhenReadyOptions_1 = require("./utils/useScrollWhenReadyOptions");
const useScheduleSaveCallbacks_1 = require("./utils/useScheduleSaveCallbacks");
const WarningBanner_1 = require("./WarningBanner/WarningBanner");
const UserWebview_1 = require("../../services/plugins/UserWebview");
const Logger_1 = require("@joplin/utils/Logger");
const usePluginEditorView_1 = require("./utils/usePluginEditorView");
const reducer_2 = require("@joplin/lib/reducer");
const NewWindowOrIFrame_1 = require("../NewWindowOrIFrame");
const PluginService_1 = require("@joplin/lib/services/plugins/PluginService");
const AsyncActionQueue_1 = require("@joplin/lib/AsyncActionQueue");
const useResourceUnwatcher_1 = require("./utils/useResourceUnwatcher");
const debounce = require('debounce');
const logger = Logger_1.default.create('NoteEditor');
const commands = [
    require('./commands/showRevisions'),
];
const toolbarButtonUtils = new ToolbarButtonUtils_1.default(CommandService_1.default.instance());
const onDragOver = event => event.preventDefault();
let editorIdCounter = 0;
const makeNoteUpdateAction = (shownEditorViewIds) => {
    return async () => {
        for (const viewId of shownEditorViewIds) {
            const controller = PluginService_1.default.instance().viewControllerByViewId(viewId);
            if (controller)
                controller.emitUpdate();
        }
    };
};
function NoteEditorContent(props) {
    const [showRevisions, setShowRevisions] = (0, react_1.useState)(false);
    const [titleHasBeenManuallyChanged, setTitleHasBeenManuallyChanged] = (0, react_1.useState)(false);
    const [isReadOnly, setIsReadOnly] = (0, react_1.useState)(false);
    const editorRef = (0, react_1.useRef)();
    const titleInputRef = (0, react_1.useRef)();
    const isMountedRef = (0, react_1.useRef)(true);
    const noteSearchBarRef = (0, react_1.useRef)(null);
    const viewUpdateAsyncQueue_ = (0, react_1.useRef)(new AsyncActionQueue_1.default(100, AsyncActionQueue_1.IntervalType.Fixed));
    const shownEditorViewIds = props['plugins.shownEditorViewIds'];
    // Should be constant and unique to this instance of the editor.
    const editorId = (0, react_1.useMemo)(() => {
        return `editor-${editorIdCounter++}`;
    }, []);
    const setFormNoteRef = (0, react_1.useRef)();
    const { saveNoteIfWillChange, scheduleSaveNote } = (0, useScheduleSaveCallbacks_1.default)({
        setFormNote: setFormNoteRef, dispatch: props.dispatch, editorRef, editorId,
    });
    const formNote_beforeLoad = (0, react_1.useCallback)(async (event) => {
        await saveNoteIfWillChange(event.formNote);
        setShowRevisions(false);
    }, [saveNoteIfWillChange]);
    const formNote_afterLoad = (0, react_1.useCallback)(async () => {
        setTitleHasBeenManuallyChanged(false);
    }, []);
    const effectiveNoteId = (0, useEffectiveNoteId_1.default)(props);
    (0, useAsyncEffect_1.default)(async (event) => {
        if (!props.startupPluginsLoaded)
            return;
        let filterObject = {
            activatedEditors: [],
        };
        filterObject = await eventManager_1.default.filterEmit('editorActivationCheck', filterObject);
        if (event.cancelled)
            return;
        for (const editor of filterObject.activatedEditors) {
            const controller = PluginService_1.default.instance().pluginById(editor.pluginId).viewController(editor.viewId);
            controller.setActive(editor.isActive);
        }
    }, [effectiveNoteId, props.startupPluginsLoaded]);
    (0, react_1.useEffect)(() => {
        if (!props.startupPluginsLoaded)
            return;
        viewUpdateAsyncQueue_.current.push(makeNoteUpdateAction(shownEditorViewIds));
    }, [effectiveNoteId, shownEditorViewIds, props.startupPluginsLoaded]);
    const { editorPlugin, editorView } = (0, usePluginEditorView_1.default)(props.plugins, shownEditorViewIds);
    const builtInEditorVisible = !editorPlugin;
    const { formNote, setFormNote, isNewNote, resourceInfos } = (0, useFormNote_1.default)({
        noteId: effectiveNoteId,
        isProvisional: props.isProvisional,
        titleInputRef: titleInputRef,
        editorRef: editorRef,
        onBeforeLoad: formNote_beforeLoad,
        onAfterLoad: formNote_afterLoad,
        builtInEditorVisible,
        editorId,
    });
    setFormNoteRef.current = setFormNote;
    const formNoteRef = (0, react_1.useRef)();
    formNoteRef.current = Object.assign({}, formNote);
    const formNoteFolder = (0, useFolder_1.default)({ folderId: formNote.parent_id });
    const { localSearch, onChange: localSearch_change, onNext: localSearch_next, onPrevious: localSearch_previous, onClose: localSearch_close, setResultCount: setLocalSearchResultCount, showLocalSearch, setShowLocalSearch, searchMarkers: localSearchMarkerOptions, } = (0, useNoteSearchBar_1.default)({ noteSearchBarRef });
    // If the note has been modified in another editor, wait for it to be saved
    // before loading it in this editor.
    // const waitingToSaveNote = props.noteId && formNote.id !== props.noteId && props.editorNoteStatuses[props.noteId] === 'saving';
    const styles = (0, styles_1.default)(props);
    const whiteBackgroundNoteRendering = formNote.markup_language === renderer_1.MarkupLanguage.Html;
    const markupToHtml = (0, useMarkupToHtml_1.default)({
        themeId: props.themeId,
        whiteBackgroundNoteRendering,
        customCss: props.customCss,
        plugins: props.plugins,
        scrollbarSize: props.scrollbarSize,
    });
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    const allAssets = (0, react_1.useCallback)(async (markupLanguage, options = null) => {
        options = Object.assign({ contentMaxWidthTarget: '' }, options);
        const theme = themeStyle(options.themeId ? options.themeId : props.themeId);
        const markupToHtml = markupLanguageUtils_1.default.newMarkupToHtml({}, {
            resourceBaseUrl: `joplin-content://note-viewer/${Setting_1.default.value('resourceDir')}/`,
            customCss: props.customCss,
        });
        return markupToHtml.allAssets(markupLanguage, theme, {
            contentMaxWidth: props.contentMaxWidth,
            contentMaxWidthTarget: options.contentMaxWidthTarget,
            scrollbarSize: props.scrollbarSize,
            whiteBackgroundNoteRendering: options.whiteBackgroundNoteRendering,
        });
    }, [props.themeId, props.scrollbarSize, props.customCss, props.contentMaxWidth]);
    const handleProvisionalFlag = (0, react_1.useCallback)(() => {
        if (props.isProvisional) {
            props.dispatch({
                type: 'NOTE_PROVISIONAL_FLAG_CLEAR',
                id: formNote.id,
            });
        }
    }, [props.isProvisional, formNote.id, props.dispatch]);
    const scheduleNoteListResort = (0, react_1.useMemo)(() => {
        return debounce(() => {
            // Although the note list will update automatically, it may take some time. This
            // forces an immediate update.
            props.dispatch({ type: 'NOTE_SORT' });
        }, 100);
    }, [props.dispatch]);
    (0, react_1.useEffect)(() => {
        var _a;
        (_a = props.onTitleChange) === null || _a === void 0 ? void 0 : _a.call(props, formNote.title);
    }, [formNote.title, props.onTitleChange]);
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    const onFieldChange = (0, react_1.useCallback)(async (field, value, changeId = 0) => {
        if (!isMountedRef.current) {
            // When the component is unmounted, various actions can happen which can
            // trigger onChange events, for example the textarea might be cleared.
            // We need to ignore these events, otherwise the note is going to be saved
            // with an invalid body.
            logger.debug('Skipping change event because the component is unmounted');
            return;
        }
        handleProvisionalFlag();
        const change = field === 'body' ? {
            body: value,
        } : {
            title: value,
        };
        const newNote = Object.assign(Object.assign(Object.assign({}, formNote), change), { bodyWillChangeId: 0, bodyChangeId: 0, hasChanged: true });
        if (field === 'title') {
            setTitleHasBeenManuallyChanged(true);
        }
        if (isNewNote && !titleHasBeenManuallyChanged && field === 'body') {
            // TODO: Handle HTML/Markdown format
            newNote.title = Note_1.default.defaultTitle(value);
        }
        if (changeId !== null && field === 'body' && formNote.bodyWillChangeId !== changeId) {
            // Note was changed, but another note was loaded before save - skipping
            // The previously loaded note, that was modified, will be saved via saveNoteIfWillChange()
        }
        else {
            setFormNote(newNote);
            await scheduleSaveNote(newNote);
        }
        if (field === 'title') {
            // Scheduling a resort needs to be:
            // - called after scheduleSaveNote so that the new note title is used for sorting
            // - debounced because many calls to scheduleSaveNote can resolve at once
            scheduleNoteListResort();
        }
    }, [handleProvisionalFlag, formNote, setFormNote, isNewNote, titleHasBeenManuallyChanged, scheduleNoteListResort, scheduleSaveNote]);
    const onDrop = (0, useDropHandler_1.default)({ editorRef });
    const onBodyChange = (0, react_1.useCallback)((event) => onFieldChange('body', event.content, event.changeId), [onFieldChange]);
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    const onTitleChange = (0, react_1.useCallback)((event) => onFieldChange('title', event.target.value), [onFieldChange]);
    const containerRef = (0, react_1.useRef)(null);
    (0, useWindowCommandHandler_1.default)({
        dispatch: props.dispatch,
        setShowLocalSearch,
        noteSearchBarRef,
        editorRef,
        titleInputRef,
        onBodyChange,
        containerRef,
    });
    // const onTitleKeydown = useCallback((event:any) => {
    // 	const keyCode = event.keyCode;
    // 	if (keyCode === 9) {
    // 		// TAB
    // 		event.preventDefault();
    // 		if (event.shiftKey) {
    // 			CommandService.instance().execute('focusElement', 'noteList');
    // 		} else {
    // 			CommandService.instance().execute('focusElement', 'noteBody');
    // 		}
    // 	}
    // }, [props.dispatch]);
    const shareCache = (0, react_1.useMemo)(() => {
        return (0, reducer_1.parseShareCache)(props.shareCacheSetting);
    }, [props.shareCacheSetting]);
    (0, useAsyncEffect_1.default)(async (event) => {
        if (!formNote.id)
            return;
        try {
            const result = await (0, readOnly_1.itemIsReadOnly)(BaseItem_1.default, BaseModel_1.ModelType.Note, ItemChange_1.default.SOURCE_UNSPECIFIED, formNote.id, props.syncUserId, shareCache);
            if (event.cancelled)
                return;
            setIsReadOnly(result);
        }
        catch (error) {
            if (error.code === errors_1.ErrorCode.NotFound) {
                // Can happen if the note has been deleted but a render is
                // triggered anyway. It can be ignored.
            }
            else {
                throw error;
            }
        }
    }, [formNote.id, props.syncUserId, shareCache]);
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    const onBodyWillChange = (0, react_1.useCallback)((event) => {
        handleProvisionalFlag();
        setFormNote(prev => {
            return Object.assign(Object.assign({}, prev), { bodyWillChangeId: event.changeId, hasChanged: true });
        });
        props.dispatch({
            type: 'EDITOR_NOTE_STATUS_SET',
            id: formNote.id,
            status: 'saving',
        });
    }, [formNote, setFormNote, handleProvisionalFlag, props.dispatch]);
    const { scrollWhenReady, clearScrollWhenReady } = (0, useScrollWhenReadyOptions_1.default)({
        noteId: formNote.id,
        selectedNoteHash: props.selectedNoteHash,
        lastEditorScrollPercents: props.lastEditorScrollPercents,
        editorRef,
    });
    const windowId = (0, react_1.useContext)(NewWindowOrIFrame_1.WindowIdContext);
    const onMessage = (0, useMessageHandler_1.default)(scrollWhenReady, clearScrollWhenReady, windowId, editorRef, setLocalSearchResultCount, props.dispatch, formNote, utils_1.htmlToMarkdown, markupToHtml);
    (0, useResourceUnwatcher_1.default)({ noteId: formNote.id, windowId });
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    const externalEditWatcher_noteChange = (0, react_1.useCallback)((event) => {
        if (event.id === formNote.id) {
            const newFormNote = Object.assign(Object.assign({}, formNote), { title: event.note.title, body: event.note.body });
            setFormNote(newFormNote);
        }
    }, [formNote, setFormNote]);
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    const onNotePropertyChange = (0, react_1.useCallback)((event) => {
        setFormNote(formNote => {
            if (formNote.id !== event.note.id)
                return formNote;
            const newFormNote = Object.assign({}, formNote);
            for (const key in event.note) {
                if (key === 'id')
                    continue;
                // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
                newFormNote[key] = event.note[key];
            }
            return newFormNote;
        });
    }, [setFormNote]);
    (0, react_1.useEffect)(() => {
        eventManager_1.default.on(eventManager_1.EventName.AlarmChange, onNotePropertyChange);
        ExternalEditWatcher_1.default.instance().on('noteChange', externalEditWatcher_noteChange);
        return () => {
            eventManager_1.default.off(eventManager_1.EventName.AlarmChange, onNotePropertyChange);
            ExternalEditWatcher_1.default.instance().off('noteChange', externalEditWatcher_noteChange);
        };
    }, [externalEditWatcher_noteChange, onNotePropertyChange]);
    (0, react_1.useEffect)(() => {
        const dependencies = {
            setShowRevisions,
            isInFocusedDocument: () => {
                var _a, _b;
                return (_b = (_a = containerRef.current) === null || _a === void 0 ? void 0 : _a.ownerDocument) === null || _b === void 0 ? void 0 : _b.hasFocus();
            },
        };
        const registeredCommands = CommandService_1.default.instance().componentRegisterCommands(dependencies, commands, true);
        return () => {
            registeredCommands.deregister();
        };
    }, [setShowRevisions]);
    const onScroll = (0, react_1.useCallback)((event) => {
        props.dispatch({
            type: 'EDITOR_SCROLL_PERCENT_SET',
            // In callbacks of setTimeout()/setInterval(), props/state cannot be used
            // to refer the current value, since they would be one or more generations old.
            // For the purpose, useRef value should be used.
            noteId: formNoteRef.current.id,
            percent: event.percent,
        });
    }, [props.dispatch]);
    function renderNoNotes(rootStyle) {
        const emptyDivStyle = Object.assign({ backgroundColor: 'black', opacity: 0.1 }, rootStyle);
        return React.createElement("div", { style: emptyDivStyle, ref: containerRef });
    }
    function renderTagButton() {
        return React.createElement(ToolbarButton_1.default, { themeId: props.themeId, toolbarButtonInfo: props.setTagsToolbarButtonInfo });
    }
    function renderTagBar() {
        const theme = themeStyle(props.themeId);
        const noteIds = [formNote.id];
        const instructions = React.createElement("span", { onClick: () => { void CommandService_1.default.instance().execute('setTags', noteIds); }, style: Object.assign(Object.assign({}, theme.clickableTextStyle), { whiteSpace: 'nowrap' }) }, (0, locale_1._)('Click to add tags...'));
        const tagList = props.selectedNoteTags.length ? React.createElement(TagList_1.default, { items: props.selectedNoteTags }) : null;
        return (React.createElement("div", { style: { paddingLeft: 8, display: 'flex', flexDirection: 'row', alignItems: 'center' } },
            tagList,
            instructions));
    }
    const searchMarkers = (0, useSearchMarkers_1.default)(showLocalSearch, localSearchMarkerOptions, props.searches, props.selectedSearchId, props.highlightedWords);
    const editorProps = {
        ref: editorRef,
        contentKey: formNote.id,
        style: styles.tinyMCE,
        whiteBackgroundNoteRendering,
        onChange: onBodyChange,
        onWillChange: onBodyWillChange,
        onMessage: onMessage,
        content: formNote.body,
        contentMarkupLanguage: formNote.markup_language,
        contentOriginalCss: formNote.originalCss,
        resourceInfos: resourceInfos,
        resourceDirectory: Setting_1.default.value('resourceDir'),
        htmlToMarkdown: utils_1.htmlToMarkdown,
        markupToHtml: markupToHtml,
        allAssets: allAssets,
        disabled: isReadOnly,
        themeId: props.themeId,
        dispatch: props.dispatch,
        noteToolbar: null,
        onScroll: onScroll,
        setLocalSearchResultCount: setLocalSearchResultCount,
        setLocalSearch: localSearch_change,
        setShowLocalSearch,
        useLocalSearch: showLocalSearch,
        searchMarkers: searchMarkers,
        visiblePanes: props.noteVisiblePanes || ['editor', 'viewer'],
        keyboardMode: Setting_1.default.value('editor.keyboardMode'),
        locale: Setting_1.default.value('locale'),
        onDrop: onDrop,
        noteToolbarButtonInfos: props.toolbarButtonInfos,
        plugins: props.plugins,
        fontSize: Setting_1.default.value('style.editor.fontSize'),
        contentMaxWidth: props.contentMaxWidth,
        scrollbarSize: props.scrollbarSize,
        isSafeMode: props.isSafeMode,
        useCustomPdfViewer: props.useCustomPdfViewer,
        // We need it to identify the context for which media is rendered.
        // It is currently used to remember pdf scroll position for each attachments of each note uniquely.
        noteId: props.noteId,
        watchedNoteFiles: props.watchedNoteFiles,
    };
    let editor = null;
    if (builtInEditorVisible) {
        if (props.bodyEditor === 'TinyMCE') {
            editor = React.createElement(TinyMCE_1.default, Object.assign({}, editorProps));
        }
        else if (props.bodyEditor === 'PlainText') {
            editor = React.createElement(PlainEditor_1.default, Object.assign({}, editorProps));
        }
        else if (props.bodyEditor === 'CodeMirror5') {
            editor = React.createElement(CodeMirror_2.default, Object.assign({}, editorProps));
        }
        else if (props.bodyEditor === 'CodeMirror6') {
            editor = React.createElement(CodeMirror_1.default, Object.assign({}, editorProps));
        }
        else {
            throw new Error(`Invalid editor: ${props.bodyEditor}`);
        }
    }
    const noteRevisionViewer_onBack = (0, react_1.useCallback)(() => {
        setShowRevisions(false);
    }, []);
    const onBannerResourceClick = (0, react_1.useCallback)(async (event) => {
        event.preventDefault();
        const resourceId = event.currentTarget.getAttribute('data-resource-id');
        await (0, contextMenu_1.openItemById)(resourceId, props.dispatch);
    }, [props.dispatch]);
    if (showRevisions) {
        const theme = themeStyle(props.themeId);
        const revStyle = {
            // ...props.style,
            display: 'inline-flex',
            padding: theme.margin,
            verticalAlign: 'top',
            boxSizing: 'border-box',
            flex: 1,
        };
        return (React.createElement("div", { style: revStyle, ref: containerRef },
            React.createElement(NoteRevisionViewer_1.default, { customCss: props.customCss, noteId: formNote.id, onBack: noteRevisionViewer_onBack })));
    }
    if (props.selectedNoteIds.length > 1) {
        return React.createElement(MultiNoteActions_1.default, { themeId: props.themeId, selectedNoteIds: props.selectedNoteIds, notes: props.notes, dispatch: props.dispatch, watchedNoteFiles: props.watchedNoteFiles, plugins: props.plugins, inConflictFolder: props.selectedFolderId === Folder_1.default.conflictFolderId(), customCss: props.customCss });
    }
    function renderSearchBar() {
        if (!showLocalSearch)
            return false;
        const theme = themeStyle(props.themeId);
        return (React.createElement(NoteSearchBar_1.default, { ref: noteSearchBarRef, themeId: props.themeId, style: {
                display: 'flex',
                height: 35,
                borderTop: `1px solid ${theme.dividerColor}`,
            }, query: localSearch.query, searching: localSearch.searching, resultCount: localSearch.resultCount, selectedIndex: localSearch.selectedIndex, onChange: localSearch_change, onNext: localSearch_next, onPrevious: localSearch_previous, onClose: localSearch_close, visiblePanes: props.noteVisiblePanes, editorType: props.bodyEditor }));
    }
    function renderResourceWatchingNotification() {
        if (!Object.keys(props.watchedResources).length)
            return null;
        const resourceTitles = Object.keys(props.watchedResources).map(id => props.watchedResources[id].title);
        return (React.createElement("div", { style: styles.resourceWatchBanner },
            React.createElement("p", { style: styles.resourceWatchBannerLine },
                (0, locale_1._)('The following attachments are being watched for changes:'),
                " ",
                React.createElement("strong", null, resourceTitles.join(', '))),
            React.createElement("p", { style: Object.assign(Object.assign({}, styles.resourceWatchBannerLine), { marginBottom: 0 }) }, (0, locale_1._)('The attachments will no longer be watched when you switch to a different note.'))));
    }
    const renderResourceInSearchResultsNotification = () => {
        const resourceResults = props.searchResults.filter(r => r.id === props.noteId && r.item_type === BaseModel_1.ModelType.Resource);
        if (!resourceResults.length)
            return null;
        const renderResource = (id, title) => {
            return React.createElement("li", { key: id },
                React.createElement("a", { "data-resource-id": id, onClick: onBannerResourceClick, href: "#" }, title));
        };
        return (React.createElement("div", { style: styles.resourceWatchBanner },
            React.createElement("p", { style: styles.resourceWatchBannerLine }, (0, locale_1._n)('The following attachment matches your search query:', 'The following attachments match your search query:', resourceResults.length)),
            React.createElement("ul", null, resourceResults.map(r => renderResource(r.item_id, r.title)))));
    };
    function renderSearchInfo() {
        const theme = themeStyle(props.themeId);
        if (formNoteFolder && ['Search', 'Tag', 'SmartFilter'].includes(props.notesParentType)) {
            return (React.createElement("div", { style: { paddingTop: 10, paddingBottom: 10, paddingLeft: theme.editorPaddingLeft } },
                React.createElement(Button_1.default, { iconName: "icon-notebooks", level: Button_1.ButtonLevel.Primary, title: (0, locale_1._)('In: %s', substrWithEllipsis(formNoteFolder.title, 0, 100)), onClick: () => {
                        props.dispatch({
                            type: 'FOLDER_AND_NOTE_SELECT',
                            folderId: formNoteFolder.id,
                            noteId: formNote.id,
                        });
                    } }),
                React.createElement("div", { style: { flex: 1 } })));
        }
        else {
            return null;
        }
    }
    const renderPluginEditor = () => {
        var _a, _b;
        if (!editorPlugin)
            return null;
        const html = (_b = (_a = props.pluginHtmlContents[editorPlugin.id]) === null || _a === void 0 ? void 0 : _a[editorView.id]) !== null && _b !== void 0 ? _b : '';
        return React.createElement(UserWebview_1.default, { key: editorView.id, viewId: editorView.id, themeId: props.themeId, html: html, scripts: editorView.scripts, pluginId: editorPlugin.id, borderBottom: true, fitToContent: false });
    };
    if (formNote.encryption_applied || !formNote.id || !effectiveNoteId) {
        return renderNoNotes(styles.root);
    }
    const theme = themeStyle(props.themeId);
    return (React.createElement("div", { style: styles.root, onDragOver: onDragOver, onDrop: onDrop, ref: containerRef },
        React.createElement("div", { style: { display: 'flex', flexDirection: 'column', height: '100%' } },
            renderResourceWatchingNotification(),
            renderResourceInSearchResultsNotification(),
            React.createElement(NoteTitleBar_1.default, { titleInputRef: titleInputRef, themeId: props.themeId, isProvisional: props.isProvisional, noteIsTodo: formNote.is_todo, noteTitle: formNote.title, noteUserUpdatedTime: formNote.user_updated_time, onTitleChange: onTitleChange, disabled: isReadOnly }),
            renderSearchInfo(),
            React.createElement("div", { style: { display: 'flex', flex: 1, paddingLeft: theme.editorPaddingLeft, maxHeight: '100%', minHeight: '0' } },
                editor,
                renderPluginEditor()),
            React.createElement("div", { style: { display: 'flex', flexDirection: 'row', alignItems: 'center' } }, renderSearchBar()),
            React.createElement("div", { className: "tag-bar", style: { paddingLeft: theme.editorPaddingLeft, display: 'flex', flexDirection: 'row', alignItems: 'center', height: 40 } },
                renderTagButton(),
                renderTagBar()),
            React.createElement(WarningBanner_1.default, { bodyEditor: props.bodyEditor }))));
}
const mapStateToProps = (state, ownProps) => {
    const whenClauseContext = (0, stateToWhenClauseContext_1.default)(state, { windowId: ownProps.windowId });
    const windowState = reducer_2.stateUtils.windowStateById(state, ownProps.windowId);
    const noteId = reducer_2.stateUtils.selectedNoteId(windowState);
    let bodyEditor = windowState.editorCodeView ? 'CodeMirror6' : 'TinyMCE';
    if (state.settings.isSafeMode) {
        bodyEditor = 'PlainText';
    }
    else if (windowState.editorCodeView && state.settings['editor.legacyMarkdown']) {
        bodyEditor = 'CodeMirror5';
    }
    return {
        noteId,
        bodyEditor,
        isProvisional: state.provisionalNoteIds.includes(noteId),
        notes: windowState.notes,
        selectedNoteIds: windowState.selectedNoteIds,
        selectedFolderId: windowState.selectedFolderId,
        editorNoteStatuses: state.editorNoteStatuses,
        themeId: state.settings.theme,
        watchedNoteFiles: state.watchedNoteFiles,
        notesParentType: windowState.notesParentType,
        selectedNoteTags: windowState.selectedNoteTags,
        lastEditorScrollPercents: state.lastEditorScrollPercents,
        selectedNoteHash: windowState.selectedNoteHash,
        searches: state.searches,
        selectedSearchId: windowState.selectedSearchId,
        customCss: state.customViewerCss,
        noteVisiblePanes: windowState.noteVisiblePanes,
        watchedResources: windowState.watchedResources,
        highlightedWords: state.highlightedWords,
        plugins: state.pluginService.plugins,
        pluginHtmlContents: state.pluginService.pluginHtmlContents,
        'plugins.shownEditorViewIds': state.settings['plugins.shownEditorViewIds'] || [],
        toolbarButtonInfos: toolbarButtonUtils.commandsToToolbarButtons([
            'historyBackward',
            'historyForward',
            'toggleEditors',
            'toggleExternalEditing',
        ], whenClauseContext),
        setTagsToolbarButtonInfo: toolbarButtonUtils.commandsToToolbarButtons([
            'setTags',
        ], whenClauseContext)[0],
        contentMaxWidth: state.settings['style.editor.contentMaxWidth'],
        scrollbarSize: state.settings['style.scrollbarSize'],
        isSafeMode: state.settings.isSafeMode,
        useCustomPdfViewer: false,
        syncUserId: state.settings['sync.userId'],
        shareCacheSetting: state.settings['sync.shareCache'],
        searchResults: state.searchResults,
    };
};
exports.default = (0, react_redux_1.connect)(mapStateToProps)(NoteEditorContent);
//# sourceMappingURL=NoteEditor.js.map