"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const locale_1 = require("@joplin/lib/locale");
const CommandService_1 = require("@joplin/lib/services/CommandService");
const react_1 = require("react");
const NoteToolbar_1 = require("../../NoteToolbar/NoteToolbar");
const theme_1 = require("@joplin/lib/theme");
const time_1 = require("@joplin/lib/time");
const NewWindowOrIFrame_1 = require("../../NewWindowOrIFrame");
function styles_(props) {
    return (0, theme_1.buildStyle)(['NoteEditorTitleBar'], props.themeId, theme => {
        return {
            titleInput: {
                flex: 1,
                display: 'inline-block',
                paddingTop: 5,
                minHeight: 38,
                boxSizing: 'border-box',
                fontWeight: 'bold',
                paddingBottom: 5,
                paddingLeft: 0,
                paddingRight: 8,
                color: theme.textStyle.color,
                fontSize: Math.round(theme.textStyle.fontSize * 1.5),
                backgroundColor: theme.backgroundColor,
                border: 'none',
                width: '100%',
            },
            titleDate: Object.assign(Object.assign({}, theme.textStyle), { color: theme.colorFaded, paddingLeft: 8, whiteSpace: 'nowrap' }),
            toolbarStyle: {
                marginBottom: 0,
                minWidth: 0,
            },
        };
    });
}
const useReselectHandlers = () => {
    const lastTitleFocus = (0, react_1.useRef)([0, 0]);
    const lastTitleValue = (0, react_1.useRef)('');
    const onTitleBlur = (0, react_1.useCallback)((event) => {
        const titleElement = event.currentTarget;
        lastTitleFocus.current = [titleElement.selectionStart, titleElement.selectionEnd];
        lastTitleValue.current = titleElement.value;
    }, []);
    const onTitleFocus = (0, react_1.useCallback)((event) => {
        const titleElement = event.currentTarget;
        // By default, focusing the note title bar can cause its content to become selected. We override
        // this with a more reasonable default:
        if (titleElement.selectionStart === 0 && titleElement.selectionEnd === titleElement.value.length) {
            if (lastTitleValue.current !== titleElement.value) {
                titleElement.selectionStart = titleElement.value.length;
            }
            else {
                titleElement.selectionStart = lastTitleFocus.current[0];
                titleElement.selectionEnd = lastTitleFocus.current[1];
            }
        }
    }, []);
    return { onTitleBlur, onTitleFocus };
};
function NoteTitleBar(props) {
    const styles = styles_(props);
    const onTitleKeydown = (0, react_1.useCallback)((event) => {
        const titleElement = event.currentTarget;
        const selectionAtEnd = titleElement.selectionEnd === titleElement.value.length;
        const isNavigationShortcut = (event.key === 'ArrowDown' && selectionAtEnd) || (event.key === 'Enter' && !event.shiftKey);
        const composing = event.nativeEvent.isComposing;
        // Don't change focus if the navigation shortcut is fired during composition. See
        // https://github.com/laurent22/joplin/issues/11485.
        if (!composing && isNavigationShortcut) {
            event.preventDefault();
            const moveCursorToStart = event.key === 'ArrowDown';
            void CommandService_1.default.instance().execute('focusElement', 'noteBody', { moveCursorToStart });
        }
    }, []);
    const { onTitleFocus, onTitleBlur } = useReselectHandlers();
    function renderTitleBarDate() {
        return React.createElement("span", { className: "updated-time-label", style: styles.titleDate }, time_1.default.formatMsToLocal(props.noteUserUpdatedTime));
    }
    const windowId = (0, react_1.useContext)(NewWindowOrIFrame_1.WindowIdContext);
    function renderNoteToolbar() {
        return React.createElement(NoteToolbar_1.default, { themeId: props.themeId, style: styles.toolbarStyle, disabled: props.disabled, windowId: windowId });
    }
    return (React.createElement("div", { className: 'note-title-wrapper' },
        React.createElement("input", { className: "title-input", type: "text", ref: props.titleInputRef, placeholder: props.isProvisional ? (props.noteIsTodo ? (0, locale_1._)('Creating new to-do...') : (0, locale_1._)('Creating new note...')) : '', "aria-label": props.isProvisional ? undefined : (0, locale_1._)('Note title'), style: styles.titleInput, readOnly: props.disabled, onChange: props.onTitleChange, onKeyDown: onTitleKeydown, onFocus: onTitleFocus, onBlur: onTitleBlur, value: props.noteTitle }),
        React.createElement("div", { className: 'note-title-info-group' },
            renderTitleBarDate(),
            renderNoteToolbar())));
}
exports.default = NoteTitleBar;
//# sourceMappingURL=NoteTitleBar.js.map