"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const react_redux_1 = require("react-redux");
const Setting_1 = require("@joplin/lib/models/Setting");
const BannerContent_1 = require("./BannerContent");
const locale_1 = require("@joplin/lib/locale");
const bridge_1 = require("../../../services/bridge");
const react_1 = require("react");
const PluginService_1 = require("@joplin/lib/services/plugins/PluginService");
const onRichTextDismissLinkClick = () => {
    Setting_1.default.setValue('richTextBannerDismissed', true);
};
const onRichTextReadMoreLinkClick = () => {
    void (0, bridge_1.default)().openExternal('https://joplinapp.org/help/apps/rich_text_editor');
};
const onSwitchToLegacyEditor = () => {
    Setting_1.default.setValue('editor.legacyMarkdown', true);
};
const onDismissLegacyEditorPrompt = () => {
    Setting_1.default.setValue('editor.pluginCompatibilityBannerDismissedFor', [...PluginService_1.default.instance().pluginIds]);
};
const incompatiblePluginIds = [
    // cSpell:disable
    'com.septemberhx.Joplin.Enhancement',
    'ylc395.noteLinkSystem',
    'outline',
    'joplin.plugin.cmoptions',
    'com.asdibiase.joplin-languagetool',
    // cSpell:enable
];
const WarningBanner = props => {
    const wysiwygBanner = (React.createElement(BannerContent_1.default, { acceptMessage: (0, locale_1._)('Read more about it'), onAccept: onRichTextReadMoreLinkClick, onDismiss: onRichTextDismissLinkClick, visible: props.bodyEditor === 'TinyMCE' && !props.richTextBannerDismissed }, (0, locale_1._)('This Rich Text editor has a number of limitations and it is recommended to be aware of them before using it.')));
    const incompatiblePluginNames = (0, react_1.useMemo)(() => {
        if (props.bodyEditor !== 'CodeMirror6') {
            return [];
        }
        const runningPluginIds = Object.keys(props.plugins);
        return runningPluginIds.map((id) => {
            var _a;
            if ((_a = props.pluginCompatibilityBannerDismissedFor) === null || _a === void 0 ? void 0 : _a.includes(id)) {
                return [];
            }
            if (incompatiblePluginIds.includes(id)) {
                return PluginService_1.default.instance().pluginById(id).manifest.name;
            }
            else {
                return [];
            }
        }).flat();
    }, [props.bodyEditor, props.plugins, props.pluginCompatibilityBannerDismissedFor]);
    const markdownPluginBanner = (React.createElement(BannerContent_1.default, { acceptMessage: (0, locale_1._)('Switch to the legacy editor'), onAccept: onSwitchToLegacyEditor, onDismiss: onDismissLegacyEditorPrompt, visible: incompatiblePluginNames.length > 0 },
        (0, locale_1._)('The following plugins may not support the current markdown editor:'),
        React.createElement("ul", null, incompatiblePluginNames.map((name, index) => React.createElement("li", { key: index }, name)))));
    return React.createElement(React.Fragment, null,
        wysiwygBanner,
        markdownPluginBanner);
};
exports.default = (0, react_redux_1.connect)((state) => {
    return {
        richTextBannerDismissed: state.settings.richTextBannerDismissed,
        pluginCompatibilityBannerDismissedFor: state.settings['editor.pluginCompatibilityBannerDismissedFor'],
        plugins: state.pluginService.plugins,
    };
})(WarningBanner);
//# sourceMappingURL=WarningBanner.js.map