"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.menuItems = exports.openItemById = void 0;
const index_1 = require("@joplin/lib/services/ResourceEditWatcher/index");
const locale_1 = require("@joplin/lib/locale");
const clipboardUtils_1 = require("./clipboardUtils");
const bridge_1 = require("../../../services/bridge");
const contextMenuUtils_1 = require("./contextMenuUtils");
const Menu = (0, bridge_1.default)().Menu;
const MenuItem = (0, bridge_1.default)().MenuItem;
const Resource_1 = require("@joplin/lib/models/Resource");
const BaseItem_1 = require("@joplin/lib/models/BaseItem");
const BaseModel_1 = require("@joplin/lib/BaseModel");
const resourceHandling_1 = require("./resourceHandling");
const types_1 = require("@joplin/lib/services/database/types");
const types_2 = require("../NoteBody/TinyMCE/utils/types");
const readOnly_1 = require("@joplin/lib/models/utils/readOnly");
const Setting_1 = require("@joplin/lib/models/Setting");
const ItemChange_1 = require("@joplin/lib/models/ItemChange");
const shim_1 = require("@joplin/lib/shim");
const utils_1 = require("@joplin/lib/services/ExternalEditWatcher/utils");
const fs = require('fs-extra');
const { writeFile } = require('fs-extra');
const { clipboard } = require('electron');
const { toSystemSlashes } = require('@joplin/lib/path-utils');
function handleCopyToClipboard(options) {
    if (options.textToCopy) {
        clipboard.writeText(options.textToCopy);
    }
    else if (options.htmlToCopy) {
        (0, clipboardUtils_1.copyHtmlToClipboard)(options.htmlToCopy);
    }
}
// eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
async function saveFileData(data, filename) {
    const newFilePath = await (0, bridge_1.default)().showSaveDialog({ defaultPath: filename });
    if (!newFilePath)
        return;
    await writeFile(newFilePath, data);
}
// eslint-disable-next-line @typescript-eslint/ban-types -- Old code before rule was applied
async function openItemById(itemId, dispatch, hash = '') {
    const item = await BaseItem_1.default.loadItemById(itemId);
    if (!item)
        throw new Error(`No item with ID ${itemId}`);
    if (item.type_ === BaseModel_1.default.TYPE_RESOURCE) {
        const resource = item;
        const localState = await Resource_1.default.localState(resource);
        if (localState.fetch_status !== Resource_1.default.FETCH_STATUS_DONE || !!resource.encryption_blob_encrypted) {
            if (localState.fetch_status === Resource_1.default.FETCH_STATUS_ERROR) {
                (0, bridge_1.default)().showErrorMessageBox(`${(0, locale_1._)('There was an error downloading this attachment:')}\n\n${localState.fetch_error}`);
            }
            else {
                (0, bridge_1.default)().showErrorMessageBox((0, locale_1._)('This attachment is not downloaded or not decrypted yet'));
            }
            return;
        }
        try {
            if ((0, readOnly_1.itemIsReadOnlySync)(BaseModel_1.ModelType.Resource, ItemChange_1.default.SOURCE_UNSPECIFIED, resource, Setting_1.default.value('sync.userId'), BaseItem_1.default.syncShareCache)) {
                await index_1.default.instance().openAsReadOnly(resource.id);
            }
            else {
                await index_1.default.instance().openAndWatch(resource.id);
            }
        }
        catch (error) {
            console.error(error);
            (0, bridge_1.default)().showErrorMessageBox(error.message);
        }
    }
    else if (item.type_ === BaseModel_1.default.TYPE_NOTE) {
        const note = item;
        dispatch({
            type: 'FOLDER_AND_NOTE_SELECT',
            folderId: note.parent_id,
            noteId: note.id,
            hash,
        });
    }
    else {
        throw new Error(`Unsupported item type: ${item.type_}`);
    }
}
exports.openItemById = openItemById;
// eslint-disable-next-line @typescript-eslint/ban-types -- Old code before rule was applied
function menuItems(dispatch, htmlToMd, mdToHtml) {
    return {
        open: {
            label: (0, locale_1._)('Open...'),
            onAction: async (options) => {
                await openItemById(options.resourceId, dispatch);
            },
            isActive: (itemType, options) => !options.textToCopy && (itemType === contextMenuUtils_1.ContextMenuItemType.Image || itemType === contextMenuUtils_1.ContextMenuItemType.Resource),
        },
        saveAs: {
            label: (0, locale_1._)('Save as...'),
            onAction: async (options) => {
                const { resourcePath, resource } = await (0, contextMenuUtils_1.resourceInfo)(options);
                const filePath = await (0, bridge_1.default)().showSaveDialog({
                    defaultPath: resource.filename ? resource.filename : resource.title,
                });
                if (!filePath)
                    return;
                await fs.copy(resourcePath, filePath);
            },
            // We handle images received as text separately as it can be saved in multiple formats
            isActive: (itemType, options) => !options.textToCopy && (itemType === contextMenuUtils_1.ContextMenuItemType.Image || itemType === contextMenuUtils_1.ContextMenuItemType.Resource),
        },
        saveAsSvg: {
            label: (0, locale_1._)('Save as %s', 'SVG'),
            onAction: async (options) => {
                await saveFileData(options.textToCopy, options.filename);
            },
            isActive: (itemType, options) => { var _a; return !!options.textToCopy && itemType === contextMenuUtils_1.ContextMenuItemType.Image && ((_a = options.mime) === null || _a === void 0 ? void 0 : _a.startsWith('image/svg')); },
        },
        saveAsPng: {
            label: (0, locale_1._)('Save as %s', 'PNG'),
            onAction: async (options) => {
                // First convert it to png then save
                if (options.mime !== 'image/svg+xml') {
                    throw new Error(`Unsupported image type: ${options.mime}`);
                }
                if (!options.filename) {
                    throw new Error('Filename is needed to save as png');
                }
                // double dimensions to make sure it's always big enough even on hdpi screens
                const [width, height] = (0, contextMenuUtils_1.svgDimensions)(document, options.textToCopy).map((x) => x * 2 || undefined);
                const dataUri = (0, contextMenuUtils_1.textToDataUri)(options.textToCopy, options.mime);
                const png = await (0, contextMenuUtils_1.svgUriToPng)(document, dataUri, width, height);
                const filename = options.filename.replace('.svg', '.png');
                await saveFileData(png, filename);
            },
            isActive: (itemType, options) => { var _a; return !!options.textToCopy && itemType === contextMenuUtils_1.ContextMenuItemType.Image && ((_a = options.mime) === null || _a === void 0 ? void 0 : _a.startsWith('image/svg')); },
        },
        revealInFolder: {
            label: (0, locale_1._)('Reveal file in folder'),
            onAction: async (options) => {
                const { resourcePath } = await (0, contextMenuUtils_1.resourceInfo)(options);
                (0, bridge_1.default)().showItemInFolder(resourcePath);
            },
            isActive: (itemType, options) => !options.textToCopy && itemType === contextMenuUtils_1.ContextMenuItemType.Image || itemType === contextMenuUtils_1.ContextMenuItemType.Resource,
        },
        copyOcrText: {
            label: (0, locale_1._)('View OCR text'),
            onAction: async (options) => {
                const { resource } = await (0, contextMenuUtils_1.resourceInfo)(options);
                if (resource.ocr_status === types_1.ResourceOcrStatus.Done) {
                    const tempFilePath = `${Setting_1.default.value('tempDir')}/${resource.id}_ocr.txt`;
                    await shim_1.default.fsDriver().writeFile(tempFilePath, resource.ocr_text, 'utf8');
                    await (0, utils_1.openFileWithExternalEditor)(tempFilePath, (0, bridge_1.default)());
                }
                else {
                    (0, bridge_1.default)().showInfoMessageBox((0, locale_1._)('This attachment does not have OCR data (Status: %s)', (0, Resource_1.resourceOcrStatusToString)(resource.ocr_status)));
                }
            },
            isActive: (itemType, options) => {
                return itemType === contextMenuUtils_1.ContextMenuItemType.Resource || (itemType === contextMenuUtils_1.ContextMenuItemType.Image && options.resourceId);
            },
        },
        copyPathToClipboard: {
            label: (0, locale_1._)('Copy path to clipboard'),
            onAction: async (options) => {
                let path = '';
                if (options.textToCopy && options.mime) {
                    path = (0, contextMenuUtils_1.textToDataUri)(options.textToCopy, options.mime);
                }
                else {
                    const { resourcePath } = await (0, contextMenuUtils_1.resourceInfo)(options);
                    if (resourcePath)
                        path = toSystemSlashes(resourcePath);
                }
                if (!path)
                    return;
                clipboard.writeText(path);
            },
            isActive: (itemType) => itemType === contextMenuUtils_1.ContextMenuItemType.Image || itemType === contextMenuUtils_1.ContextMenuItemType.Resource,
        },
        copyImage: {
            label: (0, locale_1._)('Copy image'),
            onAction: async (options) => {
                const { resourcePath } = await (0, contextMenuUtils_1.resourceInfo)(options);
                const image = (0, bridge_1.default)().createImageFromPath(resourcePath);
                clipboard.writeImage(image);
            },
            isActive: (itemType, options) => !options.textToCopy && itemType === contextMenuUtils_1.ContextMenuItemType.Image,
        },
        cut: {
            label: (0, locale_1._)('Cut'),
            onAction: async (options) => {
                handleCopyToClipboard(options);
                options.insertContent('');
            },
            isActive: (itemType, options) => itemType !== contextMenuUtils_1.ContextMenuItemType.Image && (!options.isReadOnly && (!!options.textToCopy || !!options.htmlToCopy)),
        },
        copy: {
            label: (0, locale_1._)('Copy'),
            onAction: async (options) => {
                handleCopyToClipboard(options);
            },
            isActive: (itemType, options) => itemType !== contextMenuUtils_1.ContextMenuItemType.Image && (!!options.textToCopy || !!options.htmlToCopy),
        },
        paste: {
            label: (0, locale_1._)('Paste'),
            onAction: async (options) => {
                const pastedHtml = clipboard.readHTML();
                let content = pastedHtml ? pastedHtml : clipboard.readText();
                if (pastedHtml) {
                    content = await (0, resourceHandling_1.processPastedHtml)(pastedHtml, htmlToMd, mdToHtml);
                }
                options.insertContent(content);
            },
            isActive: (_itemType, options) => !options.isReadOnly && (!!clipboard.readText() || !!clipboard.readHTML()),
        },
        pasteAsText: {
            label: (0, locale_1._)('Paste as text'),
            onAction: async (options) => {
                options.fireEditorEvent(types_2.TinyMceEditorEvents.PasteAsText);
            },
            isActive: (_itemType, options) => !options.isReadOnly && (!!clipboard.readText() || !!clipboard.readHTML()),
        },
        copyLinkUrl: {
            label: (0, locale_1._)('Copy Link Address'),
            onAction: async (options) => {
                clipboard.writeText(options.linkToCopy !== null ? options.linkToCopy : options.textToCopy);
            },
            isActive: (itemType, options) => itemType === contextMenuUtils_1.ContextMenuItemType.Link || !!options.linkToCopy,
        },
    };
}
exports.menuItems = menuItems;
// eslint-disable-next-line @typescript-eslint/ban-types -- Old code before rule was applied
async function contextMenu(options, dispatch) {
    const menu = new Menu();
    const items = menuItems(dispatch, options.htmlToMd, options.mdToHtml);
    if (!('readyOnly' in options))
        options.isReadOnly = true;
    for (const itemKey in items) {
        const item = items[itemKey];
        if (!item.isActive(options.itemType, options))
            continue;
        menu.append(new MenuItem({
            label: item.label,
            click: () => {
                item.onAction(options);
            },
        }));
    }
    return menu;
}
exports.default = contextMenu;
//# sourceMappingURL=contextMenu.js.map