"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.formNoteToNote = exports.htmlToMarkdown = void 0;
const HtmlToMd_1 = require("@joplin/lib/HtmlToMd");
const Note_1 = require("@joplin/lib/models/Note");
const { MarkupToHtml } = require('@joplin/renderer');
async function htmlToMarkdown(markupLanguage, html, originalCss, parseOptions = null) {
    let newBody = '';
    if (markupLanguage === MarkupToHtml.MARKUP_LANGUAGE_MARKDOWN) {
        const htmlToMd = new HtmlToMd_1.default();
        newBody = htmlToMd.parse(html, Object.assign({ preserveImageTagsWithSize: true, preserveNestedTables: true, preserveColorStyles: true }, parseOptions));
        newBody = await Note_1.default.replaceResourceExternalToInternalLinks(newBody, { useAbsolutePaths: true });
    }
    else {
        newBody = await Note_1.default.replaceResourceExternalToInternalLinks(html, { useAbsolutePaths: true });
        if (originalCss)
            newBody = `<style>${originalCss}</style>\n${newBody}`;
    }
    return newBody;
}
exports.htmlToMarkdown = htmlToMarkdown;
async function formNoteToNote(formNote) {
    return {
        id: formNote.id,
        // Should also include parent_id and deleted_time so that the reducer
        // can know in which folder the note should go when saving.
        // https://discourse.joplinapp.org/t/experimental-wysiwyg-editor-in-joplin/6915/57?u=laurent
        parent_id: formNote.parent_id,
        deleted_time: formNote.deleted_time,
        is_conflict: formNote.is_conflict,
        title: formNote.title,
        body: formNote.body,
    };
}
exports.formNoteToNote = formNoteToNote;
//# sourceMappingURL=index.js.map