"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Setting_1 = require("@joplin/lib/models/Setting");
const resourceHandling_1 = require("./resourceHandling");
const markupLanguageUtils_1 = require("@joplin/lib/markupLanguageUtils");
const HtmlToMd_1 = require("@joplin/lib/HtmlToMd");
const createTestMarkupConverters = () => {
    const markupToHtml = async (markupLanguage, markup, options) => {
        const conv = markupLanguageUtils_1.default.newMarkupToHtml({}, {
            resourceBaseUrl: `file://${Setting_1.default.value('resourceDir')}/`,
            customCss: '',
        });
        return conv.render(markupLanguage, markup, {}, options);
    };
    const htmlToMd = async (_markupLanguage, html, _originalCss) => {
        const conv = new HtmlToMd_1.default();
        return conv.parse(html);
    };
    return { markupToHtml, htmlToMd };
};
describe('resourceHandling', () => {
    it('should sanitize pasted HTML', async () => {
        Setting_1.default.setConstant('resourceDir', '/home/.config/joplin/resources');
        const testCases = [
            ['Test: <style onload="evil()"></style>', 'Test: <style></style>'],
            ['<a href="javascript: alert()">test</a>', '<a href="#">test</a>'],
            ['<a href="file:///home/.config/joplin/resources/test.pdf">test</a>', '<a href="file:///home/.config/joplin/resources/test.pdf">test</a>'],
            ['<a href="file:///etc/passwd">evil.pdf</a>', '<a href="#">evil.pdf</a>'],
            ['<script >evil()</script>', ''],
            ['<script>evil()</script>', ''],
            [
                '<img onload="document.body.innerHTML = evil;" src="data:image/svg+xml;base64,=="/>',
                '<img src="data:image/svg+xml;base64,=="/>',
            ],
        ];
        for (const [html, expected] of testCases) {
            expect(await (0, resourceHandling_1.processPastedHtml)(html, null, null)).toBe(expected);
        }
    });
    it('should clean up pasted HTML', async () => {
        const { markupToHtml, htmlToMd } = createTestMarkupConverters();
        const testCases = [
            ['<p style="background-color: red">Hello</p><p style="display: hidden;">World</p>', '<p>Hello</p>\n<p>World</p>\n'],
            ['', ''],
        ];
        for (const [html, expected] of testCases) {
            expect(await (0, resourceHandling_1.processPastedHtml)(html, htmlToMd, markupToHtml)).toBe(expected);
        }
    });
    it('should preserve images pasted from the resource directory', async () => {
        const { markupToHtml, htmlToMd } = createTestMarkupConverters();
        // All images in the resource directory should be preserved.
        const html = `<img src="file://${encodeURI(Setting_1.default.value('resourceDir'))}/resource.png" alt="test"/>`;
        expect(await (0, resourceHandling_1.processPastedHtml)(html, htmlToMd, markupToHtml)).toBe(html);
    });
});
//# sourceMappingURL=resourceHandling.test.js.map