"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Logger_1 = require("@joplin/utils/Logger");
const react_1 = require("react");
const _1 = require(".");
const ExternalEditWatcher_1 = require("@joplin/lib/services/ExternalEditWatcher");
const Note_1 = require("@joplin/lib/models/Note");
const eventManager_1 = require("@joplin/lib/eventManager");
const logger = Logger_1.default.create('useScheduleSaveCallbacks');
const useScheduleSaveCallbacks = (props) => {
    const scheduleSaveNote = (0, react_1.useCallback)((formNote) => {
        if (!formNote.saveActionQueue)
            throw new Error('saveActionQueue is not set!!'); // Sanity check
        // reg.logger().debug('Scheduling...', formNote);
        const makeAction = (formNote) => {
            return async function () {
                const note = await (0, _1.formNoteToNote)(formNote);
                logger.debug('Saving note...', note);
                const savedNote = await Note_1.default.save(note, { changeId: `editorChange-${props.editorId}` });
                props.setFormNote.current((prev) => {
                    return Object.assign(Object.assign({}, prev), { user_updated_time: savedNote.user_updated_time, hasChanged: false });
                });
                void ExternalEditWatcher_1.default.instance().updateNoteFile(savedNote);
                props.dispatch({
                    type: 'EDITOR_NOTE_STATUS_REMOVE',
                    id: formNote.id,
                });
                eventManager_1.default.emit(eventManager_1.EventName.NoteContentChange, { note: savedNote });
            };
        };
        formNote.saveActionQueue.push(makeAction(formNote));
        return formNote.saveActionQueue.waitForAllDone();
    }, [props.dispatch, props.editorId, props.setFormNote]);
    const saveNoteIfWillChange = (0, react_1.useCallback)(async (formNote) => {
        if (!formNote.id || !formNote.bodyWillChangeId)
            return;
        const body = await props.editorRef.current.content();
        void scheduleSaveNote(Object.assign(Object.assign({}, formNote), { body: body, bodyWillChangeId: 0, bodyChangeId: 0 }));
    }, [scheduleSaveNote, props.editorRef]);
    return { saveNoteIfWillChange, scheduleSaveNote };
};
exports.default = useScheduleSaveCallbacks;
//# sourceMappingURL=useScheduleSaveCallbacks.js.map