"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const BaseModel_1 = require("@joplin/lib/BaseModel");
const Note_1 = require("@joplin/lib/models/Note");
const CommandService_1 = require("@joplin/lib/services/CommandService");
const react_1 = require("react");
const noteListType_1 = require("@joplin/lib/services/plugins/api/noteListType");
const announceForAccessibility_1 = require("../../utils/announceForAccessibility");
const locale_1 = require("@joplin/lib/locale");
const useOnKeyDown = (activeNoteId, selectedNoteIds, moveNote, makeItemIndexVisible, focusNote, notes, dispatch, visibleItemCount, noteCount, flow, itemsPerLine) => {
    const scrollNoteIndex = (0, react_1.useCallback)((visibleItemCount, key, ctrlKey, metaKey, noteIndex) => {
        if (flow === noteListType_1.ItemFlow.TopToBottom) {
            if (key === 'PageUp') {
                noteIndex -= (visibleItemCount - 1);
            }
            else if (key === 'PageDown') {
                noteIndex += (visibleItemCount - 1);
            }
            else if ((key === 'End' && ctrlKey) || (key === 'ArrowDown' && metaKey)) {
                noteIndex = noteCount - 1;
            }
            else if ((key === 'Home' && ctrlKey) || (key === 'ArrowUp' && metaKey)) {
                noteIndex = 0;
            }
            else if (key === 'ArrowUp' && !metaKey) {
                noteIndex -= 1;
            }
            else if (key === 'ArrowDown' && !metaKey) {
                noteIndex += 1;
            }
            if (noteIndex < 0)
                noteIndex = 0;
            if (noteIndex > noteCount - 1)
                noteIndex = noteCount - 1;
        }
        if (flow === noteListType_1.ItemFlow.LeftToRight) {
            if (key === 'PageUp') {
                noteIndex -= (visibleItemCount - itemsPerLine);
            }
            else if (key === 'PageDown') {
                noteIndex += (visibleItemCount - itemsPerLine);
            }
            else if ((key === 'End' && ctrlKey) || (key === 'ArrowDown' && metaKey)) {
                noteIndex = noteCount - 1;
            }
            else if ((key === 'Home' && ctrlKey) || (key === 'ArrowUp' && metaKey)) {
                noteIndex = 0;
            }
            else if (key === 'ArrowUp' && !metaKey) {
                noteIndex -= itemsPerLine;
            }
            else if (key === 'ArrowDown' && !metaKey) {
                noteIndex += itemsPerLine;
            }
            else if (key === 'ArrowLeft' && !metaKey) {
                noteIndex -= 1;
            }
            else if (key === 'ArrowRight' && !metaKey) {
                noteIndex += 1;
            }
            if (noteIndex < 0)
                noteIndex = 0;
            if (noteIndex > noteCount - 1)
                noteIndex = noteCount - 1;
        }
        return noteIndex;
    }, [noteCount, flow, itemsPerLine]);
    const onKeyDown = (0, react_1.useCallback)(async (event) => {
        var _a;
        const noteIds = selectedNoteIds;
        const key = event.key;
        if (['ArrowDown', 'ArrowUp', 'ArrowLeft', 'ArrowRight'].includes(key) && event.altKey) {
            if (flow === noteListType_1.ItemFlow.TopToBottom) {
                await moveNote(key === 'ArrowDown' ? 1 : -1, 1);
            }
            else {
                if (key === 'ArrowRight') {
                    await moveNote(1, 1);
                }
                else if (key === 'ArrowLeft') {
                    await moveNote(-1, 1);
                }
                else if (key === 'ArrowUp') {
                    await moveNote(-1, itemsPerLine);
                }
                else if (key === 'ArrowDown') {
                    await moveNote(1, itemsPerLine);
                }
            }
            event.preventDefault();
        }
        else if (notes.length > 0 && (key === 'ArrowDown' || key === 'ArrowUp' || key === 'ArrowLeft' || key === 'ArrowRight' || key === 'PageDown' || key === 'PageUp' || key === 'End' || key === 'Home')) {
            const noteId = activeNoteId !== null && activeNoteId !== void 0 ? activeNoteId : (_a = notes[0]) === null || _a === void 0 ? void 0 : _a.id;
            let noteIndex = BaseModel_1.default.modelIndexById(notes, noteId);
            noteIndex = scrollNoteIndex(visibleItemCount, key, event.ctrlKey, event.metaKey, noteIndex);
            const newSelectedNote = notes[noteIndex];
            if (event.shiftKey) {
                if (selectedNoteIds.includes(newSelectedNote.id)) {
                    dispatch({
                        type: 'NOTE_SELECT_REMOVE',
                        id: noteId,
                    });
                }
                dispatch({
                    type: 'NOTE_SELECT_ADD',
                    id: newSelectedNote.id,
                });
            }
            else {
                dispatch({
                    type: 'NOTE_SELECT',
                    id: newSelectedNote.id,
                });
            }
            makeItemIndexVisible(noteIndex);
            focusNote(newSelectedNote.id);
            event.preventDefault();
        }
        if (noteIds.length) {
            if (key === 'Delete' && event.shiftKey || (key === 'Backspace' && event.metaKey && event.altKey)) {
                event.preventDefault();
                if (CommandService_1.default.instance().isEnabled('permanentlyDeleteNote')) {
                    void CommandService_1.default.instance().execute('permanentlyDeleteNote', noteIds);
                }
            }
            else if (key === 'Delete' || (key === 'Backspace' && event.metaKey)) {
                event.preventDefault();
                if (CommandService_1.default.instance().isEnabled('deleteNote')) {
                    void CommandService_1.default.instance().execute('deleteNote', noteIds);
                }
            }
        }
        if (noteIds.length && key === ' ') {
            event.preventDefault();
            const selectedNotes = BaseModel_1.default.modelsByIds(notes, noteIds);
            const todos = selectedNotes.filter(n => !!n.is_todo);
            if (!todos.length)
                return;
            for (let i = 0; i < todos.length; i++) {
                const toggledTodo = Note_1.default.toggleTodoCompleted(todos[i]);
                await Note_1.default.save(toggledTodo);
            }
            dispatch({ type: 'NOTE_SORT' });
            focusNote(todos[0].id);
            const wasCompleted = !!todos[0].todo_completed;
            (0, announceForAccessibility_1.default)(!wasCompleted ? (0, locale_1._)('Complete') : (0, locale_1._)('Incomplete'));
        }
        // Check for isDefaultPrevented to allow plugins to call .preventDefault
        if (key === 'Enter' && !event.isDefaultPrevented()) {
            event.preventDefault();
            if (event.shiftKey) {
                void CommandService_1.default.instance().execute('focusElement', 'sideBar');
            }
            else {
                void CommandService_1.default.instance().execute('focusElement', 'noteTitle');
            }
        }
        if (key.toUpperCase() === 'A' && (event.ctrlKey || event.metaKey)) {
            event.preventDefault();
            dispatch({
                type: 'NOTE_SELECT_ALL',
            });
        }
    }, [moveNote, focusNote, visibleItemCount, scrollNoteIndex, makeItemIndexVisible, notes, selectedNoteIds, activeNoteId, dispatch, flow, itemsPerLine]);
    return onKeyDown;
};
exports.default = useOnKeyDown;
//# sourceMappingURL=useOnKeyDown.js.map