"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const react_1 = require("react");
const noteListType_1 = require("@joplin/lib/services/plugins/api/noteListType");
const useRootElement_1 = require("./utils/useRootElement");
const useItemElement_1 = require("./utils/useItemElement");
const useItemEventHandlers_1 = require("./utils/useItemEventHandlers");
const Note_1 = require("@joplin/lib/models/Note");
const useRenderedNote_1 = require("./utils/useRenderedNote");
const getNoteElementIdFromJoplinId_1 = require("./utils/getNoteElementIdFromJoplinId");
const NoteListItem = (props, ref) => {
    const noteId = props.note.id;
    const elementId = (0, getNoteElementIdFromJoplinId_1.default)(noteId);
    const onInputChange = (0, react_1.useCallback)(async (event) => {
        const getValue = (element) => {
            if (element.type === 'checkbox')
                return element.checked;
            if (element.type === 'text')
                return element.value;
            throw new Error(`Unsupported element: ${element.type}`);
        };
        const changeEvent = {
            noteId: noteId,
            elementId: event.currentTarget.getAttribute('data-id'),
            value: getValue(event.currentTarget),
        };
        if (changeEvent.elementId === 'todo-checkbox') {
            await Note_1.default.save({
                id: changeEvent.noteId,
                todo_completed: changeEvent.value ? Date.now() : 0,
            }, { userSideValidation: true });
            props.dispatch({ type: 'NOTE_SORT' });
        }
        else {
            if (props.onChange)
                await props.onChange(changeEvent);
        }
    }, [props.onChange, noteId, props.dispatch]);
    const rootElement = (0, useRootElement_1.default)(elementId);
    const renderedNote = (0, useRenderedNote_1.default)(props.note, props.isSelected, props.isWatched, props.listRenderer, props.highlightedWords, props.index, props.columns);
    const itemElement = (0, useItemElement_1.default)(rootElement, noteId, renderedNote ? renderedNote.html : '', props.focusVisible, props.style, props.itemSize, props.onClick, props.flow);
    (0, useItemEventHandlers_1.default)(rootElement, itemElement, onInputChange, null);
    const className = (0, react_1.useMemo)(() => {
        return [
            'note-list-item-wrapper',
            // This is not used by the app, but kept here because it may be used
            // by users for custom CSS.
            (props.index + 1) % 2 === 0 ? 'even' : 'odd',
            props.isProvisional && '-provisional',
        ].filter(e => !!e).join(' ');
    }, [props.index, props.isProvisional]);
    const isActiveDragItem = props.dragIndex === props.index;
    const isLastActiveDragItem = props.index === props.noteCount - 1 && props.dragIndex >= props.noteCount;
    const dragCursorStyle = (0, react_1.useMemo)(() => {
        if (props.flow === noteListType_1.ItemFlow.TopToBottom) {
            let dragItemPosition = '';
            if (isActiveDragItem) {
                dragItemPosition = 'top';
            }
            else if (isLastActiveDragItem) {
                dragItemPosition = 'bottom';
            }
            const output = {
                width: props.itemSize.width,
                display: dragItemPosition ? 'block' : 'none',
                left: 0,
            };
            if (dragItemPosition === 'top') {
                output.top = 0;
            }
            else {
                output.bottom = 0;
            }
            return output;
        }
        if (props.flow === noteListType_1.ItemFlow.LeftToRight) {
            let dragItemPosition = '';
            if (isActiveDragItem) {
                dragItemPosition = 'left';
            }
            else if (isLastActiveDragItem) {
                dragItemPosition = 'right';
            }
            const output = {
                height: props.itemSize.height,
                display: dragItemPosition ? 'block' : 'none',
                top: 0,
            };
            if (dragItemPosition === 'left') {
                output.left = 0;
            }
            else {
                output.right = 0;
            }
            return output;
        }
        throw new Error('Unreachable');
    }, [isActiveDragItem, isLastActiveDragItem, props.flow, props.itemSize]);
    return React.createElement("div", { id: elementId, ref: ref, draggable: true, tabIndex: props.tabIndex, className: className, "data-id": noteId, style: { height: props.itemSize.height }, onContextMenu: props.onContextMenu, onDragStart: props.onDragStart, onDragOver: props.onDragOver, "aria-selected": props.isSelected, "aria-posinset": 1 + props.index, "aria-setsize": props.noteCount, role: 'option' },
        React.createElement("div", { className: "dragcursor", style: dragCursorStyle }));
};
exports.default = (0, react_1.memo)((0, react_1.forwardRef)(NoteListItem));
//# sourceMappingURL=NoteListItem.js.map