"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const prepareViewProps_1 = require("./prepareViewProps");
const Note_1 = require("@joplin/lib/models/Note");
const test_utils_1 = require("@joplin/lib/testing/test-utils");
// Same as `prepareViewProps` but with default arguments to make testing code simpler.
const prepare = async (dependencies, note, itemSize = { width: 100, height: 20 }, selected = false, noteTitleHtml = '', noteIsWatched = false, noteTags = [], folder = null, itemIndex = 0) => {
    return (0, prepareViewProps_1.default)(dependencies, note, itemSize, selected, noteTitleHtml, noteIsWatched, noteTags, folder, itemIndex);
};
describe('prepareViewProps', () => {
    beforeEach(async () => {
        await (0, test_utils_1.setupDatabaseAndSynchronizer)(1);
        await (0, test_utils_1.switchClient)(1);
    });
    it('should prepare note properties', async () => {
        const note = await Note_1.default.save({ title: 'test' });
        expect(await prepare(['note.title', 'note.user_updated_time'], note)).toEqual({
            note: {
                title: 'test',
                user_updated_time: note.user_updated_time,
            },
        });
        expect(await prepare(['item.size.height'], note)).toEqual({
            item: {
                size: {
                    height: 20,
                },
            },
        });
        expect(await prepare(['item.selected'], note)).toEqual({
            item: {
                selected: false,
            },
        });
        expect(await prepare(['item.selected'], note, {}, true)).toEqual({
            item: {
                selected: true,
            },
        });
        expect(await prepare(['note.titleHtml'], note, {}, false, '<b>test</b>')).toEqual({
            note: {
                titleHtml: '<b>test</b>',
            },
        });
        expect(await prepare(['note.isWatched'], note, {}, false, '', true)).toEqual({
            note: {
                isWatched: true,
            },
        });
        expect(await prepare(['note.isWatched'], note, {}, false, '', false)).toEqual({
            note: {
                isWatched: false,
            },
        });
        expect(await prepare(['item.index'], note, {}, false, '', false, [], null, 5)).toEqual({
            item: {
                index: 5,
            },
        });
        expect(await prepare(['note.tags'], note, {}, false, '', false, [{ id: '1', title: 'one' }])).toEqual({
            note: {
                tags: [{ id: '1', title: 'one' }],
            },
        });
    });
});
//# sourceMappingURL=prepareViewProps.test.js.map