"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const locale_1 = require("@joplin/lib/locale");
const theme_1 = require("@joplin/lib/theme");
const time_1 = require("@joplin/lib/time");
const DialogButtonRow_1 = require("./DialogButtonRow");
const Note_1 = require("@joplin/lib/models/Note");
const bridge_1 = require("../services/bridge");
const shim_1 = require("@joplin/lib/shim");
const focusHandler_1 = require("@joplin/lib/utils/focusHandler");
const Dialog_1 = require("./Dialog");
const Datetime = require('react-datetime').default;
const { clipboard } = require('electron');
const formatcoords = require('formatcoords');
class NotePropertiesDialog extends React.Component {
    constructor(props) {
        super(props);
        this.revisionsLink_click = this.revisionsLink_click.bind(this);
        this.buttonRow_click = this.buttonRow_click.bind(this);
        this.okButton = React.createRef();
        this.state = {
            formNote: null,
            editedKey: null,
            editedValue: null,
        };
        this.keyToLabel_ = {
            id: (0, locale_1._)('ID'),
            user_created_time: (0, locale_1._)('Created'),
            user_updated_time: (0, locale_1._)('Updated'),
            deleted_time: (0, locale_1._)('Deleted'),
            todo_completed: (0, locale_1._)('Completed'),
            location: (0, locale_1._)('Location'),
            source_url: (0, locale_1._)('URL'),
            revisionsLink: (0, locale_1._)('Note History'),
            markup_language: (0, locale_1._)('Markup'),
        };
    }
    componentDidMount() {
        void this.loadNote(this.props.noteId);
    }
    componentDidUpdate() {
        if (this.state.editedKey === null) {
            if (this.okButton.current)
                (0, focusHandler_1.focus)('NotePropertiesDialog::componentDidUpdate', this.okButton.current);
        }
    }
    async loadNote(noteId) {
        if (!noteId) {
            this.setState({ formNote: null });
        }
        else {
            const note = await Note_1.default.load(noteId);
            const formNote = this.noteToFormNote(note);
            this.setState({ formNote: formNote });
        }
    }
    isReadOnly() {
        return this.state.formNote && !!this.state.formNote.deleted_time;
    }
    latLongFromLocation(location) {
        // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
        const o = {};
        const l = location.split(',');
        if (l.length === 2) {
            o.latitude = l[0].trim();
            o.longitude = l[1].trim();
        }
        else {
            o.latitude = '';
            o.longitude = '';
        }
        return o;
    }
    noteToFormNote(note) {
        const formNote = {
            id: note.id,
            user_updated_time: time_1.default.formatMsToLocal(note.user_updated_time),
            user_created_time: time_1.default.formatMsToLocal(note.user_created_time),
            source_url: note.source_url,
            location: '',
            revisionsLink: note.id,
            markup_language: Note_1.default.markupLanguageToLabel(note.markup_language),
            deleted_time: note.deleted_time ? time_1.default.formatMsToLocal(note.deleted_time) : '',
        };
        if (note.todo_completed) {
            formNote.todo_completed = time_1.default.formatMsToLocal(note.todo_completed);
        }
        if (Number(note.latitude) || Number(note.longitude)) {
            formNote.location = `${note.latitude}, ${note.longitude}`;
        }
        return formNote;
    }
    formNoteToNote(formNote) {
        const note = Object.assign({ id: formNote.id }, this.latLongFromLocation(formNote.location));
        note.user_created_time = time_1.default.formatLocalToMs(formNote.user_created_time);
        note.user_updated_time = time_1.default.formatLocalToMs(formNote.user_updated_time);
        if (formNote.todo_completed) {
            note.todo_completed = time_1.default.formatLocalToMs(formNote.todo_completed);
        }
        note.source_url = formNote.source_url;
        return note;
    }
    styles(themeId) {
        const styleKey = themeId;
        if (styleKey === this.styleKey_)
            return this.styles_;
        const theme = (0, theme_1.themeStyle)(themeId);
        this.styles_ = {};
        this.styleKey_ = styleKey;
        this.styles_.button = {
            minWidth: theme.buttonMinWidth,
            minHeight: theme.buttonMinHeight,
            marginLeft: 5,
            color: theme.color,
            backgroundColor: theme.backgroundColor,
            border: '1px solid',
            borderColor: theme.dividerColor,
        };
        this.styles_.editPropertyButton = {
            color: theme.color,
            textDecoration: 'none',
            backgroundColor: theme.backgroundColor,
            padding: '.14em',
            display: 'inline-flex',
            alignItems: 'center',
            justifyContent: 'center',
            marginLeft: '0.5em',
        };
        this.styles_.input = {
            display: 'inline-block',
            color: theme.color,
            backgroundColor: theme.backgroundColor,
            border: '1px solid',
            borderColor: theme.dividerColor,
        };
        return this.styles_;
    }
    async closeDialog(applyChanges) {
        if (applyChanges) {
            await this.saveProperty();
            const note = this.formNoteToNote(this.state.formNote);
            note.updated_time = Date.now();
            await Note_1.default.save(note, { autoTimestamp: false });
        }
        else {
            await this.cancelProperty();
        }
        if (this.props.onClose) {
            this.props.onClose();
        }
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    buttonRow_click(event) {
        void this.closeDialog(event.buttonName === 'ok');
    }
    revisionsLink_click() {
        void this.closeDialog(false);
        if (this.props.onRevisionLinkClick)
            this.props.onRevisionLinkClick();
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    editPropertyButtonClick(key, initialValue) {
        this.setState({
            editedKey: key,
            editedValue: initialValue,
        });
        shim_1.default.setTimeout(() => {
            // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
            if (this.refs.editField.openCalendar) {
                // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
                this.refs.editField.openCalendar();
            }
            else {
                // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
                (0, focusHandler_1.focus)('NotePropertiesDialog::editPropertyButtonClick', this.refs.editField);
            }
        }, 100);
    }
    async saveProperty() {
        if (!this.state.editedKey)
            return null;
        // eslint-disable-next-line @typescript-eslint/ban-types -- Old code before rule was applied
        return new Promise((resolve) => {
            const newFormNote = Object.assign({}, this.state.formNote);
            if (this.state.editedKey.indexOf('_time') >= 0) {
                const dt = time_1.default.anythingToDateTime(this.state.editedValue, new Date());
                // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
                newFormNote[this.state.editedKey] = time_1.default.formatMsToLocal(dt.getTime());
            }
            else {
                // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
                newFormNote[this.state.editedKey] = this.state.editedValue;
            }
            this.setState({
                formNote: newFormNote,
                editedKey: null,
                editedValue: null,
            }, () => {
                resolve();
            });
        });
    }
    async cancelProperty() {
        // eslint-disable-next-line @typescript-eslint/ban-types -- Old code before rule was applied
        return new Promise((resolve) => {
            if (this.okButton.current)
                (0, focusHandler_1.focus)('NotePropertiesDialog::focus', this.okButton.current);
            this.setState({
                editedKey: null,
                editedValue: null,
            }, () => {
                resolve();
            });
        });
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    createNoteField(key, value) {
        const styles = this.styles(this.props.themeId);
        const theme = (0, theme_1.themeStyle)(this.props.themeId);
        const labelText = this.formatLabel(key);
        const labelComp = React.createElement("label", { role: 'rowheader', style: Object.assign(Object.assign({}, theme.textStyle), theme.controlBoxLabel) }, labelText);
        let controlComp = null;
        let editComp = null;
        let editCompHandler = null;
        let editCompIcon = null;
        let editComDescription = null;
        // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
        const onKeyDown = (event) => {
            if (event.keyCode === 13) {
                void this.saveProperty();
            }
            else if (event.keyCode === 27) {
                void this.cancelProperty();
            }
        };
        if (this.state.editedKey === key) {
            if (key.indexOf('_time') >= 0) {
                controlComp = (React.createElement(Datetime, { ref: "editField", initialValue: value, dateFormat: time_1.default.dateFormat(), timeFormat: time_1.default.timeFormat(), inputProps: {
                        // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
                        onKeyDown: (event) => onKeyDown(event),
                        style: styles.input,
                    }, 
                    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
                    onChange: (momentObject) => {
                        this.setState({ editedValue: momentObject });
                    } }));
                editCompHandler = () => {
                    void this.saveProperty();
                };
                editCompIcon = 'fa-save';
                editComDescription = (0, locale_1._)('Save changes');
            }
            else {
                controlComp = (React.createElement("input", { defaultValue: value, type: "text", ref: "editField", onChange: event => {
                        this.setState({ editedValue: event.target.value });
                    }, onKeyDown: event => onKeyDown(event), style: styles.input }));
            }
        }
        else {
            let displayedValue = value;
            if (key === 'location') {
                try {
                    const dms = formatcoords(value);
                    displayedValue = dms.format('DD MM ss X', { latLonSeparator: ', ', decimalPlaces: 2 });
                }
                catch (error) {
                    displayedValue = '';
                }
            }
            if (['source_url', 'location'].indexOf(key) >= 0) {
                let url = '';
                if (key === 'source_url')
                    url = value;
                if (key === 'location') {
                    const ll = this.latLongFromLocation(value);
                    url = Note_1.default.geoLocationUrlFromLatLong(ll.latitude, ll.longitude);
                }
                const urlStyle = Object.assign(Object.assign({}, theme.urlStyle), { maxWidth: '180px', overflow: 'hidden', whiteSpace: 'nowrap', textOverflow: 'ellipsis' });
                controlComp = (React.createElement("a", { href: "#", onClick: () => (0, bridge_1.default)().openExternal(url), style: urlStyle }, displayedValue));
            }
            else if (key === 'revisionsLink') {
                controlComp = (React.createElement("a", { href: "#", onClick: this.revisionsLink_click, style: theme.urlStyle }, (0, locale_1._)('Previous versions of this note')));
            }
            else {
                controlComp = React.createElement("div", { style: Object.assign(Object.assign({}, theme.textStyle), theme.controlBoxValue) }, displayedValue);
            }
            if (['id', 'revisionsLink', 'markup_language'].indexOf(key) < 0) {
                editCompHandler = () => {
                    this.editPropertyButtonClick(key, value);
                };
                editCompIcon = 'fa-edit';
                editComDescription = (0, locale_1._)('Edit');
            }
            // Add the copy icon and the 'copy on click' event
            if (key === 'id') {
                editCompIcon = 'fa-copy';
                editCompHandler = () => clipboard.writeText(value);
                editComDescription = (0, locale_1._)('Copy');
            }
        }
        if (editCompHandler && !this.isReadOnly()) {
            editComp = (React.createElement("a", { href: "#", onClick: editCompHandler, style: styles.editPropertyButton, "aria-label": editComDescription, title: editComDescription },
                React.createElement("i", { className: `fas ${editCompIcon}`, "aria-hidden": "true" })));
        }
        return (React.createElement("div", { role: 'row', key: key, style: theme.controlBox, className: "note-property-box" },
            labelComp,
            React.createElement("span", { role: 'cell' },
                controlComp,
                " ",
                editComp)));
    }
    formatLabel(key) {
        if (this.keyToLabel_[key])
            return this.keyToLabel_[key];
        return key;
    }
    formatValue(key, note) {
        if (key === 'location') {
            if (!Number(note.latitude) && !Number(note.longitude))
                return null;
            const dms = formatcoords(Number(note.latitude), Number(note.longitude));
            return dms.format('DDMMss', { decimalPlaces: 0 });
        }
        if (['user_updated_time', 'user_created_time', 'todo_completed'].indexOf(key) >= 0) {
            // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
            return time_1.default.formatMsToLocal(note[key]);
        }
        // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
        return note[key];
    }
    render() {
        const theme = (0, theme_1.themeStyle)(this.props.themeId);
        const formNote = this.state.formNote;
        const noteComps = [];
        if (formNote) {
            for (const key of Object.keys(formNote)) {
                if (key === 'deleted_time' && !formNote.deleted_time)
                    continue;
                // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
                const comp = this.createNoteField(key, formNote[key]);
                noteComps.push(comp);
            }
        }
        return (React.createElement(Dialog_1.default, { onCancel: this.props.onClose },
            React.createElement("div", { style: theme.dialogTitle, id: 'note-properties-dialog-title' }, (0, locale_1._)('Note properties')),
            React.createElement("div", { role: 'table', "aria-labelledby": 'note-properties-dialog-title' }, noteComps),
            React.createElement(DialogButtonRow_1.default, { themeId: this.props.themeId, okButtonShow: !this.isReadOnly(), okButtonRef: this.okButton, onClick: this.buttonRow_click })));
    }
}
exports.default = NotePropertiesDialog;
//# sourceMappingURL=NotePropertiesDialog.js.map