"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const ButtonBar_1 = require("./ConfigScreen/ButtonBar");
const locale_1 = require("@joplin/lib/locale");
const { connect } = require('react-redux');
const registry_1 = require("@joplin/lib/registry");
const Setting_1 = require("@joplin/lib/models/Setting");
const bridge = require('@electron/remote').require('./bridge').default;
const { themeStyle } = require('@joplin/lib/theme');
const { OneDriveApiNodeUtils } = require('@joplin/lib/onedrive-api-node-utils.js');
// eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
class OneDriveLoginScreenComponent extends React.Component {
    constructor(props) {
        super(props);
        this.state = {
            authLog: [],
        };
    }
    async componentDidMount() {
        // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
        const log = (s) => {
            // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
            this.setState((state) => {
                const authLog = state.authLog.slice();
                authLog.push({ key: (Date.now() + Math.random()).toString(), text: s });
                return { authLog: authLog };
            });
        };
        const syncTargetId = Setting_1.default.value('sync.target');
        const syncTarget = registry_1.reg.syncTarget(syncTargetId);
        const oneDriveApiUtils = new OneDriveApiNodeUtils(syncTarget.api());
        const auth = await oneDriveApiUtils.oauthDance({
            // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
            log: (s) => log(s),
        });
        Setting_1.default.setValue(`sync.${syncTargetId}.auth`, auth ? JSON.stringify(auth) : null);
        syncTarget.api().setAuth(auth);
        if (!auth) {
            log((0, locale_1._)('Authentication was not completed (did not receive an authentication token).'));
        }
        else {
            void registry_1.reg.scheduleSync(0);
        }
    }
    startUrl() {
        return registry_1.reg.syncTarget().api().authCodeUrl(this.redirectUrl());
    }
    redirectUrl() {
        return registry_1.reg.syncTarget().api().nativeClientRedirectUrl();
    }
    render() {
        const theme = themeStyle(this.props.themeId);
        const logComps = [];
        for (const l of this.state.authLog) {
            if (l.text.indexOf('http:') === 0) {
                logComps.push(React.createElement("a", { key: l.key, style: theme.urlStyle, href: "#", onClick: () => { bridge().openExternal(l.text); } }, l.text));
            }
            else {
                logComps.push(React.createElement("p", { key: l.key, style: theme.textStyle }, l.text));
            }
        }
        return (React.createElement("div", { style: { display: 'flex', flexDirection: 'column', height: '100%', backgroundColor: theme.backgroundColor } },
            React.createElement("div", { style: { padding: theme.configScreenPadding, flex: 1, color: theme.color } }, logComps),
            React.createElement(ButtonBar_1.default, { onCancelClick: () => this.props.dispatch({ type: 'NAV_BACK' }) })));
    }
}
// eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
const mapStateToProps = (state) => {
    return {
        themeId: state.settings.theme,
    };
};
exports.default = connect(mapStateToProps)(OneDriveLoginScreenComponent);
//# sourceMappingURL=OneDriveLoginScreen.js.map