"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const locale_1 = require("@joplin/lib/locale");
const theme_1 = require("@joplin/lib/theme");
const time_1 = require("@joplin/lib/time");
const Datetime = require('react-datetime').default;
const creatable_1 = require("react-select/creatable");
const react_select_1 = require("react-select");
const animated_1 = require("react-select/animated");
const focusHandler_1 = require("@joplin/lib/utils/focusHandler");
const Dialog_1 = require("./Dialog");
// eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
class PromptDialog extends React.Component {
    constructor(props) {
        super(props);
        this.menuIsOpened_ = false;
        this.answerInput_ = React.createRef();
        this.select_menuOpen = this.select_menuOpen.bind(this);
        this.select_menuClose = this.select_menuClose.bind(this);
    }
    UNSAFE_componentWillMount() {
        this.setState({
            visible: false,
            answer: this.props.defaultValue ? this.props.defaultValue : '',
        });
        this.focusInput_ = true;
        this.menuIsOpened_ = false;
    }
    UNSAFE_componentWillReceiveProps(newProps) {
        if ('visible' in newProps && newProps.visible !== this.props.visible) {
            this.setState({ visible: newProps.visible });
            if (newProps.visible)
                this.focusInput_ = true;
        }
        if ('defaultValue' in newProps && newProps.defaultValue !== this.props.defaultValue) {
            this.setState({ answer: newProps.defaultValue });
        }
    }
    select_menuOpen() {
        this.menuIsOpened_ = true;
    }
    select_menuClose() {
        this.menuIsOpened_ = false;
    }
    componentDidUpdate() {
        if (this.focusInput_ && this.answerInput_.current)
            (0, focusHandler_1.focus)('PromptDialog::componentDidUpdate', this.answerInput_.current);
        this.focusInput_ = false;
    }
    styles(themeId, visible) {
        const styleKey = `${themeId}_${visible}`;
        if (styleKey === this.styleKey_)
            return this.styles_;
        const theme = (0, theme_1.themeStyle)(themeId);
        this.styleKey_ = styleKey;
        this.styles_ = {};
        this.styles_.button = {
            minWidth: theme.buttonMinWidth,
            minHeight: theme.buttonMinHeight,
            marginLeft: 5,
            color: theme.color,
            backgroundColor: theme.backgroundColor,
            border: '1px solid',
            borderColor: theme.dividerColor,
        };
        this.styles_.label = {
            marginRight: 5,
            fontSize: theme.fontSize,
            color: theme.color,
            fontFamily: theme.fontFamily,
            verticalAlign: 'middle',
        };
        this.styles_.input = {
            width: 'calc(0.5 * var(--prompt-width))',
            maxWidth: 400,
            color: theme.color,
            backgroundColor: theme.backgroundColor,
            border: '1px solid',
            borderColor: theme.dividerColor,
        };
        this.styles_.select = {
            // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
            control: (provided) => {
                return Object.assign(Object.assign({}, provided), { minWidth: 'calc(var(--prompt-width) * 0.2)', maxWidth: 'calc(var(--prompt-width) * 0.5)', fontFamily: theme.fontFamily });
            },
            // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
            input: (provided) => {
                return Object.assign(Object.assign({}, provided), { minWidth: '20px', color: theme.color });
            },
            // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
            menu: (provided) => {
                return Object.assign(Object.assign({}, provided), { color: theme.color, fontFamily: theme.fontFamily, backgroundColor: theme.backgroundColor });
            },
            // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
            option: (provided, state) => {
                return Object.assign(Object.assign({}, provided), { color: theme.color, fontFamily: theme.fontFamily, paddingLeft: `${10 + (state.data.indentDepth || 0) * 20}px` });
            },
            // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
            multiValueLabel: (provided) => {
                return Object.assign(Object.assign({}, provided), { fontFamily: theme.fontFamily });
            },
            // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
            multiValueRemove: (provided) => {
                return Object.assign(Object.assign({}, provided), { color: theme.color });
            },
        };
        // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
        this.styles_.selectTheme = (tagTheme) => {
            return Object.assign(Object.assign({}, tagTheme), { borderRadius: 2, colors: Object.assign(Object.assign({}, tagTheme.colors), { primary: theme.raisedBackgroundColor, primary25: theme.raisedBackgroundColor, neutral0: theme.backgroundColor, neutral5: theme.backgroundColor, neutral10: theme.raisedBackgroundColor, neutral20: theme.raisedBackgroundColor, neutral30: theme.raisedBackgroundColor, neutral40: theme.color, neutral50: theme.color, neutral60: theme.color, neutral70: theme.color, neutral80: theme.color, neutral90: theme.color, danger: theme.backgroundColor, dangerLight: theme.colorError2 }) });
        };
        this.styles_.desc = Object.assign(Object.assign({}, theme.textStyle), { marginTop: 10 });
        return this.styles_;
    }
    render() {
        if (!this.state.visible)
            return null;
        const theme = (0, theme_1.themeStyle)(this.props.themeId);
        const buttonTypes = this.props.buttons ? this.props.buttons : ['ok', 'cancel'];
        const styles = this.styles(this.props.themeId, this.state.visible);
        const onClose = (accept, buttonType = null) => {
            if (this.props.onClose) {
                let outputAnswer = this.state.answer;
                if (this.props.inputType === 'datetime') {
                    // outputAnswer = anythingToDate(outputAnswer);
                    outputAnswer = time_1.default.anythingToDateTime(outputAnswer);
                }
                this.props.onClose(accept ? outputAnswer : null, buttonType);
            }
            this.setState({ visible: false, answer: '' });
        };
        // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
        const onChange = (event) => {
            this.setState({ answer: event.target.value });
        };
        // const anythingToDate = (o) => {
        // 	if (o && o.toDate) return o.toDate();
        // 	if (!o) return null;
        // 	let m = moment(o, time.dateTimeFormat());
        // 	if (m.isValid()) return m.toDate();
        // 	m = moment(o, time.dateFormat());
        // 	return m.isValid() ? m.toDate() : null;
        // }
        // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
        const onDateTimeChange = (momentObject) => {
            this.setState({ answer: momentObject });
        };
        // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
        const onSelectChange = (newValue) => {
            this.setState({ answer: newValue });
            this.focusInput_ = true;
        };
        // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
        const onKeyDown = (event) => {
            if (event.key === 'Enter') {
                // If the dropdown is open, we don't close the dialog - instead
                // the currently item will be selected. If it is closed however
                // we confirm the dialog.
                if ((this.props.inputType === 'tags' || this.props.inputType === 'dropdown') && this.menuIsOpened_) {
                    // Do nothing
                }
                else {
                    onClose(true);
                }
            }
            else if (event.key === 'Escape') {
                onClose(false);
            }
        };
        const descComp = this.props.description ? React.createElement("div", { style: styles.desc }, this.props.description) : null;
        let inputComp = null;
        if (this.props.inputType === 'datetime') {
            // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
            inputComp = React.createElement(Datetime, { className: "datetime-picker", value: this.state.answer, inputProps: { style: styles.input }, dateFormat: time_1.default.dateFormat(), timeFormat: time_1.default.timeFormat(), onChange: (momentObject) => onDateTimeChange(momentObject) });
        }
        else if (this.props.inputType === 'tags') {
            // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
            inputComp = React.createElement(creatable_1.default, { className: "tag-selector", onMenuOpen: this.select_menuOpen, onMenuClose: this.select_menuClose, styles: styles.select, theme: styles.selectTheme, ref: this.answerInput_, value: this.state.answer, placeholder: "", components: (0, animated_1.default)(), isMulti: true, isClearable: false, backspaceRemovesValue: true, options: this.props.autocomplete, onChange: onSelectChange, onKeyDown: (event) => onKeyDown(event) });
        }
        else if (this.props.inputType === 'dropdown') {
            // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
            inputComp = React.createElement(react_select_1.default, { className: "item-selector", onMenuOpen: this.select_menuOpen, onMenuClose: this.select_menuClose, styles: styles.select, theme: styles.selectTheme, ref: this.answerInput_, components: (0, animated_1.default)(), value: this.props.answer, defaultValue: this.props.defaultValue, isClearable: false, options: this.props.autocomplete, onChange: onSelectChange, onKeyDown: (event) => onKeyDown(event) });
        }
        else {
            inputComp = React.createElement("input", { style: styles.input, ref: this.answerInput_, value: this.state.answer, type: "text", onChange: event => onChange(event), onKeyDown: event => onKeyDown(event) });
        }
        const buttonComps = [];
        if (buttonTypes.indexOf('create') >= 0) {
            buttonComps.push(React.createElement("button", { key: "create", disabled: !this.state.answer, style: styles.button, onClick: () => onClose(true, 'create') }, (0, locale_1._)('Create')));
        }
        if (buttonTypes.indexOf('ok') >= 0) {
            buttonComps.push(React.createElement("button", { key: "ok", disabled: !this.state.answer, style: styles.button, onClick: () => onClose(true, 'ok') }, (0, locale_1._)('OK')));
        }
        if (buttonTypes.indexOf('cancel') >= 0) {
            buttonComps.push(React.createElement("button", { key: "cancel", style: styles.button, onClick: () => onClose(false, 'cancel') }, (0, locale_1._)('Cancel')));
        }
        if (buttonTypes.indexOf('clear') >= 0) {
            buttonComps.push(React.createElement("button", { key: "clear", style: styles.button, onClick: () => onClose(false, 'clear') }, (0, locale_1._)('Clear')));
        }
        return (React.createElement(Dialog_1.default, { className: 'prompt-dialog', contentStyle: styles.dialog },
            React.createElement("label", { style: styles.label }, this.props.label ? this.props.label : ''),
            React.createElement("div", { style: { display: 'inline-block', color: 'black', backgroundColor: theme.backgroundColor } },
                inputComp,
                descComp),
            React.createElement("div", { style: { textAlign: 'right', marginTop: 10 } }, buttonComps)));
    }
}
exports.default = PromptDialog;
//# sourceMappingURL=PromptDialog.js.map