"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const react_1 = require("react");
const Button_1 = require("../Button/Button");
const movements_1 = require("./utils/movements");
const styled_components_1 = require("styled-components");
const StyledRoot = styled_components_1.default.div `
	display: flex;
	flex-direction: column;
	padding: 5px;
	background-color: ${props => props.theme.backgroundColor};
	border-radius: 5px;
`;
const ButtonRow = styled_components_1.default.div `
	display: flex;
	flex-direction: row;
	justify-content: center;
`;
const EmptyButton = (0, styled_components_1.default)(Button_1.default) `
	visibility: hidden;
`;
const ArrowButton = (0, styled_components_1.default)(Button_1.default) `
	opacity: ${props => props.disabled ? 0.2 : 1};
`;
function MoveButtons(props) {
    const onButtonClick = (0, react_1.useCallback)((direction) => {
        props.onClick({ direction, itemKey: props.itemKey });
    }, [props.onClick, props.itemKey]);
    function canMove(dir) {
        if (dir === movements_1.MoveDirection.Up)
            return props.canMoveUp;
        if (dir === movements_1.MoveDirection.Down)
            return props.canMoveDown;
        if (dir === movements_1.MoveDirection.Left)
            return props.canMoveLeft;
        if (dir === movements_1.MoveDirection.Right)
            return props.canMoveRight;
        throw new Error('Unreachable');
    }
    function renderButton(dir) {
        return React.createElement(ArrowButton, { disabled: !canMove(dir), level: Button_1.ButtonLevel.Primary, iconName: `fas fa-arrow-${dir}`, onClick: () => onButtonClick(dir) });
    }
    return (React.createElement(StyledRoot, null,
        React.createElement(ButtonRow, null, renderButton(movements_1.MoveDirection.Up)),
        React.createElement(ButtonRow, null,
            renderButton(movements_1.MoveDirection.Left),
            React.createElement(EmptyButton, { iconName: "fas fa-arrow-down", disabled: true }),
            renderButton(movements_1.MoveDirection.Right)),
        React.createElement(ButtonRow, null, renderButton(movements_1.MoveDirection.Down))));
}
exports.default = MoveButtons;
//# sourceMappingURL=MoveButtons.js.map