"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const react_1 = require("react");
const useWindowResizeEvent_1 = require("./utils/useWindowResizeEvent");
const setLayoutItemProps_1 = require("./utils/setLayoutItemProps");
const useLayoutItemSizes_1 = require("./utils/useLayoutItemSizes");
const validateLayout_1 = require("./utils/validateLayout");
const movements_1 = require("./utils/movements");
const MoveButtons_1 = require("./MoveButtons");
const style_1 = require("./utils/style");
const re_resizable_1 = require("re-resizable");
const EventEmitter = require('events');
function itemVisible(item, moveMode) {
    if (moveMode)
        return true;
    if (item.children && !item.children.length)
        return false;
    return item.visible !== false;
}
// eslint-disable-next-line @typescript-eslint/ban-types, @typescript-eslint/no-explicit-any -- Old code before rule was applied, Old code before rule was applied
function renderContainer(item, parent, sizes, resizedItemMaxSize, onResizeStart, onResize, onResizeStop, children, isLastChild, moveMode) {
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    const style = {
        display: itemVisible(item, moveMode) ? 'flex' : 'none',
        flexDirection: item.direction,
    };
    const size = (0, useLayoutItemSizes_1.itemSize)(item, parent, sizes, true);
    const className = `resizableLayoutItem rli-${item.key}`;
    if (item.resizableRight || item.resizableBottom) {
        const enable = {
            top: false,
            right: !!item.resizableRight && !isLastChild,
            bottom: !!item.resizableBottom && !isLastChild,
            left: false,
            topRight: false,
            bottomRight: false,
            bottomLeft: false,
            topLeft: false,
        };
        return (React.createElement(re_resizable_1.Resizable, { key: item.key, className: className, style: style, size: size, 
            // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
            onResizeStart: onResizeStart, 
            // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
            onResize: onResize, 
            // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
            onResizeStop: onResizeStop, enable: enable, minWidth: 'minWidth' in item ? item.minWidth : useLayoutItemSizes_1.itemMinWidth, minHeight: 'minHeight' in item ? item.minHeight : useLayoutItemSizes_1.itemMinHeight, maxWidth: resizedItemMaxSize === null || resizedItemMaxSize === void 0 ? void 0 : resizedItemMaxSize.width, maxHeight: resizedItemMaxSize === null || resizedItemMaxSize === void 0 ? void 0 : resizedItemMaxSize.height }, children));
    }
    else {
        return (React.createElement("div", { key: item.key, className: className, style: Object.assign(Object.assign({}, style), size) }, children));
    }
}
function ResizableLayout(props) {
    const eventEmitter = (0, react_1.useRef)(new EventEmitter());
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    const [resizedItem, setResizedItem] = (0, react_1.useState)(null);
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    function renderItemWrapper(comp, item, parent, size, moveMode) {
        const moveOverlay = moveMode ? (React.createElement(style_1.StyledMoveOverlay, null,
            React.createElement(MoveButtons_1.default, { itemKey: item.key, onClick: props.onMoveButtonClick, canMoveLeft: (0, movements_1.canMove)(movements_1.MoveDirection.Left, item, parent), canMoveRight: (0, movements_1.canMove)(movements_1.MoveDirection.Right, item, parent), canMoveUp: (0, movements_1.canMove)(movements_1.MoveDirection.Up, item, parent), canMoveDown: (0, movements_1.canMove)(movements_1.MoveDirection.Down, item, parent) }))) : null;
        return (React.createElement(style_1.StyledWrapperRoot, { key: item.key, size: size },
            moveOverlay,
            comp));
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    function renderLayoutItem(item, parent, sizes, isVisible, isLastChild) {
        function onResizeStart() {
            setResizedItem({
                key: item.key,
                initialWidth: sizes[item.key].width,
                initialHeight: sizes[item.key].height,
                maxSize: (0, useLayoutItemSizes_1.calculateMaxSizeAvailableForItem)(item, parent, sizes),
            });
        }
        // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
        function onResize(_event, direction, _refToElement, delta) {
            const newWidth = Math.max(useLayoutItemSizes_1.itemMinWidth, resizedItem.initialWidth + delta.width);
            const newHeight = Math.max(useLayoutItemSizes_1.itemMinHeight, resizedItem.initialHeight + delta.height);
            // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
            const newSize = {};
            if (item.width)
                newSize.width = item.width;
            if (item.height)
                newSize.height = item.height;
            if (direction === 'bottom') {
                newSize.height = newHeight;
            }
            else {
                newSize.width = newWidth;
            }
            const newLayout = (0, setLayoutItemProps_1.default)(props.layout, item.key, newSize);
            props.onResize({ layout: newLayout });
            eventEmitter.current.emit('resize');
        }
        // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
        function onResizeStop(_event, _direction, _refToElement, delta) {
            onResize(_event, _direction, _refToElement, delta);
            setResizedItem(null);
        }
        const resizedItemMaxSize = resizedItem && item.key === resizedItem.key ? resizedItem.maxSize : null;
        if (!item.children) {
            const size = (0, useLayoutItemSizes_1.itemSize)(item, parent, sizes, false);
            const comp = props.renderItem(item.key, {
                item: item,
                eventEmitter: eventEmitter.current,
                size: size,
                visible: isVisible,
            });
            const wrapper = renderItemWrapper(comp, item, parent, size, props.moveMode);
            return renderContainer(item, parent, sizes, resizedItemMaxSize, onResizeStart, onResize, onResizeStop, [wrapper], isLastChild, props.moveMode);
        }
        else {
            const childrenComponents = [];
            for (let i = 0; i < item.children.length; i++) {
                const child = item.children[i];
                childrenComponents.push(renderLayoutItem(child, item, sizes, isVisible && itemVisible(child, props.moveMode), i === item.children.length - 1));
            }
            return renderContainer(item, parent, sizes, resizedItemMaxSize, onResizeStart, onResize, onResizeStop, childrenComponents, isLastChild, props.moveMode);
        }
    }
    (0, react_1.useEffect)(() => {
        (0, validateLayout_1.default)(props.layout);
    }, [props.layout]);
    (0, useWindowResizeEvent_1.default)(eventEmitter);
    const sizes = (0, useLayoutItemSizes_1.default)(props.layout, props.moveMode);
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    function renderMoveModeBox(rootComp) {
        return (React.createElement(style_1.MoveModeRootWrapper, null,
            React.createElement(style_1.MoveModeRootMessage, null, props.moveModeMessage),
            rootComp));
    }
    const rootComp = renderLayoutItem(props.layout, null, sizes, itemVisible(props.layout, props.moveMode), true);
    if (props.moveMode) {
        return renderMoveModeBox(rootComp);
    }
    else {
        return rootComp;
    }
}
exports.default = ResizableLayout;
//# sourceMappingURL=ResizableLayout.js.map