"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const ButtonBar_1 = require("./ConfigScreen/ButtonBar");
const locale_1 = require("@joplin/lib/locale");
const { connect } = require('react-redux');
const { themeStyle } = require('@joplin/lib/theme');
const bridge_1 = require("../services/bridge");
const prettyBytes = require('pretty-bytes');
const Resource_1 = require("@joplin/lib/models/Resource");
const ResourceTableComp = (props) => {
    const theme = themeStyle(props.themeId);
    const titleCellStyle = Object.assign(Object.assign({}, theme.textStyle), { textOverflow: 'ellipsis', overflowX: 'hidden', maxWidth: 1, width: '100%', whiteSpace: 'nowrap' });
    const cellStyle = Object.assign(Object.assign({}, theme.textStyle), { whiteSpace: 'nowrap', color: theme.colorFaded, width: 1 });
    const headerStyle = Object.assign(Object.assign({}, theme.textStyle), { whiteSpace: 'nowrap', width: 1, fontWeight: 'bold' });
    const filteredResources = props.resources.filter((resource) => { var _a; return !props.filter || ((_a = resource.title) === null || _a === void 0 ? void 0 : _a.includes(props.filter)) || resource.id.includes(props.filter); });
    const renderSortableHeader = (title, order) => {
        const sortedDescending = props.sorting.order === order && props.sorting.type === 'desc';
        const sortButtonLabel = sortedDescending ? (0, locale_1._)('Sort "%s" in ascending order', title) : (0, locale_1._)('Sort "%s" in descending order', title);
        const reverseSortButton = (React.createElement("a", { href: "#", style: { color: theme.urlColor }, onClick: () => props.onToggleSorting(order), "aria-label": sortButtonLabel, title: sortButtonLabel, role: 'button' }, sortedDescending ? '▾' : '▴'));
        return React.createElement("th", { key: `header-${title}`, style: headerStyle },
            title,
            " ",
            reverseSortButton);
    };
    return (React.createElement("table", { style: { width: '100%' } },
        React.createElement("thead", null,
            React.createElement("tr", null,
                renderSortableHeader((0, locale_1._)('Title'), 'name'),
                renderSortableHeader((0, locale_1._)('Size'), 'size'),
                React.createElement("th", { style: headerStyle }, (0, locale_1._)('ID')),
                React.createElement("th", { style: headerStyle }, (0, locale_1._)('Action')))),
        React.createElement("tbody", null, filteredResources.map((resource, index) => React.createElement("tr", { key: index },
            React.createElement("td", { style: titleCellStyle, className: "titleCell" },
                React.createElement("a", { style: { color: theme.urlColor }, href: "#", onClick: () => props.onResourceClick(resource) }, resource.title || `(${(0, locale_1._)('Untitled')})`)),
            React.createElement("td", { style: cellStyle, className: "dataCell" }, prettyBytes(resource.size)),
            React.createElement("td", { style: cellStyle, className: "dataCell" }, resource.id),
            React.createElement("td", { style: cellStyle, className: "dataCell" },
                React.createElement("button", { style: theme.buttonStyle, onClick: () => props.onResourceDelete(resource) }, (0, locale_1._)('Delete'))))))));
};
const getSortingOrderColumn = (s) => {
    switch (s) {
        case 'name': return 'title';
        case 'size': return 'size';
    }
};
const getNextSortingOrderType = (s) => {
    if (s === 'asc') {
        return 'desc';
    }
    else {
        return 'asc';
    }
};
const defaultMaxResources = 10000;
const searchMaxResources = 1000;
class ResourceScreenComponent extends React.Component {
    constructor(props) {
        super(props);
        this.reloadResourcesCounter = 0;
        this.onFilterUpdate = (updateEvent) => {
            this.setState({ filter: updateEvent.target.value });
        };
        this.state = {
            resources: undefined,
            filter: '',
            sorting: {
                type: 'asc',
                order: 'name',
            },
            isLoading: false,
        };
    }
    get maxResources() {
        // Use a smaller maximum when searching for performance -- results update
        // when the search input changes.
        if (this.state.filter) {
            return searchMaxResources;
        }
        else {
            return defaultMaxResources;
        }
    }
    async reloadResources() {
        this.setState({ isLoading: true });
        this.reloadResourcesCounter++;
        const currentCounterValue = this.reloadResourcesCounter;
        let searchOptions = {};
        if (this.state.filter) {
            const search = `%${this.state.filter}%`;
            searchOptions = {
                where: 'id LIKE ? OR title LIKE ?',
                whereParams: [search, search],
            };
        }
        const resources = await Resource_1.default.all(Object.assign({ order: [{
                    by: getSortingOrderColumn(this.state.sorting.order),
                    dir: this.state.sorting.type,
                    caseInsensitive: true,
                }], limit: this.maxResources, fields: ['title', 'id', 'size', 'file_extension'] }, searchOptions));
        const cancelled = currentCounterValue !== this.reloadResourcesCounter;
        if (cancelled)
            return;
        this.setState({ resources, isLoading: false });
    }
    componentDidMount() {
        void this.reloadResources();
    }
    componentDidUpdate(_prevProps, prevState) {
        if (prevState.sorting !== this.state.sorting || prevState.filter !== this.state.filter) {
            void this.reloadResources();
        }
    }
    onResourceDelete(resource) {
        const ok = (0, bridge_1.default)().showConfirmMessageBox((0, locale_1._)('Delete attachment "%s"?', resource.title), {
            buttons: [(0, locale_1._)('Delete'), (0, locale_1._)('Cancel')],
            defaultId: 1,
        });
        if (!ok) {
            return;
        }
        Resource_1.default.delete(resource.id, { sourceDescription: 'ResourceScreen' })
            // eslint-disable-next-line promise/prefer-await-to-then -- Old code before rule was applied
            .catch((error) => {
            console.error(error);
            (0, bridge_1.default)().showErrorMessageBox(error.message);
        })
            // eslint-disable-next-line promise/prefer-await-to-then -- Old code before rule was applied
            .finally(() => {
            void this.reloadResources();
        });
    }
    openResource(resource) {
        const resourcePath = Resource_1.default.fullPath(resource);
        const ok = (0, bridge_1.default)().openItem(resourcePath);
        if (!ok) {
            (0, bridge_1.default)().showErrorMessageBox(`This file could not be opened: ${resourcePath}`);
        }
    }
    onToggleSortOrder(sortOrder) {
        let newSorting = Object.assign({}, this.state.sorting);
        if (sortOrder === this.state.sorting.order) {
            newSorting.type = getNextSortingOrderType(newSorting.type);
        }
        else {
            newSorting = {
                order: sortOrder,
                type: 'desc',
            };
        }
        this.setState({ sorting: newSorting });
    }
    render() {
        const style = this.props.style;
        const theme = themeStyle(this.props.themeId);
        // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
        const rootStyle = Object.assign(Object.assign({}, style), { overflowY: 'scroll', color: theme.color, padding: 20, boxSizing: 'border-box', flex: 1 });
        // rootStyle.height = style.height - 35; // Minus the header height
        delete rootStyle.height;
        delete rootStyle.width;
        const containerHeight = style.height;
        return (React.createElement("div", { style: Object.assign(Object.assign({}, theme.containerStyle), { fontFamily: theme.fontFamily, height: containerHeight, display: 'flex', flexDirection: 'column' }) },
            React.createElement("div", { style: rootStyle },
                React.createElement("div", { style: Object.assign(Object.assign({}, theme.notificationBox), { marginBottom: 10 }) }, (0, locale_1._)('This is an advanced tool to show the attachments that are linked to your notes. Please be careful when deleting one of them as they cannot be restored afterwards.')),
                React.createElement("div", { style: { float: 'right' } },
                    React.createElement("input", { style: theme.inputStyle, type: "search", value: this.state.filter, onChange: this.onFilterUpdate, placeholder: (0, locale_1._)('Search...') })),
                this.state.isLoading && React.createElement("div", null, (0, locale_1._)('Please wait...')),
                !this.state.isLoading && React.createElement("div", null,
                    !this.state.resources && React.createElement("div", null, (0, locale_1._)('No resources!')),
                    this.state.resources && this.state.resources.length === this.maxResources &&
                        React.createElement("div", null, (0, locale_1._)('Warning: not all resources shown for performance reasons (limit: %s).', this.maxResources)),
                    this.state.resources && React.createElement(ResourceTableComp, { themeId: this.props.themeId, style: style, resources: this.state.resources, sorting: this.state.sorting, filter: this.state.filter, onToggleSorting: (order) => this.onToggleSortOrder(order), onResourceClick: (resource) => this.openResource(resource), onResourceDelete: (resource) => this.onResourceDelete(resource) }))),
            React.createElement(ButtonBar_1.default, { onCancelClick: () => this.props.dispatch({ type: 'NAV_BACK' }) })));
    }
}
// eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
const mapStateToProps = (state) => ({
    themeId: state.settings.theme,
});
const ResourceScreen = connect(mapStateToProps)(ResourceScreenComponent);
module.exports = { ResourceScreen };
//# sourceMappingURL=ResourceScreen.js.map