"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const app_1 = require("../app");
const MainScreen_1 = require("./MainScreen");
const ConfigScreen_1 = require("./ConfigScreen/ConfigScreen");
const StatusScreen_1 = require("./StatusScreen/StatusScreen");
const OneDriveLoginScreen_1 = require("./OneDriveLoginScreen");
const DropboxLoginScreen_1 = require("./DropboxLoginScreen");
const ErrorBoundary_1 = require("./ErrorBoundary");
const theme_1 = require("@joplin/lib/theme");
const MenuBar_1 = require("./MenuBar");
const locale_1 = require("@joplin/lib/locale");
const React = require('react');
const { createRoot } = require('react-dom/client');
const { connect, Provider } = require('react-redux');
const Setting_1 = require("@joplin/lib/models/Setting");
const shim_1 = require("@joplin/lib/shim");
const ClipperServer_1 = require("@joplin/lib/ClipperServer");
const DialogTitle_1 = require("./DialogTitle");
const DialogButtonRow_1 = require("./DialogButtonRow");
const Dialog_1 = require("./Dialog");
const StyleSheetContainer_1 = require("./StyleSheets/StyleSheetContainer");
const ImportScreen_1 = require("./ImportScreen");
const { ResourceScreen } = require('./ResourceScreen.js');
const Navigator_1 = require("./Navigator");
const WelcomeUtils_1 = require("@joplin/lib/WelcomeUtils");
const JoplinCloudLoginScreen_1 = require("./JoplinCloudLoginScreen");
const InteropService_1 = require("@joplin/lib/services/interop/InteropService");
const WindowCommandsAndDialogs_1 = require("./WindowCommandsAndDialogs/WindowCommandsAndDialogs");
const reducer_1 = require("@joplin/lib/reducer");
const bridge_1 = require("../services/bridge");
const EditorWindow_1 = require("./NoteEditor/EditorWindow");
const { ThemeProvider, StyleSheetManager, createGlobalStyle } = require('styled-components');
const GlobalStyle = createGlobalStyle `
	* {
		box-sizing: border-box;
	}
`;
// eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
let wcsTimeoutId_ = null;
async function initialize() {
    (0, bridge_1.default)().activeWindow().on('resize', () => {
        if (wcsTimeoutId_)
            shim_1.default.clearTimeout(wcsTimeoutId_);
        wcsTimeoutId_ = shim_1.default.setTimeout(() => {
            store.dispatch({
                type: 'WINDOW_CONTENT_SIZE_SET',
                size: (0, bridge_1.default)().windowContentSize(),
            });
            wcsTimeoutId_ = null;
        }, 10);
    });
    // Need to dispatch this to make sure the components are
    // displayed at the right size. The windowContentSize is
    // also set in the store default state, but at that point
    // the window might not be at its final size.
    store.dispatch({
        type: 'WINDOW_CONTENT_SIZE_SET',
        size: (0, bridge_1.default)().windowContentSize(),
    });
    store.dispatch({
        type: 'EDITOR_CODE_VIEW_CHANGE',
        value: Setting_1.default.value('editor.codeView'),
    });
    store.dispatch({
        type: 'NOTE_VISIBLE_PANES_SET',
        panes: Setting_1.default.value('noteVisiblePanes'),
    });
    InteropService_1.default.instance().document = document;
    InteropService_1.default.instance().xmlSerializer = new XMLSerializer();
}
// eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
class RootComponent extends React.Component {
    async componentDidMount() {
        if (this.props.appState === 'starting') {
            this.props.dispatch({
                type: 'APP_STATE_SET',
                state: 'initializing',
            });
            await initialize();
            this.props.dispatch({
                type: 'APP_STATE_SET',
                state: 'ready',
            });
        }
        await WelcomeUtils_1.default.install(Setting_1.default.value('locale'), this.props.dispatch);
    }
    renderModalMessage(props) {
        if (!props)
            return null;
        const renderContent = () => {
            return (React.createElement("div", null,
                React.createElement(DialogTitle_1.default, { title: (0, locale_1._)('Confirmation') }),
                React.createElement("p", null, props.message),
                React.createElement(DialogButtonRow_1.default, { themeId: props.themeId, onClick: props.onClick, okButtonShow: false, cancelButtonShow: false, customButtons: props.buttonSpecs })));
        };
        return React.createElement(Dialog_1.default, null, renderContent());
    }
    modalDialogProps() {
        if (!this.props.needApiAuth)
            return null;
        let message = '';
        const buttonSpecs = [];
        let onClick = null;
        if (this.props.needApiAuth) {
            message = (0, locale_1._)('The Web Clipper needs your authorisation to access your data.');
            buttonSpecs.push({ name: 'ok', label: (0, locale_1._)('Grant authorisation') });
            buttonSpecs.push({ name: 'cancel', label: (0, locale_1._)('Reject') });
            onClick = (event) => {
                ClipperServer_1.default.instance().api.acceptAuthToken(event.buttonName === 'ok');
            };
        }
        else {
            return null;
        }
        return {
            themeId: this.props.themeId,
            buttonSpecs,
            message,
            onClick,
        };
    }
    renderSecondaryWindows() {
        return this.props.secondaryWindowStates.map((windowState) => {
            return React.createElement(EditorWindow_1.default, { key: `new-window-note-${windowState.windowId}`, windowId: windowState.windowId, newWindow: true });
        });
    }
    render() {
        const navigatorStyle = {
            width: this.props.size.width / this.props.zoomFactor,
            height: this.props.size.height / this.props.zoomFactor,
        };
        const theme = (0, theme_1.themeStyle)(this.props.themeId);
        const screens = {
            Main: { screen: MainScreen_1.default },
            OneDriveLogin: { screen: OneDriveLoginScreen_1.default, title: () => (0, locale_1._)('OneDrive Login') },
            DropboxLogin: { screen: DropboxLoginScreen_1.default, title: () => (0, locale_1._)('Dropbox Login') },
            JoplinCloudLogin: { screen: JoplinCloudLoginScreen_1.default, title: () => (0, locale_1._)('Joplin Cloud Login') },
            Import: { screen: ImportScreen_1.default, title: () => (0, locale_1._)('Import') },
            Config: { screen: ConfigScreen_1.default, title: () => (0, locale_1._)('Options') },
            Resources: { screen: ResourceScreen, title: () => (0, locale_1._)('Note attachments') },
            Status: { screen: StatusScreen_1.default, title: () => (0, locale_1._)('Synchronisation Status') },
        };
        return (React.createElement(StyleSheetManager, { disableVendorPrefixes: true },
            React.createElement(ThemeProvider, { theme: theme },
                React.createElement(StyleSheetContainer_1.default, null),
                React.createElement(MenuBar_1.default, null),
                React.createElement(GlobalStyle, null),
                React.createElement(WindowCommandsAndDialogs_1.default, { windowId: reducer_1.defaultWindowId }),
                React.createElement(Navigator_1.default, { style: navigatorStyle, screens: screens, className: `profile-${this.props.profileConfigCurrentProfileId}` }),
                this.renderSecondaryWindows(),
                this.renderModalMessage(this.modalDialogProps()))));
    }
}
const mapStateToProps = (state) => {
    return {
        size: state.windowContentSize,
        zoomFactor: state.settings.windowContentZoomFactor / 100,
        appState: state.appState,
        themeId: state.settings.theme,
        needApiAuth: state.needApiAuth,
        dialogs: state.dialogs,
        profileConfigCurrentProfileId: state.profileConfig.currentProfileId,
        secondaryWindowStates: reducer_1.stateUtils.secondaryWindowStates(state),
    };
};
const Root = connect(mapStateToProps)(RootComponent);
const store = (0, app_1.default)().store();
const root = createRoot(document.getElementById('react-root'));
root.render(React.createElement(Provider, { store: store },
    React.createElement(ErrorBoundary_1.default, null,
        React.createElement(Root, null))));
//# sourceMappingURL=Root.js.map