"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Root = void 0;
const React = require("react");
const react_1 = require("react");
const SearchInput_1 = require("../lib/SearchInput/SearchInput");
const Setting_1 = require("@joplin/lib/models/Setting");
const reducer_1 = require("@joplin/lib/reducer");
const BaseModel_1 = require("@joplin/lib/BaseModel");
const uuid_1 = require("@joplin/lib/uuid");
const { connect } = require('react-redux');
const Note_1 = require("@joplin/lib/models/Note");
const focusHandler_1 = require("@joplin/lib/utils/focusHandler");
const debounce = require('debounce');
const styled = require('styled-components').default;
exports.Root = styled.div `
	position: relative;
	display: flex;
	width: 100%;
	min-width: 30px;
`;
function SearchBar(props) {
    const [query, setQuery] = (0, react_1.useState)('');
    const [searchStarted, setSearchStarted] = (0, react_1.useState)(false);
    const searchId = (0, react_1.useRef)(uuid_1.default.create());
    (0, react_1.useEffect)(() => {
        // eslint-disable-next-line @typescript-eslint/ban-types -- Old code before rule was applied
        function search(searchId, query, dispatch) {
            dispatch({
                type: 'SEARCH_UPDATE',
                search: {
                    id: searchId,
                    title: query,
                    query_pattern: query,
                    query_folder_id: null,
                    type_: BaseModel_1.default.TYPE_SEARCH,
                },
            });
            dispatch({
                type: 'SEARCH_SELECT',
                id: searchId,
            });
        }
        const debouncedSearch = debounce(search, 500);
        if (searchStarted)
            debouncedSearch(searchId.current, query, props.dispatch);
        return () => {
            debouncedSearch.clear();
        };
        // eslint-disable-next-line @seiyab/react-hooks/exhaustive-deps -- Old code before rule was applied
    }, [query, searchStarted]);
    const onExitSearch = (0, react_1.useCallback)(async (navigateAway = true) => {
        setQuery('');
        setSearchStarted(false);
        if (navigateAway) {
            const note = props.selectedNoteId ? await Note_1.default.load(props.selectedNoteId) : null;
            if (note) {
                props.dispatch({
                    type: 'FOLDER_AND_NOTE_SELECT',
                    folderId: note.parent_id,
                    noteId: note.id,
                });
            }
            else {
                const folderId = Setting_1.default.value('activeFolderId');
                if (folderId) {
                    props.dispatch({
                        type: 'FOLDER_SELECT',
                        id: folderId,
                    });
                }
            }
        }
        // eslint-disable-next-line @seiyab/react-hooks/exhaustive-deps -- Old code before rule was applied
    }, [props.selectedNoteId]);
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    function onChange(event) {
        if (event.value.length === 0) {
            // Revert to previous state if query string becomes empty
            void onExitSearch();
            return;
        }
        setSearchStarted(true);
        setQuery(event.value);
    }
    function onFocus() {
        props.dispatch({
            type: 'FOCUS_SET',
            field: 'globalSearch',
        });
    }
    function onBlur() {
        // Do it after a delay so that the "Clear" button
        // can be clicked on (otherwise the field loses focus
        // and is resized before the click event has been processed)
        setTimeout(() => {
            props.dispatch({
                type: 'FOCUS_CLEAR',
                field: 'globalSearch',
            });
        }, 300);
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    const onKeyDown = (0, react_1.useCallback)((event) => {
        if (event.key === 'Escape') {
            // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
            if (document.activeElement)
                (0, focusHandler_1.blur)('SearchBar::onKeyDown', document.activeElement);
            void onExitSearch();
        }
    }, [onExitSearch]);
    const onSearchButtonClick = (0, react_1.useCallback)(() => {
        if (props.isFocused || searchStarted) {
            void onExitSearch();
        }
        else {
            setSearchStarted(true);
            (0, focusHandler_1.focus)('SearchBar::onSearchButtonClick', props.inputRef.current);
            props.dispatch({
                type: 'FOCUS_SET',
                field: 'globalSearch',
            });
        }
        // eslint-disable-next-line @seiyab/react-hooks/exhaustive-deps -- Old code before rule was applied
    }, [onExitSearch, props.isFocused, searchStarted]);
    (0, react_1.useEffect)(() => {
        if (props.notesParentType !== 'Search') {
            void onExitSearch(false);
        }
    }, [props.notesParentType, onExitSearch]);
    // When the searchbar is remounted, exit the search if it was previously open
    // or else other buttons stay hidden (e.g. when opening Layout Editor and closing it)
    // https://github.com/laurent22/joplin/issues/5953
    (0, react_1.useEffect)(() => {
        if (props.notesParentType === 'Search' || props.isFocused) {
            if (props.isFocused) {
                props.dispatch({
                    type: 'FOCUS_CLEAR',
                    field: 'globalSearch',
                });
            }
            void onExitSearch(true);
        }
        // eslint-disable-next-line @seiyab/react-hooks/exhaustive-deps -- Old code before rule was applied
    }, []);
    return (React.createElement(exports.Root, { className: "search-bar" },
        React.createElement(SearchInput_1.default, { inputRef: props.inputRef, value: query, onChange: onChange, onFocus: onFocus, onBlur: onBlur, onKeyDown: onKeyDown, onSearchButtonClick: onSearchButtonClick, searchStarted: searchStarted })));
}
const mapStateToProps = (state) => {
    return {
        notesParentType: state.notesParentType,
        selectedNoteId: reducer_1.stateUtils.selectedNoteId(state),
        isFocused: state.focusedField === 'globalSearch',
    };
};
exports.default = connect(mapStateToProps)(SearchBar);
//# sourceMappingURL=SearchBar.js.map