"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Dialog_1 = require("../Dialog");
const DialogButtonRow_1 = require("../DialogButtonRow");
const DialogTitle_1 = require("../DialogTitle");
const locale_1 = require("@joplin/lib/locale");
const react_1 = require("react");
const Folder_1 = require("@joplin/lib/models/Folder");
const ShareService_1 = require("@joplin/lib/services/share/ShareService");
const styled_components_1 = require("styled-components");
const StyledFormLabel_1 = require("../style/StyledFormLabel");
const StyledInput_1 = require("../style/StyledInput");
const Button_1 = require("../Button/Button");
const Logger_1 = require("@joplin/utils/Logger");
const StyledMessage_1 = require("../style/StyledMessage");
const reducer_1 = require("@joplin/lib/services/share/reducer");
const react_redux_1 = require("react-redux");
const registry_1 = require("@joplin/lib/registry");
const useAsyncEffect_1 = require("@joplin/lib/hooks/useAsyncEffect");
const Dropdown_1 = require("../Dropdown/Dropdown");
const shim_1 = require("@joplin/lib/shim");
const logger = Logger_1.default.create('ShareFolderDialog');
const StyledRoot = styled_components_1.default.div `
	min-width: 500px;
`;
const StyledFolder = styled_components_1.default.div `
	border: 1px solid ${(props) => props.theme.dividerColor};
	padding: 0.5em;
	margin-bottom: 1em;
	display: flex;
	align-items: center;
`;
const StyledRecipientControls = styled_components_1.default.div `
	display: flex;
	flex-direction: row;
`;
const StyledRecipientInput = (0, styled_components_1.default)(StyledInput_1.default) `
	width: 100%;
	margin-right: 10px;
`;
const StyledAddRecipient = styled_components_1.default.div `
	margin-bottom: 1em;
`;
// eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
const StyledRecipient = (0, styled_components_1.default)(StyledMessage_1.default) `
	display: flex;
	flex-direction: row;
	padding: .6em 1em;
	background-color: ${props => props.index % 2 === 0 ? props.theme.backgroundColor : props.theme.oddBackgroundColor};
	align-items: center;
`;
const StyledRecipientName = styled_components_1.default.div `
	display: flex;
	flex: 1;
`;
const StyledRecipientStatusIcon = styled_components_1.default.i `
	margin-right: .6em;
`;
const StyledRecipients = styled_components_1.default.div `
	margin-bottom: 10px;
`;
const StyledRecipientList = styled_components_1.default.div `
	border: 1px solid ${(props) => props.theme.dividerColor};
	border-radius: 3px;
	height: 300px;
	overflow-x: hidden;
	overflow-y: scroll;
`;
const StyledError = (0, styled_components_1.default)(StyledMessage_1.default) `
	word-break: break-all;
	margin-bottom: 1em;
`;
// eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
const StyledShareState = (0, styled_components_1.default)(StyledMessage_1.default) `
	word-break: break-all;
	margin-bottom: 1em;
`;
const StyledIcon = styled_components_1.default.i `
	margin-right: 8px;
`;
var ShareState;
(function (ShareState) {
    ShareState[ShareState["Idle"] = 0] = "Idle";
    ShareState[ShareState["Synchronizing"] = 1] = "Synchronizing";
    ShareState[ShareState["Creating"] = 2] = "Creating";
})(ShareState || (ShareState = {}));
function ShareFolderDialog(props) {
    const permissionOptions = {
        'can_read': (0, locale_1._)('Can view'),
        'can_read_and_write': (0, locale_1._)('Can view and edit'),
    };
    const [folder, setFolder] = (0, react_1.useState)(null);
    const [recipientEmail, setRecipientEmail] = (0, react_1.useState)('');
    const [recipientPermissions, setRecipientPermissions] = (0, react_1.useState)('can_read_and_write');
    const [latestError, setLatestError] = (0, react_1.useState)(null);
    const [share, setShare] = (0, react_1.useState)(null);
    const [shareUsers, setShareUsers] = (0, react_1.useState)([]);
    const [shareState, setShareState] = (0, react_1.useState)(ShareState.Idle);
    const [customButtons, setCustomButtons] = (0, react_1.useState)([]);
    const [recipientsBeingUpdated, setRecipientsBeingUpdated] = (0, react_1.useState)({});
    async function synchronize(event = null) {
        setShareState(ShareState.Synchronizing);
        await registry_1.reg.waitForSyncFinishedThenSync();
        if (event && event.cancelled)
            return;
        setShareState(ShareState.Idle);
    }
    (0, useAsyncEffect_1.default)(async (event) => {
        const f = await Folder_1.default.load(props.folderId);
        if (event.cancelled)
            return;
        setFolder(f);
    }, [props.folderId]);
    (0, react_1.useEffect)(() => {
        void ShareService_1.default.instance().refreshShares();
    }, []);
    (0, useAsyncEffect_1.default)(async (event) => {
        await synchronize(event);
    }, []);
    (0, react_1.useEffect)(() => {
        const s = props.shares.find(s => s.folder_id === props.folderId);
        setShare(s);
    }, [props.shares, props.folderId]);
    (0, react_1.useEffect)(() => {
        if (!share)
            return;
        void ShareService_1.default.instance().refreshShareUsers(share.id);
    }, [share]);
    (0, react_1.useEffect)(() => {
        setCustomButtons(share ? [{
                name: 'unshare',
                label: (0, locale_1._)('Unshare'),
            }] : []);
    }, [share]);
    (0, react_1.useEffect)(() => {
        if (!share)
            return;
        const sus = props.shareUsers[share.id];
        if (!sus)
            return;
        setShareUsers(sus);
    }, [share, props.shareUsers]);
    (0, react_1.useEffect)(() => {
        void ShareService_1.default.instance().refreshShares();
    }, [props.folderId]);
    const permissionsFromString = (p) => {
        return {
            can_read: 1,
            can_write: p === 'can_read_and_write' ? 1 : 0,
        };
    };
    const shareRecipient_click = (0, react_1.useCallback)(async () => {
        setShareState(ShareState.Creating);
        setLatestError(null);
        let errorSet = false;
        // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
        const handleError = (error) => {
            if (!errorSet)
                setLatestError(error);
            errorSet = true;
            logger.error(error);
        };
        // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
        const defer = (error) => {
            if (error)
                handleError(error);
            setShareState(ShareState.Idle);
        };
        let share = null;
        try {
            share = await ShareService_1.default.instance().shareFolder(props.folderId);
        }
        catch (error) {
            return defer(error);
        }
        try {
            await ShareService_1.default.instance().addShareRecipient(share.id, share.master_key_id, recipientEmail, permissionsFromString(recipientPermissions));
        }
        catch (error) {
            // Handle the error but continue the process because we need to at
            // least refresh the shares since one has been created above.
            handleError(error);
        }
        try {
            await Promise.all([
                ShareService_1.default.instance().refreshShares(),
                ShareService_1.default.instance().refreshShareUsers(share.id),
            ]);
            setRecipientEmail('');
            await synchronize();
        }
        catch (error) {
            handleError(error);
        }
        finally {
            defer(null);
        }
    }, [recipientPermissions, props.folderId, recipientEmail]);
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    function recipientEmail_change(event) {
        setRecipientEmail(event.target.value);
    }
    async function recipient_delete(event) {
        if (!await shim_1.default.showConfirmationDialog((0, locale_1._)('Delete this invitation? The recipient will no longer have access to this shared notebook.')))
            return;
        try {
            await ShareService_1.default.instance().deleteShareRecipient(event.shareUserId);
        }
        catch (error) {
            logger.error(error);
            await shim_1.default.showErrorDialog((0, locale_1._)('The recipient could not be removed from the list. Please try again.\n\nThe error was: "%s"', error.message));
        }
        await ShareService_1.default.instance().refreshShareUsers(share.id);
    }
    function renderFolder() {
        return (React.createElement(StyledFolder, null,
            React.createElement(StyledIcon, { className: "icon-notebooks" }),
            folder ? folder.title : '...'));
    }
    const recipientPermissions_change = (0, react_1.useCallback)((event) => {
        setRecipientPermissions(event.value);
    }, []);
    function renderAddRecipient() {
        const disabled = shareState !== ShareState.Idle;
        const dropdown = !props.canUseSharePermissions ? null : React.createElement(Dropdown_1.Dropdown, { className: "permission-dropdown", options: permissionOptions, value: recipientPermissions, onChange: recipientPermissions_change });
        return (React.createElement(StyledAddRecipient, null,
            React.createElement(StyledFormLabel_1.default, null, (0, locale_1._)('Add recipient:')),
            React.createElement(StyledRecipientControls, null,
                React.createElement(StyledRecipientInput, { disabled: disabled, type: "email", placeholder: "example@domain.com", value: recipientEmail, onChange: recipientEmail_change }),
                dropdown,
                React.createElement(Button_1.default, { size: Button_1.ButtonSize.Small, disabled: disabled, title: (0, locale_1._)('Share'), onClick: shareRecipient_click }))));
    }
    const recipient_permissionChange = (0, react_1.useCallback)(async (shareUserId, value) => {
        try {
            setRecipientsBeingUpdated(prev => {
                return Object.assign(Object.assign({}, prev), { [shareUserId]: true });
            });
            await ShareService_1.default.instance().setPermissions(share.id, shareUserId, permissionsFromString(value));
        }
        catch (error) {
            void shim_1.default.showErrorDialog(`Could not set permissions: ${error.message}`);
            logger.error(error);
        }
        finally {
            setRecipientsBeingUpdated(prev => {
                return Object.assign(Object.assign({}, prev), { [shareUserId]: false });
            });
        }
    }, [share]);
    function renderRecipient(index, shareUser) {
        const statusToIcon = {
            [reducer_1.ShareUserStatus.Waiting]: 'fas fa-question',
            [reducer_1.ShareUserStatus.Rejected]: 'fas fa-times',
            [reducer_1.ShareUserStatus.Accepted]: 'fas fa-check',
        };
        const statusToMessage = {
            [reducer_1.ShareUserStatus.Waiting]: (0, locale_1._)('Recipient has not yet accepted the invitation'),
            [reducer_1.ShareUserStatus.Rejected]: (0, locale_1._)('Recipient has rejected the invitation'),
            [reducer_1.ShareUserStatus.Accepted]: (0, locale_1._)('Recipient has accepted the invitation'),
        };
        const permission = shareUser.can_write ? 'can_read_and_write' : 'can_read';
        const enabled = !recipientsBeingUpdated[shareUser.id];
        const dropdown = !props.canUseSharePermissions ? null : React.createElement(Dropdown_1.Dropdown, { disabled: !enabled, className: "permission-dropdown", value: permission, options: permissionOptions, variant: Dropdown_1.DropdownVariant.NoBorder, onChange: event => recipient_permissionChange(shareUser.id, event.value) });
        return (React.createElement(StyledRecipient, { key: shareUser.user.email, index: index },
            React.createElement(StyledRecipientName, null, shareUser.user.email),
            dropdown,
            React.createElement(StyledRecipientStatusIcon, { title: statusToMessage[shareUser.status], className: statusToIcon[shareUser.status] }),
            React.createElement(Button_1.default, { disabled: !enabled, size: Button_1.ButtonSize.Small, iconName: "far fa-times-circle", onClick: () => recipient_delete({ shareUserId: shareUser.id }), tooltip: (0, locale_1._)('Remove %s from share', shareUser.user.email) })));
    }
    function renderRecipients() {
        const listItems = shareUsers.map((su, index) => renderRecipient(index, su));
        return (React.createElement(StyledRecipients, null,
            React.createElement(StyledFormLabel_1.default, null, (0, locale_1._)('Recipients:')),
            React.createElement(StyledRecipientList, null, listItems)));
    }
    function renderError() {
        if (!latestError)
            return null;
        return (React.createElement(StyledError, { type: "error" }, latestError.message));
    }
    function renderShareState() {
        if (shareState === ShareState.Idle)
            return null;
        const messages = {
            [ShareState.Synchronizing]: (0, locale_1._)('Synchronizing...'),
            [ShareState.Creating]: (0, locale_1._)('Sharing notebook...'),
        };
        const message = messages[shareState];
        if (!message)
            throw new Error(`Unsupported state: ${shareState}`);
        return (React.createElement(StyledShareState, null, message));
    }
    const renderInfo = () => {
        return (React.createElement("p", { className: "info-text -small" }, (0, locale_1._)('Please note that if it is a large notebook, it may take a few minutes for all the notes to show up on the recipient\'s device.')));
    };
    async function buttonRow_click(event) {
        if (event.buttonName === 'unshare') {
            if (!await shim_1.default.showConfirmationDialog((0, locale_1._)('Unshare this notebook? The recipients will no longer have access to its content.'))) {
                return;
            }
            await ShareService_1.default.instance().unshareFolder(props.folderId);
            void synchronize();
        }
        props.onClose();
    }
    function renderContent() {
        return (React.createElement(StyledRoot, { className: "share-folder-dialog" },
            React.createElement(DialogTitle_1.default, { title: (0, locale_1._)('Share Notebook') }),
            renderFolder(),
            renderAddRecipient(),
            renderShareState(),
            renderError(),
            renderRecipients(),
            renderInfo(),
            React.createElement(DialogButtonRow_1.default, { themeId: props.themeId, onClick: buttonRow_click, okButtonShow: false, cancelButtonLabel: (0, locale_1._)('Close'), customButtons: customButtons })));
    }
    return (React.createElement(Dialog_1.default, null, renderContent()));
}
const mapStateToProps = (state) => {
    return {
        shares: state.shareService.shares,
        shareUsers: state.shareService.shareUsers,
        canUseSharePermissions: state.settings['sync.target'] === 10 && state.settings['sync.10.canUseSharePermissions'],
    };
};
exports.default = (0, react_redux_1.connect)(mapStateToProps)(ShareFolderDialog);
//# sourceMappingURL=ShareFolderDialog.js.map