"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ShareNoteDialog = void 0;
const React = require("react");
const react_1 = require("react");
const JoplinServerApi_1 = require("@joplin/lib/JoplinServerApi");
const locale_1 = require("@joplin/lib/locale");
const Note_1 = require("@joplin/lib/models/Note");
const DialogButtonRow_1 = require("./DialogButtonRow");
const theme_1 = require("@joplin/lib/theme");
const registry_1 = require("@joplin/lib/registry");
const Dialog_1 = require("./Dialog");
const DialogTitle_1 = require("./DialogTitle");
const ShareService_1 = require("@joplin/lib/services/share/ShareService");
const Button_1 = require("./Button/Button");
const react_redux_1 = require("react-redux");
const syncInfoUtils_1 = require("@joplin/lib/services/synchronizer/syncInfoUtils");
const SyncTargetRegistry_1 = require("@joplin/lib/SyncTargetRegistry");
const shim_1 = require("@joplin/lib/shim");
const { clipboard } = require('electron');
function styles_(props) {
    return (0, theme_1.buildStyle)('ShareNoteDialog', props.themeId, theme => {
        return {
            root: {
                minWidth: 500,
            },
            noteList: {
                marginBottom: 10,
            },
            note: {
                flex: 1,
                flexDirection: 'row',
                display: 'flex',
                alignItems: 'center',
                border: '1px solid',
                borderColor: theme.dividerColor,
                padding: '0.5em',
                marginBottom: 5,
            },
            noteTitle: Object.assign(Object.assign({}, theme.textStyle), { flex: 1, display: 'flex', color: theme.color }),
            noteRemoveButton: {
                background: 'none',
                border: 'none',
            },
            noteRemoveButtonIcon: {
                color: theme.color,
                fontSize: '1.4em',
            },
            copyShareLinkButton: Object.assign(Object.assign({}, theme.buttonStyle), { marginBottom: 10 }),
        };
    });
}
function ShareNoteDialog(props) {
    const [notes, setNotes] = (0, react_1.useState)([]);
    const [recursiveShare, setRecursiveShare] = (0, react_1.useState)(false);
    const [sharesState, setSharesState] = (0, react_1.useState)('unknown');
    const syncTargetInfo = (0, react_1.useMemo)(() => SyncTargetRegistry_1.default.infoById(props.syncTargetId), [props.syncTargetId]);
    const noteCount = notes.length;
    const theme = (0, theme_1.themeStyle)(props.themeId);
    const styles = styles_(props);
    (0, react_1.useEffect)(() => {
        void ShareService_1.default.instance().refreshShares();
    }, []);
    (0, react_1.useEffect)(() => {
        async function fetchNotes() {
            const result = [];
            for (const noteId of props.noteIds) {
                result.push(await Note_1.default.load(noteId));
            }
            setNotes(result);
        }
        void fetchNotes();
    }, [props.noteIds]);
    const buttonRow_click = () => {
        props.onClose();
    };
    const copyLinksToClipboard = (shares) => {
        const links = [];
        for (const share of shares)
            links.push(ShareService_1.default.instance().shareUrl(ShareService_1.default.instance().userId, share));
        clipboard.writeText(links.join('\n'));
    };
    const shareLinkButton_click = (0, react_1.useCallback)(async () => {
        const service = ShareService_1.default.instance();
        let hasSynced = false;
        let tryToSync = false;
        while (true) {
            try {
                if (tryToSync) {
                    setSharesState('synchronizing');
                    await registry_1.reg.waitForSyncFinishedThenSync();
                    tryToSync = false;
                    hasSynced = true;
                }
                setSharesState('creating');
                const newShares = [];
                for (const note of notes) {
                    const share = await service.shareNote(note.id, recursiveShare);
                    newShares.push(share);
                }
                setSharesState('synchronizing');
                await registry_1.reg.waitForSyncFinishedThenSync();
                setSharesState('creating');
                copyLinksToClipboard(newShares);
                setSharesState('created');
                await ShareService_1.default.instance().refreshShares();
            }
            catch (error) {
                if (error.code === 404 && !hasSynced) {
                    registry_1.reg.logger().info('ShareNoteDialog: Note does not exist on server - trying to sync it.', error);
                    tryToSync = true;
                    continue;
                }
                registry_1.reg.logger().error('ShareNoteDialog: Cannot publish note:', error);
                setSharesState('idle');
                void shim_1.default.showErrorDialog(JoplinServerApi_1.default.connectionErrorMessage(error));
            }
            break;
        }
    }, [recursiveShare, notes]);
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    const unshareNoteButton_click = async (event) => {
        await ShareService_1.default.instance().unshareNote(event.noteId);
        await ShareService_1.default.instance().refreshShares();
    };
    const renderNote = (note) => {
        const unshareButton = !props.shares.find(s => s.note_id === note.id) ? null : (React.createElement(Button_1.default, { tooltip: (0, locale_1._)('Unpublish note'), iconName: "fas fa-share-alt", onClick: () => unshareNoteButton_click({ noteId: note.id }) }));
        return (React.createElement("div", { key: note.id, style: styles.note },
            React.createElement("span", { style: styles.noteTitle }, note.title),
            unshareButton));
    };
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    const renderNoteList = (notes) => {
        const noteComps = [];
        for (const note of notes) {
            noteComps.push(renderNote(note));
        }
        return React.createElement("div", { style: styles.noteList }, noteComps);
    };
    const statusMessage = (sharesState) => {
        if (sharesState === 'synchronizing')
            return (0, locale_1._)('Synchronising...');
        if (sharesState === 'creating')
            return (0, locale_1._n)('Generating link...', 'Generating links...', noteCount);
        if (sharesState === 'created')
            return (0, locale_1._n)('Link has been copied to clipboard!', 'Links have been copied to clipboard!', noteCount);
        return '';
    };
    function renderEncryptionWarningMessage() {
        if (!(0, syncInfoUtils_1.getEncryptionEnabled)())
            return null;
        return React.createElement("div", { style: theme.textStyle },
            (0, locale_1._)('Note: When a note is shared, it will no longer be encrypted on the server.'),
            React.createElement("hr", null));
    }
    const onRecursiveShareChange = (0, react_1.useCallback)(() => {
        setRecursiveShare(v => !v);
    }, []);
    const renderRecursiveShareCheckbox = () => {
        if (!syncTargetInfo.supportsRecursiveLinkedNotes)
            return null;
        return (React.createElement("div", { className: "form-input-group form-input-group-checkbox" },
            React.createElement("input", { id: "recursiveShare", name: "recursiveShare", type: "checkbox", checked: !!recursiveShare, onChange: onRecursiveShareChange }),
            " ",
            React.createElement("label", { htmlFor: "recursiveShare" }, (0, locale_1._)('Also publish linked notes'))));
    };
    const renderContent = () => {
        return (React.createElement("div", { style: styles.root, className: "form" },
            React.createElement(DialogTitle_1.default, { title: (0, locale_1._)('Publish Notes') }),
            renderNoteList(notes),
            renderRecursiveShareCheckbox(),
            React.createElement("button", { disabled: ['creating', 'synchronizing'].indexOf(sharesState) >= 0, style: styles.copyShareLinkButton, onClick: shareLinkButton_click }, (0, locale_1._n)('Copy Shareable Link', 'Copy Shareable Links', noteCount)),
            React.createElement("div", { style: theme.textStyle }, statusMessage(sharesState)),
            renderEncryptionWarningMessage(),
            React.createElement(DialogButtonRow_1.default, { themeId: props.themeId, onClick: buttonRow_click, okButtonShow: false, cancelButtonLabel: (0, locale_1._)('Close') })));
    };
    return (React.createElement(Dialog_1.default, null, renderContent()));
}
exports.ShareNoteDialog = ShareNoteDialog;
const mapStateToProps = (state) => {
    return {
        shares: state.shareService.shares.filter(s => !!s.note_id),
        syncTargetId: state.settings['sync.target'],
    };
};
exports.default = (0, react_redux_1.connect)(mapStateToProps)(ShareNoteDialog);
//# sourceMappingURL=ShareNoteDialog.js.map