"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const styles_1 = require("./styles");
const Button_1 = require("../Button/Button");
const CommandService_1 = require("@joplin/lib/services/CommandService");
const Synchronizer_1 = require("@joplin/lib/Synchronizer");
const locale_1 = require("@joplin/lib/locale");
const react_redux_1 = require("react-redux");
const theme_1 = require("@joplin/lib/theme");
const FolderAndTagList_1 = require("./FolderAndTagList");
const SidebarComponent = (props) => {
    const renderSynchronizeButton = (type) => {
        const label = type === 'sync' ? (0, locale_1._)('Synchronise') : (0, locale_1._)('Cancel');
        const iconAnimation = type !== 'sync' ? 'icon-infinite-rotation 1s linear infinite' : '';
        return (React.createElement(styles_1.StyledSynchronizeButton, { level: Button_1.ButtonLevel.SidebarSecondary, iconName: "icon-sync", key: "sync_button", iconAnimation: iconAnimation, title: label, onClick: () => {
                void CommandService_1.default.instance().execute('synchronize', type !== 'sync');
            } }));
    };
    const theme = (0, theme_1.themeStyle)(props.themeId);
    let decryptionReportText = '';
    if (props.decryptionWorker && props.decryptionWorker.state !== 'idle' && props.decryptionWorker.itemCount) {
        decryptionReportText = (0, locale_1._)('Decrypting items: %d/%d', props.decryptionWorker.itemIndex + 1, props.decryptionWorker.itemCount);
    }
    let resourceFetcherText = '';
    if (props.resourceFetcher && props.resourceFetcher.toFetchCount) {
        resourceFetcherText = (0, locale_1._)('Fetching resources: %d/%d', props.resourceFetcher.fetchingCount, props.resourceFetcher.toFetchCount);
    }
    const lines = Synchronizer_1.default.reportToLines(props.syncReport);
    if (resourceFetcherText)
        lines.push(resourceFetcherText);
    if (decryptionReportText)
        lines.push(decryptionReportText);
    const syncReportText = [];
    for (let i = 0; i < lines.length; i++) {
        syncReportText.push(React.createElement(styles_1.StyledSyncReportText, { key: i }, lines[i]));
    }
    const syncButton = renderSynchronizeButton(props.syncStarted ? 'cancel' : 'sync');
    const syncReportComp = !syncReportText.length ? null : (React.createElement(styles_1.StyledSyncReport, { key: "sync_report" }, syncReportText));
    return (React.createElement(styles_1.StyledRoot, { className: "sidebar", role: 'navigation', "aria-label": (0, locale_1._)('Sidebar') },
        React.createElement("div", { style: { flex: 1 } },
            React.createElement(FolderAndTagList_1.default, null)),
        React.createElement("div", { style: { flex: 0, padding: theme.mainPadding } },
            syncReportComp,
            syncButton)));
};
const mapStateToProps = (state) => {
    return {
        searches: state.searches,
        syncStarted: state.syncStarted,
        syncReport: state.syncReport,
        selectedSearchId: state.selectedSearchId,
        selectedSmartFilterId: state.selectedSmartFilterId,
        locale: state.settings.locale,
        themeId: state.settings.theme,
        collapsedFolderIds: state.collapsedFolderIds,
        decryptionWorker: state.decryptionWorker,
        resourceFetcher: state.resourceFetcher,
    };
};
exports.default = (0, react_redux_1.connect)(mapStateToProps)(SidebarComponent);
//# sourceMappingURL=Sidebar.js.map