"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const react_1 = require("react");
const types_1 = require("../types");
const TagItem_1 = require("../listItemComponents/TagItem");
const electron_1 = require("electron");
const trash_1 = require("@joplin/lib/services/trash");
const BaseModel_1 = require("@joplin/lib/BaseModel");
const Tag_1 = require("@joplin/lib/models/Tag");
const locale_1 = require("@joplin/lib/locale");
const string_utils_1 = require("@joplin/lib/string-utils");
const reducer_1 = require("@joplin/lib/reducer");
const Folder_1 = require("@joplin/lib/models/Folder");
const bridge_1 = require("../../../services/bridge");
const MenuUtils_1 = require("@joplin/lib/services/commands/MenuUtils");
const CommandService_1 = require("@joplin/lib/services/CommandService");
const InteropService_1 = require("@joplin/lib/services/interop/InteropService");
const InteropServiceHelper_1 = require("../../../InteropServiceHelper");
const stateToWhenClauseContext_1 = require("@joplin/lib/services/commands/stateToWhenClauseContext");
const Setting_1 = require("@joplin/lib/models/Setting");
const PerFolderSortOrderService_1 = require("../../../services/sortOrder/PerFolderSortOrderService");
const callbackUrlUtils_1 = require("@joplin/lib/callbackUrlUtils");
const reducer_2 = require("@joplin/lib/services/plugins/reducer");
const types_2 = require("@joplin/lib/services/plugins/api/types");
const FolderItem_1 = require("../listItemComponents/FolderItem");
const Logger_1 = require("@joplin/utils/Logger");
const onFolderDrop_1 = require("@joplin/lib/models/utils/onFolderDrop");
const HeaderItem_1 = require("../listItemComponents/HeaderItem");
const AllNotesItem_1 = require("../listItemComponents/AllNotesItem");
const ListItemWrapper_1 = require("../listItemComponents/ListItemWrapper");
const focusHandler_1 = require("@joplin/lib/utils/focusHandler");
const shim_1 = require("@joplin/lib/shim");
const Menu = (0, bridge_1.default)().Menu;
const MenuItem = (0, bridge_1.default)().MenuItem;
const logger = Logger_1.default.create('useOnRenderItem');
const menuUtils = new MenuUtils_1.default(CommandService_1.default.instance());
const focusListItem = (item) => {
    if (item) {
        // Avoid scrolling to the selected item when refocusing the note list. Such a refocus
        // can happen if the note list rerenders and the selection is scrolled out of view and
        // can cause scroll to change unexpectedly.
        (0, focusHandler_1.focus)('useOnRenderItem', item, { preventScroll: true });
    }
};
const noFocusListItem = () => { };
const useOnRenderItem = (props) => {
    const pluginsRef = (0, react_1.useRef)(null);
    pluginsRef.current = props.plugins;
    const foldersRef = (0, react_1.useRef)(null);
    foldersRef.current = props.folders;
    const tagItem_click = (0, react_1.useCallback)(({ tag }) => {
        props.dispatch({
            type: 'TAG_SELECT',
            id: tag ? tag.id : null,
        });
    }, [props.dispatch]);
    const onTagDrop_ = (0, react_1.useCallback)(async (event) => {
        const tagId = event.currentTarget.getAttribute('data-tag-id');
        const dt = event.dataTransfer;
        if (!dt)
            return;
        if (dt.types.indexOf('text/x-jop-note-ids') >= 0) {
            event.preventDefault();
            const noteIds = JSON.parse(dt.getData('text/x-jop-note-ids'));
            for (let i = 0; i < noteIds.length; i++) {
                await Tag_1.default.addNote(tagId, noteIds[i]);
            }
        }
    }, []);
    const onItemContextMenu = (0, react_1.useCallback)(async (event) => {
        const itemId = event.currentTarget.getAttribute('data-id');
        if (itemId === Folder_1.default.conflictFolderId())
            return;
        const itemType = Number(event.currentTarget.getAttribute('data-type'));
        if (!itemId || !itemType)
            throw new Error('No data on element');
        const state = (0, reducer_1.store)().getState();
        let deleteMessage = '';
        const deleteButtonLabel = (0, locale_1._)('Remove');
        if (itemType === BaseModel_1.default.TYPE_TAG) {
            const tag = await Tag_1.default.load(itemId);
            deleteMessage = (0, locale_1._)('Remove tag "%s" from all notes?', (0, string_utils_1.substrWithEllipsis)(tag.title, 0, 32));
        }
        else if (itemType === BaseModel_1.default.TYPE_SEARCH) {
            deleteMessage = (0, locale_1._)('Remove this search from the sidebar?');
        }
        const menu = new Menu();
        if (itemId === (0, trash_1.getTrashFolderId)()) {
            menu.append(new MenuItem(menuUtils.commandToStatefulMenuItem('emptyTrash')));
            menu.popup({ window: (0, bridge_1.default)().activeWindow() });
            return;
        }
        let item = null;
        if (itemType === BaseModel_1.default.TYPE_FOLDER) {
            item = BaseModel_1.default.byId(foldersRef.current, itemId);
        }
        const isDeleted = item ? !!item.deleted_time : false;
        if (!isDeleted) {
            if (itemType === BaseModel_1.default.TYPE_FOLDER && !item.encryption_applied) {
                menu.append(new MenuItem(menuUtils.commandToStatefulMenuItem('newFolder', itemId)));
            }
            if (itemType === BaseModel_1.default.TYPE_FOLDER) {
                menu.append(new MenuItem(menuUtils.commandToStatefulMenuItem('deleteFolder', itemId)));
            }
            else {
                menu.append(new MenuItem({
                    label: deleteButtonLabel,
                    click: async () => {
                        const ok = (0, bridge_1.default)().showConfirmMessageBox(deleteMessage, {
                            buttons: [deleteButtonLabel, (0, locale_1._)('Cancel')],
                            defaultId: 1,
                        });
                        if (!ok)
                            return;
                        if (itemType === BaseModel_1.default.TYPE_TAG) {
                            await Tag_1.default.untagAll(itemId);
                        }
                        else if (itemType === BaseModel_1.default.TYPE_SEARCH) {
                            props.dispatch({
                                type: 'SEARCH_DELETE',
                                id: itemId,
                            });
                        }
                    },
                }));
            }
            if (itemType === BaseModel_1.default.TYPE_FOLDER && !item.encryption_applied) {
                menu.append(new MenuItem(Object.assign(Object.assign({}, menuUtils.commandToStatefulMenuItem('moveToFolder', [itemId])), { 
                    // By default, enabled is based on the selected folder. However, the right-click
                    // menu can be shown for unselected folders.
                    enabled: true })));
                menu.append(new MenuItem(menuUtils.commandToStatefulMenuItem('openFolderDialog', { folderId: itemId })));
                menu.append(new MenuItem({ type: 'separator' }));
                const exportMenu = new Menu();
                const ioService = InteropService_1.default.instance();
                const ioModules = ioService.modules();
                for (let i = 0; i < ioModules.length; i++) {
                    const module = ioModules[i];
                    if (module.type !== 'exporter')
                        continue;
                    exportMenu.append(new MenuItem({
                        label: module.fullLabel(),
                        click: async () => {
                            await InteropServiceHelper_1.default.export(props.dispatch, module, { sourceFolderIds: [itemId], plugins: pluginsRef.current });
                        },
                    }));
                }
                // We don't display the "Share notebook" menu item for sub-notebooks
                // that are within a shared notebook. If user wants to do this,
                // they'd have to move the notebook out of the shared notebook
                // first.
                const whenClause = (0, stateToWhenClauseContext_1.default)(state, { commandFolderId: itemId });
                if (CommandService_1.default.instance().isEnabled('showShareFolderDialog', whenClause)) {
                    menu.append(new MenuItem(menuUtils.commandToStatefulMenuItem('showShareFolderDialog', itemId)));
                }
                if (CommandService_1.default.instance().isEnabled('leaveSharedFolder', whenClause)) {
                    menu.append(new MenuItem(menuUtils.commandToStatefulMenuItem('leaveSharedFolder', itemId)));
                }
                menu.append(new MenuItem({
                    label: (0, locale_1._)('Export'),
                    submenu: exportMenu,
                }));
                if (Setting_1.default.value('notes.perFolderSortOrderEnabled')) {
                    menu.append(new MenuItem(Object.assign(Object.assign({}, menuUtils.commandToStatefulMenuItem('togglePerFolderSortOrder', itemId)), { type: 'checkbox', checked: PerFolderSortOrderService_1.default.isSet(itemId) })));
                }
            }
            if (itemType === BaseModel_1.default.TYPE_FOLDER) {
                menu.append(new MenuItem({
                    label: (0, locale_1._)('Copy external link'),
                    click: () => {
                        electron_1.clipboard.writeText((0, callbackUrlUtils_1.getFolderCallbackUrl)(itemId));
                    },
                }));
            }
            if (itemType === BaseModel_1.default.TYPE_TAG) {
                menu.append(new MenuItem(menuUtils.commandToStatefulMenuItem('renameTag', itemId)));
                menu.append(new MenuItem({
                    label: (0, locale_1._)('Copy external link'),
                    click: () => {
                        electron_1.clipboard.writeText((0, callbackUrlUtils_1.getTagCallbackUrl)(itemId));
                    },
                }));
            }
            const pluginViews = reducer_2.utils.viewsByType(pluginsRef.current, 'menuItem');
            for (const view of pluginViews) {
                const location = view.location;
                if (itemType === BaseModel_1.ModelType.Tag && location === types_2.MenuItemLocation.TagContextMenu ||
                    itemType === BaseModel_1.ModelType.Folder && location === types_2.MenuItemLocation.FolderContextMenu) {
                    menu.append(new MenuItem(menuUtils.commandToStatefulMenuItem(view.commandName, itemId)));
                }
            }
        }
        else {
            if (itemType === BaseModel_1.default.TYPE_FOLDER) {
                menu.append(new MenuItem(menuUtils.commandToStatefulMenuItem('restoreFolder', itemId)));
            }
        }
        menu.popup({ window: (0, bridge_1.default)().activeWindow() });
    }, [props.dispatch, pluginsRef]);
    const onFolderDragStart_ = (0, react_1.useCallback)(event => {
        const folderId = event.currentTarget.getAttribute('data-folder-id');
        if (!folderId)
            return;
        event.dataTransfer.setDragImage(new Image(), 1, 1);
        event.dataTransfer.clearData();
        event.dataTransfer.setData('text/x-jop-folder-ids', JSON.stringify([folderId]));
    }, []);
    const onFolderDragOver_ = (0, react_1.useCallback)(event => {
        if (event.dataTransfer.types.indexOf('text/x-jop-note-ids') >= 0)
            event.preventDefault();
        if (event.dataTransfer.types.indexOf('text/x-jop-folder-ids') >= 0)
            event.preventDefault();
    }, []);
    const onFolderDrop_ = (0, react_1.useCallback)(async (event) => {
        const folderId = event.currentTarget.getAttribute('data-folder-id');
        const dt = event.dataTransfer;
        if (!dt)
            return;
        // folderId can be NULL when dropping on the sidebar Notebook header. In that case, it's used
        // to put the dropped folder at the root. But for notes, folderId needs to always be defined
        // since there's no such thing as a root note.
        try {
            if (dt.types.indexOf('text/x-jop-note-ids') >= 0) {
                event.preventDefault();
                if (!folderId)
                    return;
                const noteIds = JSON.parse(dt.getData('text/x-jop-note-ids'));
                await (0, onFolderDrop_1.default)(noteIds, [], folderId);
            }
            else if (dt.types.indexOf('text/x-jop-folder-ids') >= 0) {
                event.preventDefault();
                const folderIds = JSON.parse(dt.getData('text/x-jop-folder-ids'));
                await (0, onFolderDrop_1.default)([], folderIds, folderId);
            }
        }
        catch (error) {
            logger.error(error);
            await shim_1.default.showErrorDialog(error.message);
        }
    }, []);
    const onFolderToggleClick_ = (0, react_1.useCallback)(event => {
        const folderId = event.currentTarget.getAttribute('data-folder-id');
        props.dispatch({
            type: 'FOLDER_TOGGLE',
            id: folderId,
        });
    }, [props.dispatch]);
    const folderItem_click = (0, react_1.useCallback)((folderId) => {
        props.dispatch({
            type: 'FOLDER_SELECT',
            id: folderId ? folderId : null,
        });
    }, [props.dispatch]);
    // If at least one of the folder has an icon, then we display icons for all
    // folders (those without one will get the default icon). This is so that
    // visual alignment is correct for all folders, otherwise the folder tree
    // looks messy.
    const showFolderIcons = (0, react_1.useMemo)(() => {
        return Folder_1.default.shouldShowFolderIcons(props.folders);
    }, [props.folders]);
    const selectedIndexRef = (0, react_1.useRef)(props.selectedIndex);
    selectedIndexRef.current = props.selectedIndex;
    const itemCount = props.listItems.length;
    return (0, react_1.useCallback)((item, index) => {
        var _a;
        const selected = props.selectedIndex === index;
        const focusInList = document.hasFocus() && ((_a = props.containerRef.current) === null || _a === void 0 ? void 0 : _a.contains(document.activeElement));
        const anchorRef = (focusInList && selected) ? focusListItem : noFocusListItem;
        if (item.kind === types_1.ListItemType.Tag) {
            const tag = item.tag;
            return React.createElement(TagItem_1.default, { key: item.key, anchorRef: anchorRef, selected: selected, onClick: tagItem_click, onTagDrop: onTagDrop_, onContextMenu: onItemContextMenu, tag: tag, itemCount: itemCount, index: index });
        }
        else if (item.kind === types_1.ListItemType.Folder) {
            const folder = item.folder;
            const isExpanded = props.collapsedFolderIds.indexOf(folder.id) < 0;
            // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
            let noteCount = folder.note_count;
            // For now hide the count for folders in the trash because it doesn't work and getting it to
            // work would be tricky.
            if (folder.deleted_time || folder.id === (0, trash_1.getTrashFolderId)())
                noteCount = 0;
            // Thunderbird count: Subtract children note_count from parent folder if it expanded.
            if (isExpanded) {
                for (let i = 0; i < props.folders.length; i++) {
                    if (props.folders[i].parent_id === folder.id) {
                        // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
                        noteCount -= props.folders[i].note_count;
                    }
                }
            }
            return React.createElement(FolderItem_1.default, { key: item.key, anchorRef: anchorRef, selected: selected, folderId: folder.id, folderTitle: Folder_1.default.displayTitle(folder), folderIcon: Folder_1.default.unserializeIcon(folder.icon), depth: item.depth, isExpanded: isExpanded, hasChildren: item.hasChildren, noteCount: noteCount, onFolderDragStart_: onFolderDragStart_, onFolderDragOver_: onFolderDragOver_, onFolderDrop_: onFolderDrop_, itemContextMenu: onItemContextMenu, folderItem_click: folderItem_click, onFolderToggleClick_: onFolderToggleClick_, shareId: folder.share_id, parentId: folder.parent_id, showFolderIcon: showFolderIcons, index: index, itemCount: itemCount });
        }
        else if (item.kind === types_1.ListItemType.Header) {
            return React.createElement(HeaderItem_1.default, { key: item.id, anchorRef: anchorRef, item: item, isSelected: selected, onDrop: item.supportsFolderDrop ? onFolderDrop_ : null, index: index, itemCount: itemCount });
        }
        else if (item.kind === types_1.ListItemType.AllNotes) {
            return React.createElement(AllNotesItem_1.default, { key: item.key, anchorRef: anchorRef, selected: selected, index: index, itemCount: itemCount });
        }
        else if (item.kind === types_1.ListItemType.Spacer) {
            return (React.createElement(ListItemWrapper_1.default, { key: item.key, containerRef: anchorRef, depth: 0, selected: selected, itemIndex: index, itemCount: itemCount, highlightOnHover: false, className: 'sidebar-spacer-item' },
                React.createElement("div", { "aria-label": (0, locale_1._)('Spacer') })));
        }
        else {
            const exhaustivenessCheck = item;
            return exhaustivenessCheck;
        }
    }, [
        folderItem_click,
        onFolderDragOver_,
        onFolderDragStart_,
        onFolderDrop_,
        onFolderToggleClick_,
        onItemContextMenu,
        onTagDrop_,
        props.collapsedFolderIds,
        props.folders,
        showFolderIcons,
        tagItem_click,
        props.selectedIndex,
        props.containerRef,
        itemCount,
    ]);
};
exports.default = useOnRenderItem;
//# sourceMappingURL=useOnRenderItem.js.map