"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const types_1 = require("../types");
const react_1 = require("react");
const CommandService_1 = require("@joplin/lib/services/CommandService");
const toggleHeader_1 = require("./utils/toggleHeader");
const isToggleShortcut = (keyCode, selectedItem, collapsedFolderIds) => {
    if (selectedItem.kind !== types_1.ListItemType.Header && selectedItem.kind !== types_1.ListItemType.Folder) {
        return false;
    }
    if (!['Space', 'ArrowLeft', 'ArrowRight'].includes(keyCode)) {
        return false;
    }
    if (keyCode === 'Space') {
        return true;
    }
    const isCollapsed = 'expanded' in selectedItem ? !selectedItem.expanded : collapsedFolderIds.includes(selectedItem.folder.id);
    return (keyCode === 'ArrowRight') === isCollapsed;
};
const useOnSidebarKeyDownHandler = (props) => {
    const { updateSelectedIndex, listItems, selectedIndex, collapsedFolderIds, dispatch } = props;
    return (0, react_1.useCallback)((event) => {
        const selectedItem = listItems[selectedIndex];
        let indexChange = 0;
        if (selectedItem && isToggleShortcut(event.code, selectedItem, collapsedFolderIds)) {
            event.preventDefault();
            if (selectedItem.kind === types_1.ListItemType.Folder) {
                dispatch({
                    type: 'FOLDER_TOGGLE',
                    id: selectedItem.folder.id,
                });
            }
            else if (selectedItem.kind === types_1.ListItemType.Header) {
                (0, toggleHeader_1.default)(selectedItem.id);
            }
        }
        else if ((event.ctrlKey || event.metaKey) && event.code === 'KeyA') { // ctrl+a or cmd+a
            event.preventDefault();
        }
        else if (event.code === 'ArrowUp') {
            indexChange = -1;
        }
        else if (event.code === 'ArrowDown') {
            indexChange = 1;
        }
        else if (event.code === 'Tab' && event.shiftKey) {
            event.preventDefault();
            void CommandService_1.default.instance().execute('focusElement', 'noteBody');
        }
        else if (event.code === 'Enter' && !event.shiftKey) {
            event.preventDefault();
            void CommandService_1.default.instance().execute('focusElement', 'noteList');
        }
        if (indexChange !== 0) {
            event.preventDefault();
            updateSelectedIndex(selectedIndex + indexChange);
        }
    }, [selectedIndex, collapsedFolderIds, listItems, updateSelectedIndex, dispatch]);
};
exports.default = useOnSidebarKeyDownHandler;
//# sourceMappingURL=useOnSidebarKeyDownHandler.js.map