"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = require("react");
const types_1 = require("../types");
const side_menu_shared_1 = require("@joplin/lib/components/shared/side-menu-shared");
const { ALL_NOTES_FILTER_ID } = require('@joplin/lib/reserved-ids');
const useSelectedSidebarIndex = (props) => {
    const appStateSelectedIndex = (0, react_1.useMemo)(() => {
        for (let i = 0; i < props.listItems.length; i++) {
            const listItem = props.listItems[i];
            let selected = false;
            if (listItem.kind === types_1.ListItemType.AllNotes) {
                selected = props.selectedSmartFilterId === ALL_NOTES_FILTER_ID && props.notesParentType === 'SmartFilter';
            }
            else if (listItem.kind === types_1.ListItemType.Header || listItem.kind === types_1.ListItemType.Spacer) {
                selected = false;
            }
            else if (listItem.kind === types_1.ListItemType.Folder) {
                selected = (0, side_menu_shared_1.isFolderSelected)(listItem.folder, { selectedFolderId: props.selectedFolderId, notesParentType: props.notesParentType });
            }
            else if (listItem.kind === types_1.ListItemType.Tag) {
                selected = (0, side_menu_shared_1.isTagSelected)(listItem.tag, { selectedTagId: props.selectedTagId, notesParentType: props.notesParentType });
            }
            else {
                const exhaustivenessCheck = listItem;
                return exhaustivenessCheck;
            }
            if (selected) {
                return i;
            }
        }
        return -1;
    }, [props.listItems, props.selectedFolderId, props.selectedTagId, props.selectedSmartFilterId, props.notesParentType]);
    // Not all list items correspond with selectable Joplin folders/tags, but we want to
    // be able to select them anyway. This is handled with selectedIndexOverride.
    //
    // When selectedIndexOverride >= 0, it corresponds to the index of a selected item with no
    // specific note parent item (e.g. a header).
    const [selectedIndexOverride, setSelectedIndexOverride] = (0, react_1.useState)(-1);
    (0, react_1.useEffect)(() => {
        setSelectedIndexOverride(-1);
    }, [appStateSelectedIndex]);
    const updateSelectedIndex = (0, react_1.useCallback)((newIndex) => {
        if (newIndex < 0) {
            newIndex = 0;
        }
        else if (newIndex >= props.listItems.length) {
            newIndex = props.listItems.length - 1;
        }
        const newItem = props.listItems[newIndex];
        let newOverrideIndex = -1;
        if (newItem.kind === types_1.ListItemType.AllNotes) {
            props.dispatch({
                type: 'SMART_FILTER_SELECT',
                id: ALL_NOTES_FILTER_ID,
            });
        }
        else if (newItem.kind === types_1.ListItemType.Folder) {
            props.dispatch({
                type: 'FOLDER_SELECT',
                id: newItem.folder.id,
            });
        }
        else if (newItem.kind === types_1.ListItemType.Tag) {
            props.dispatch({
                type: 'TAG_SELECT',
                id: newItem.tag.id,
            });
        }
        else {
            newOverrideIndex = newIndex;
        }
        setSelectedIndexOverride(newOverrideIndex);
    }, [props.listItems, props.dispatch]);
    const selectedIndex = selectedIndexOverride === -1 ? appStateSelectedIndex : selectedIndexOverride;
    return { selectedIndex, updateSelectedIndex };
};
exports.default = useSelectedSidebarIndex;
//# sourceMappingURL=useSelectedSidebarIndex.js.map