"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const types_1 = require("@joplin/lib/services/database/types");
const ExpandLink_1 = require("./ExpandLink");
const styles_1 = require("../styles");
const FolderIconBox_1 = require("../../FolderIconBox");
const trash_1 = require("@joplin/lib/services/trash");
const Folder_1 = require("@joplin/lib/models/Folder");
const BaseModel_1 = require("@joplin/lib/BaseModel");
const locale_1 = require("@joplin/lib/locale");
const NoteCount_1 = require("./NoteCount");
const ListItemWrapper_1 = require("./ListItemWrapper");
const renderFolderIcon = (folderIcon) => {
    if (!folderIcon) {
        const defaultFolderIcon = {
            dataUrl: '',
            emoji: '',
            name: 'far fa-folder',
            type: types_1.FolderIconType.FontAwesome,
        };
        return React.createElement("div", { style: { marginRight: 7, display: 'flex' } },
            React.createElement(FolderIconBox_1.default, { opacity: 0.7, folderIcon: defaultFolderIcon }));
    }
    return React.createElement("div", { style: { marginRight: 7, display: 'flex' } },
        React.createElement(FolderIconBox_1.default, { folderIcon: folderIcon }));
};
function FolderItem(props) {
    const { hasChildren, showFolderIcon, isExpanded, parentId, depth, selected, folderId, folderTitle, folderIcon, noteCount, onFolderDragStart_, onFolderDragOver_, onFolderDrop_, itemContextMenu, folderItem_click, onFolderToggleClick_, shareId } = props;
    const shareTitle = (0, locale_1._)('Shared');
    const shareIcon = shareId && !parentId ? React.createElement(styles_1.StyledShareIcon, { "aria-label": shareTitle, title: shareTitle, className: "fas fa-share-alt" }) : null;
    const draggable = ![(0, trash_1.getTrashFolderId)(), Folder_1.default.conflictFolderId()].includes(folderId);
    const doRenderFolderIcon = () => {
        if (folderId === (0, trash_1.getTrashFolderId)()) {
            return renderFolderIcon((0, trash_1.getTrashFolderIcon)(types_1.FolderIconType.FontAwesome));
        }
        if (!showFolderIcon)
            return null;
        return renderFolderIcon(folderIcon);
    };
    return (React.createElement(ListItemWrapper_1.default, { containerRef: props.anchorRef, 
        // Folders are contained within the "Notebooks" section (which has depth 0):
        depth: depth + 1, selected: selected, itemIndex: props.index, itemCount: props.itemCount, expanded: hasChildren ? props.isExpanded : undefined, className: `list-item-container list-item-depth-${depth} ${selected ? 'selected' : ''}`, highlightOnHover: true, onDragStart: onFolderDragStart_, onDragOver: onFolderDragOver_, onDrop: onFolderDrop_, onContextMenu: itemContextMenu, draggable: draggable, "data-folder-id": folderId, "data-id": folderId, "data-type": BaseModel_1.ModelType.Folder },
        React.createElement(styles_1.StyledListItemAnchor, { className: "list-item", isConflictFolder: folderId === Folder_1.default.conflictFolderId(), selected: selected, shareId: shareId, "data-folder-id": folderId, onDoubleClick: onFolderToggleClick_, onClick: () => {
                folderItem_click(folderId);
            } },
            doRenderFolderIcon(),
            React.createElement(styles_1.StyledSpanFix, { className: "title" }, folderTitle),
            shareIcon,
            " ",
            React.createElement(NoteCount_1.default, { count: noteCount })),
        React.createElement(ExpandLink_1.default
        // The ExpandLink is included after the title so that the screen reader reads the
        // title first.
        , { 
            // The ExpandLink is included after the title so that the screen reader reads the
            // title first.
            className: 'toggle', hasChildren: hasChildren, folderTitle: folderTitle, folderId: folderId, onClick: onFolderToggleClick_, isExpanded: isExpanded })));
}
exports.default = FolderItem;
//# sourceMappingURL=FolderItem.js.map