"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const react_1 = require("react");
const locale_1 = require("@joplin/lib/locale");
const DialogButtonRow_1 = require("../DialogButtonRow");
const Dialog_1 = require("../Dialog");
const styled_components_1 = require("styled-components");
const DialogTitle_1 = require("../DialogTitle");
const SyncTargetRegistry_1 = require("@joplin/lib/SyncTargetRegistry");
const useElementSize_1 = require("@joplin/lib/hooks/useElementSize");
const Button_1 = require("../Button/Button");
const bridge_1 = require("../../services/bridge");
const Setting_1 = require("@joplin/lib/models/Setting");
const StyledRoot = styled_components_1.default.div `
	min-width: 500px;
	max-width: 1200px;
`;
const SyncTargetDescription = styled_components_1.default.div `
	${props => props.height ? `height: ${props.height}px` : ''};
	margin-bottom: 1.3em;
	line-height: ${props => props.theme.lineHeight};
	font-size: 16px;
`;
const ContentRoot = styled_components_1.default.div `
	background-color: ${props => props.theme.backgroundColor3};
	padding: 1em;
	padding-right: 0;
`;
const SelfHostingMessage = styled_components_1.default.div `
	color: ${props => props.theme.color};
	padding-right: 1em;
	font-style: italic;
	margin-top: 1em;
	opacity: 0.6;
`;
const SyncTargetBoxes = styled_components_1.default.div `
	display: flex;
	flex-direction: row;
	justify-content: center;
`;
const SyncTargetTitle = styled_components_1.default.p `
	display: flex;
	flex-direction: row;
	font-weight: bold;
	font-size: 1.7em;
	align-items: center;
	white-space: nowrap;
`;
const SyncTargetLogo = styled_components_1.default.img `
	height: 1.3em;
	margin-right: 0.4em;
`;
const SyncTargetBox = styled_components_1.default.div `
	display: flex;
	flex: 1;
	flex-direction: column;
	font-family: ${props => props.theme.fontFamily};
	color: ${props => props.theme.color};
	background-color: ${props => props.theme.backgroundColor};
	border: 1px solid ${props => props.theme.dividerColor};
	border-radius: 8px;
	padding: 2em 2.2em 2em 2.2em;
	margin-right: 1em;
	max-width: 400px;
	opacity: 1;
`;
const FeatureList = styled_components_1.default.div `
	margin-bottom: 1em;
`;
const FeatureIcon = styled_components_1.default.i `
	display: inline-flex;
	width: 16px;
	justify-content: center;
	color: ${props => props.theme.color4};
	position: absolute;
`;
const FeatureLine = styled_components_1.default.div `
	margin-bottom: .5em;
	opacity: ${props => props.enabled ? 1 : 0.5};
	position: relative;
	font-size: 16px;
`;
const FeatureLabel = styled_components_1.default.div `
	margin-left: 24px;
	line-height: ${props => props.theme.lineHeight};
`;
const SelectButton = (0, styled_components_1.default)(Button_1.default) `
	padding: 10px 10px;
    height: auto;
    min-height: auto;
    max-height: fit-content;
    font-size: 1em;
`;
const SlowSyncWarning = styled_components_1.default.div `
	margin-top: 1em;
	opacity: 0.8;
	font-family: ${props => props.theme.fontFamily};
	color: ${props => props.theme.color};
	font-size: 14px;
`;
const syncTargetNames = [
    'joplinCloud',
    'dropbox',
    'onedrive',
    'nextcloud',
    'webdav',
    'amazon_s3',
    'joplinServer',
];
const logosImageNames = {
    'dropbox': 'SyncTarget_Dropbox.svg',
    'joplinCloud': 'SyncTarget_JoplinCloud.svg',
    'onedrive': 'SyncTarget_OneDrive.svg',
};
function default_1(props) {
    const joplinCloudDescriptionRef = (0, react_1.useRef)(null);
    const closeDialog = (0, react_1.useCallback)(() => {
        props.dispatch({
            type: 'DIALOG_CLOSE',
            name: 'syncWizard',
        });
    }, [props.dispatch]);
    const onButtonRowClick = (0, react_1.useCallback)(() => {
        closeDialog();
    }, [closeDialog]);
    const { height: descriptionHeight } = (0, useElementSize_1.default)(joplinCloudDescriptionRef);
    function renderFeature(enabled, label) {
        const className = enabled ? 'fas fa-check' : 'fas fa-times';
        return (React.createElement(FeatureLine, { enabled: enabled, key: label },
            React.createElement(FeatureIcon, { className: className }),
            " ",
            React.createElement(FeatureLabel, null, label)));
    }
    function renderFeatures(name) {
        return (React.createElement(FeatureList, null, [
            renderFeature(true, (0, locale_1._)('Sync your notes')),
            renderFeature(name === 'joplinCloud', (0, locale_1._)('Publish notes to the internet')),
            renderFeature(name === 'joplinCloud', (0, locale_1._)('Collaborate on notebooks with others')),
        ]));
    }
    const onSelectButtonClick = (0, react_1.useCallback)(async (name) => {
        const routes = {
            'dropbox': { name: 'DropboxLogin', target: 7 },
            'onedrive': { name: 'OneDriveLogin', target: 3 },
            'joplinCloud': { name: 'JoplinCloudLogin', target: 10 },
        };
        const route = routes[name];
        if (!route)
            return; // throw error??
        Setting_1.default.setValue('sync.target', route.target);
        await Setting_1.default.saveAll();
        closeDialog();
        props.dispatch({
            type: 'NAV_GO',
            routeName: route.name,
        });
    }, [props.dispatch, closeDialog]);
    function renderSelectArea(info) {
        return (React.createElement(SelectButton, { level: Button_1.ButtonLevel.Primary, title: (0, locale_1._)('Select'), onClick: () => onSelectButtonClick(info.name), disabled: false }));
    }
    function renderSyncTarget(info) {
        const key = `syncTarget_${info.name}`;
        const height = info.name !== 'joplinCloud' ? descriptionHeight : null;
        const logoImageName = logosImageNames[info.name];
        const logoImageSrc = logoImageName ? `${(0, bridge_1.default)().buildDir()}/images/${logoImageName}` : '';
        const logo = logoImageSrc ? React.createElement(SyncTargetLogo, { src: logoImageSrc }) : null;
        const descriptionComp = React.createElement(SyncTargetDescription, { height: height, ref: info.name === 'joplinCloud' ? joplinCloudDescriptionRef : null }, info.description);
        const featuresComp = renderFeatures(info.name);
        const renderSlowSyncWarning = () => {
            if (info.name === 'joplinCloud')
                return null;
            return React.createElement(SlowSyncWarning, null, `⚠️ ${(0, locale_1._)('%s is not optimised for synchronising many small files so your initial synchronisation will be slow.', info.label)}`);
        };
        return (React.createElement(SyncTargetBox, { id: key, key: key },
            React.createElement(SyncTargetTitle, null,
                logo,
                info.label),
            descriptionComp,
            featuresComp,
            renderSelectArea(info),
            renderSlowSyncWarning()));
    }
    const onSelfHostingClick = (0, react_1.useCallback)(() => {
        closeDialog();
        props.dispatch({
            type: 'NAV_GO',
            routeName: 'Config',
            props: {
                defaultSection: 'sync',
            },
        });
    }, [props.dispatch, closeDialog]);
    function renderContent() {
        // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
        const boxes = [];
        for (const name of syncTargetNames) {
            const info = SyncTargetRegistry_1.default.infoByName(name);
            if (info.supportsSelfHosted)
                continue;
            boxes.push(renderSyncTarget(info));
        }
        const selfHostingMessage = React.createElement(SelfHostingMessage, null,
            "Self-hosting? Joplin also supports various self-hosting options such as Nextcloud, WebDAV, AWS S3 and Joplin Server. ",
            React.createElement("a", { href: "#", onClick: onSelfHostingClick }, "Click here to select one"),
            ".");
        return (React.createElement(ContentRoot, null,
            React.createElement(SyncTargetBoxes, null, boxes),
            selfHostingMessage));
    }
    function renderDialogWrapper() {
        return (React.createElement(StyledRoot, null,
            React.createElement(DialogTitle_1.default, { title: (0, locale_1._)('Joplin can synchronise your notes using various providers. Select one from the list below.'), justifyContent: "center" }),
            renderContent(),
            React.createElement(DialogButtonRow_1.default, { themeId: props.themeId, onClick: onButtonRowClick, okButtonShow: false, cancelButtonLabel: (0, locale_1._)('Close') })));
    }
    return (React.createElement(Dialog_1.default, { onCancel: closeDialog }, renderDialogWrapper()));
}
exports.default = default_1;
//# sourceMappingURL=Dialog.js.map