"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
function isFontAwesomeIcon(iconName) {
    const s = iconName.split(' ');
    return s.length === 2 && ['fa', 'fas'].includes(s[0]);
}
// eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
function getProp(props, name, defaultValue = null) {
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    if (props.toolbarButtonInfo && (name in props.toolbarButtonInfo))
        return props.toolbarButtonInfo[name];
    if (!(name in props))
        return defaultValue;
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    return props[name];
}
function ToolbarButton(props) {
    const title = getProp(props, 'title', '');
    const tooltip = getProp(props, 'tooltip', title);
    let icon = null;
    const iconName = getProp(props, 'iconName');
    if (iconName) {
        const iconProps = {
            'aria-hidden': true,
            role: 'img',
            className: `toolbar-icon ${title ? '-has-title' : ''} ${iconName}`,
        };
        icon = isFontAwesomeIcon(iconName) ? React.createElement("i", Object.assign({}, iconProps)) : React.createElement("span", Object.assign({}, iconProps));
    }
    // Keep this for legacy compatibility but for consistency we should use "disabled" prop
    let isEnabled = getProp(props, 'enabled', null);
    if (isEnabled === null)
        isEnabled = true;
    if (props.disabled)
        isEnabled = false;
    const classes = ['button', 'toolbar-button'];
    if (!isEnabled)
        classes.push('disabled');
    if (title)
        classes.push('-has-title');
    const onClick = getProp(props, 'onClick');
    const style = {
        whiteSpace: 'nowrap',
        overflow: 'hidden',
        textOverflow: 'ellipsis'
    };
    const disabled = !isEnabled;
    return (React.createElement("button", { className: classes.join(' '), title: tooltip, onClick: () => {
            if (isEnabled && onClick)
                onClick();
        }, ref: props.buttonRef, 
        // At least on MacOS, the disabled HTML prop isn't sufficient for the screen reader
        // to read the element as disable. For this, aria-disabled is necessary.
        disabled: disabled, "aria-label": !title ? tooltip : undefined, "aria-description": title ? tooltip : undefined, "aria-disabled": !isEnabled, tabIndex: props.tabIndex },
        icon,
        React.createElement("span", { style: style }, title)));
}
exports.default = ToolbarButton;
//# sourceMappingURL=ToolbarButton.js.map