"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = require("react");
const locale_1 = require("@joplin/lib/locale");
const NotyfContext_1 = require("../NotyfContext");
const dom_1 = require("@joplin/lib/dom");
const useAsyncEffect_1 = require("@joplin/lib/hooks/useAsyncEffect");
const html_1 = require("@joplin/utils/html");
const restoreItems_1 = require("@joplin/lib/services/trash/restoreItems");
const BaseModel_1 = require("@joplin/lib/BaseModel");
const theme_1 = require("@joplin/lib/theme");
exports.default = (props) => {
    const notyfContext = (0, react_1.useContext)(NotyfContext_1.default);
    const notificationRef = (0, react_1.useRef)(null);
    const theme = (0, react_1.useMemo)(() => {
        return (0, theme_1.themeStyle)(props.themeId);
    }, [props.themeId]);
    const notyf = (0, react_1.useMemo)(() => {
        const output = notyfContext;
        output.options.types = notyfContext.options.types.map(type => {
            if (type.type === 'success') {
                type.background = theme.backgroundColor5;
                // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
                type.icon.color = theme.backgroundColor5;
            }
            return type;
        });
        return output;
    }, [notyfContext, theme]);
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    const onCancelClick = (0, react_1.useCallback)(async (event) => {
        notyf.dismiss(notificationRef.current);
        notificationRef.current = null;
        const lastDeletion = JSON.parse(event.currentTarget.getAttribute('data-lastDeletion'));
        if (lastDeletion.folderIds.length) {
            await (0, restoreItems_1.default)(BaseModel_1.ModelType.Folder, lastDeletion.folderIds);
        }
        if (lastDeletion.noteIds.length) {
            await (0, restoreItems_1.default)(BaseModel_1.ModelType.Note, lastDeletion.noteIds);
        }
    }, [notyf]);
    (0, useAsyncEffect_1.default)(async (event) => {
        if (!props.lastDeletion || props.lastDeletion.timestamp <= props.lastDeletionNotificationTime)
            return;
        props.dispatch({ type: 'DELETION_NOTIFICATION_DONE' });
        let msg = '';
        if (props.lastDeletion.folderIds.length) {
            msg = (0, locale_1._)('The notebook and its content was successfully moved to the trash.');
        }
        else if (props.lastDeletion.noteIds.length) {
            msg = (0, locale_1._n)('The note was successfully moved to the trash.', 'The notes were successfully moved to the trash.', props.lastDeletion.noteIds.length);
        }
        else {
            return;
        }
        const linkId = `deletion-notification-cancel-${Math.floor(Math.random() * 1000000)}`;
        const cancelLabel = (0, locale_1._)('Cancel');
        const notification = notyf.success(`${msg} <a href="#" class="cancel" data-lastDeletion="${(0, html_1.htmlentities)(JSON.stringify(props.lastDeletion))}" id="${linkId}">${cancelLabel}</a>`);
        notificationRef.current = notification;
        const element = await (0, dom_1.waitForElement)(document, linkId);
        if (event.cancelled)
            return;
        element.addEventListener('click', onCancelClick);
    }, [props.lastDeletion, notyf, props.dispatch]);
    return React.createElement("div", { style: { display: 'none' } });
};
//# sourceMappingURL=TrashNotification.js.map