"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const PromptDialog_1 = require("../PromptDialog");
const ShareFolderDialog_1 = require("../ShareFolderDialog/ShareFolderDialog");
const NotePropertiesDialog_1 = require("../NotePropertiesDialog");
const NoteContentPropertiesDialog_1 = require("../NoteContentPropertiesDialog");
const ShareNoteDialog_1 = require("../ShareNoteDialog");
const react_1 = require("react");
const react_redux_1 = require("react-redux");
const ModalMessageOverlay_1 = require("./ModalMessageOverlay");
const reducer_1 = require("@joplin/lib/reducer");
const dialogs_1 = require("../dialogs");
const useDocument_1 = require("../hooks/useDocument");
const useWindowCommands_1 = require("./utils/useWindowCommands");
const PluginDialogs_1 = require("./PluginDialogs");
const useSyncDialogState_1 = require("./utils/useSyncDialogState");
const AppDialogs_1 = require("./AppDialogs");
const PluginManager = require('@joplin/lib/services/PluginManager');
const defaultDialogState = {
    noteContentPropertiesDialogOptions: {
        visible: false,
    },
    shareNoteDialogOptions: {
        visible: false,
    },
    notePropertiesDialogOptions: {
        visible: false,
    },
    shareFolderDialogOptions: {
        visible: false,
    },
    promptOptions: null,
};
// Certain dialog libraries need a reference to the active window:
const useSyncActiveWindow = (containerWindow) => {
    (0, react_1.useEffect)(() => {
        if (!containerWindow)
            return () => { };
        const onFocusCallback = () => {
            dialogs_1.default.setActiveWindow(containerWindow);
        };
        if (containerWindow.document.hasFocus()) {
            onFocusCallback();
        }
        containerWindow.addEventListener('focus', onFocusCallback);
        return () => {
            containerWindow.removeEventListener('focus', onFocusCallback);
        };
    }, [containerWindow]);
};
const WindowCommandsAndDialogs = props => {
    const [referenceElement, setReferenceElement] = (0, react_1.useState)(null);
    const containerDocument = (0, useDocument_1.default)(referenceElement);
    const documentRef = (0, react_1.useRef)(null);
    documentRef.current = containerDocument;
    const [dialogState, setDialogState] = (0, react_1.useState)(defaultDialogState);
    (0, useSyncDialogState_1.default)(dialogState, props.dispatch);
    (0, useWindowCommands_1.default)({
        documentRef,
        customCss: props.customCss,
        plugins: props.plugins,
        editorNoteStatuses: props.editorNoteStatuses,
        setDialogState,
    });
    useSyncActiveWindow(containerDocument === null || containerDocument === void 0 ? void 0 : containerDocument.defaultView);
    const onDialogHideCallbacks = (0, react_1.useMemo)(() => {
        const result = {};
        for (const key of Object.keys(defaultDialogState)) {
            result[key] = () => {
                setDialogState(dialogState => {
                    return Object.assign(Object.assign({}, dialogState), { [key]: { visible: false } });
                });
            };
        }
        return result;
    }, []);
    const promptOnClose = (0, react_1.useCallback)((answer, buttonType) => {
        dialogState.promptOptions.onClose(answer, buttonType);
    }, [dialogState.promptOptions]);
    const dialogInfo = PluginManager.instance().pluginDialogToShow(props.pluginsLegacy);
    const pluginDialog = !dialogInfo ? null : React.createElement(dialogInfo.Dialog, Object.assign({}, dialogInfo.props));
    const { noteContentPropertiesDialogOptions, notePropertiesDialogOptions, shareNoteDialogOptions, shareFolderDialogOptions, promptOptions } = dialogState;
    return React.createElement(React.Fragment, null,
        React.createElement("div", { ref: setReferenceElement }),
        pluginDialog,
        props.modalMessage !== null ? React.createElement(ModalMessageOverlay_1.default, { message: props.modalMessage }) : null,
        React.createElement(PluginDialogs_1.default, { themeId: props.themeId, visibleDialogs: props.visibleDialogs, pluginHtmlContents: props.pluginHtmlContents, plugins: props.plugins }),
        React.createElement(AppDialogs_1.default, { appDialogStates: props.appDialogStates, themeId: props.themeId, dispatch: props.dispatch }),
        noteContentPropertiesDialogOptions.visible && (React.createElement(NoteContentPropertiesDialog_1.default, { markupLanguage: noteContentPropertiesDialogOptions.markupLanguage, themeId: props.themeId, onClose: onDialogHideCallbacks.noteContentPropertiesDialogOptions, text: noteContentPropertiesDialogOptions.text })),
        notePropertiesDialogOptions.visible && (React.createElement(NotePropertiesDialog_1.default, { themeId: props.themeId, noteId: notePropertiesDialogOptions.noteId, onClose: onDialogHideCallbacks.notePropertiesDialogOptions, onRevisionLinkClick: notePropertiesDialogOptions.onRevisionLinkClick })),
        shareNoteDialogOptions.visible && (React.createElement(ShareNoteDialog_1.default, { themeId: props.themeId, noteIds: shareNoteDialogOptions.noteIds, onClose: onDialogHideCallbacks.shareNoteDialogOptions })),
        shareFolderDialogOptions.visible && (React.createElement(ShareFolderDialog_1.default, { themeId: props.themeId, folderId: shareFolderDialogOptions.folderId, onClose: onDialogHideCallbacks.shareFolderDialogOptions })),
        React.createElement(PromptDialog_1.default, { autocomplete: promptOptions && 'autocomplete' in promptOptions ? promptOptions.autocomplete : null, defaultValue: promptOptions && promptOptions.value ? promptOptions.value : '', themeId: props.themeId, onClose: promptOnClose, label: promptOptions ? promptOptions.label : '', description: promptOptions ? promptOptions.description : null, visible: !!promptOptions, buttons: promptOptions && 'buttons' in promptOptions ? promptOptions.buttons : null, inputType: promptOptions && 'inputType' in promptOptions ? promptOptions.inputType : null }));
};
exports.default = (0, react_redux_1.connect)((state, ownProps) => {
    const windowState = reducer_1.stateUtils.windowStateById(state, ownProps.windowId);
    return {
        themeId: state.settings.theme,
        plugins: state.pluginService.plugins,
        visibleDialogs: windowState.visibleDialogs,
        appDialogStates: windowState.dialogs,
        pluginHtmlContents: state.pluginService.pluginHtmlContents,
        customCss: state.customViewerCss,
        editorNoteStatuses: state.editorNoteStatuses,
        pluginsLegacy: state.pluginsLegacy,
        modalMessage: state.modalOverlayMessage,
    };
})(WindowCommandsAndDialogs);
//# sourceMappingURL=WindowCommandsAndDialogs.js.map