"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.runtime = exports.declaration = void 0;
const locale_1 = require("@joplin/lib/locale");
const bridge_1 = require("../../../services/bridge");
const Folder_1 = require("@joplin/lib/models/Folder");
const { substrWithEllipsis } = require('@joplin/lib/string-utils');
exports.declaration = {
    name: 'deleteFolder',
    label: () => (0, locale_1._)('Delete notebook'),
};
const runtime = () => {
    return {
        execute: async (context, folderId = null) => {
            if (folderId === null)
                folderId = context.state.selectedFolderId;
            const folder = await Folder_1.default.load(folderId);
            if (!folder)
                throw new Error(`No such folder: ${folderId}`);
            let deleteMessage = (0, locale_1._)('Move notebook "%s" to the trash?\n\nAll notes and sub-notebooks within this notebook will also be moved to the trash.', substrWithEllipsis(folder.title, 0, 32));
            if (folderId === context.state.settings['sync.10.inboxId']) {
                deleteMessage = (0, locale_1._)('Delete the Inbox notebook?\n\nIf you delete the inbox notebook, any email that\'s recently been sent to it may be lost.');
            }
            const ok = (0, bridge_1.default)().showConfirmMessageBox(deleteMessage);
            if (!ok)
                return;
            await Folder_1.default.delete(folderId, { toTrash: true, sourceDescription: 'deleteFolder command' });
        },
        enabledCondition: '!folderIsReadOnly',
    };
};
exports.runtime = runtime;
//# sourceMappingURL=deleteFolder.js.map