"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.runtime = exports.declaration = void 0;
const eventManager_1 = require("@joplin/lib/eventManager");
const locale_1 = require("@joplin/lib/locale");
const reducer_1 = require("@joplin/lib/reducer");
const Note_1 = require("@joplin/lib/models/Note");
const time_1 = require("@joplin/lib/time");
exports.declaration = {
    name: 'editAlarm',
    label: () => (0, locale_1._)('Set alarm'),
    iconName: 'icon-alarm',
};
// eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
const runtime = (comp) => {
    return {
        execute: async (context, noteId = null) => {
            noteId = noteId || reducer_1.stateUtils.selectedNoteId(context.state);
            const note = await Note_1.default.load(noteId);
            const defaultDate = new Date(Date.now() + 2 * 3600 * 1000);
            defaultDate.setMinutes(0);
            defaultDate.setSeconds(0);
            comp.setState({
                promptOptions: {
                    label: (0, locale_1._)('Set alarm:'),
                    inputType: 'datetime',
                    buttons: ['ok', 'cancel', 'clear'],
                    value: note.todo_due ? new Date(note.todo_due) : defaultDate,
                    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
                    onClose: async (answer, buttonType) => {
                        let newNote = null;
                        if (buttonType === 'clear') {
                            newNote = {
                                id: note.id,
                                todo_due: 0,
                            };
                        }
                        else if (answer !== null) {
                            newNote = {
                                id: note.id,
                                todo_due: answer.getTime(),
                            };
                        }
                        if (newNote) {
                            await Note_1.default.save(newNote);
                            eventManager_1.default.emit(eventManager_1.EventName.AlarmChange, { noteId: note.id, note: newNote });
                        }
                        comp.setState({ promptOptions: null });
                    },
                },
            });
        },
        enabledCondition: 'oneNoteSelected && noteIsTodo && !noteTodoCompleted',
        // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
        mapStateToTitle: (state) => {
            const note = reducer_1.stateUtils.selectedNote(state);
            return note && note.todo_due ? time_1.default.formatMsToLocal(note.todo_due) : null;
        },
    };
};
exports.runtime = runtime;
//# sourceMappingURL=editAlarm.js.map