"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.runtime = exports.declaration = void 0;
const locale_1 = require("@joplin/lib/locale");
const Folder_1 = require("@joplin/lib/models/Folder");
const Note_1 = require("@joplin/lib/models/Note");
const BaseItem_1 = require("@joplin/lib/models/BaseItem");
const BaseModel_1 = require("@joplin/lib/BaseModel");
const Logger_1 = require("@joplin/utils/Logger");
const shim_1 = require("@joplin/lib/shim");
const logger = Logger_1.default.create('commands/moveToFolder');
exports.declaration = {
    name: 'moveToFolder',
    label: () => (0, locale_1._)('Move to notebook'),
};
// eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
const runtime = (comp) => {
    return {
        execute: async (context, itemIds = null) => {
            itemIds = itemIds || context.state.selectedNoteIds;
            let allAreFolders = true;
            const itemIdToType = new Map();
            for (const id of itemIds) {
                const item = await BaseItem_1.default.loadItemById(id);
                itemIdToType.set(id, item.type_);
                if (item.type_ !== BaseModel_1.ModelType.Folder) {
                    allAreFolders = false;
                }
            }
            const folders = await Folder_1.default.sortFolderTree();
            // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
            const startFolders = [];
            const maxDepth = 15;
            // It's okay for folders (but not notes) to have no parent folder:
            if (allAreFolders) {
                startFolders.push({
                    key: '',
                    value: '',
                    label: (0, locale_1._)('None'),
                    indentDepth: 0,
                });
            }
            const addOptions = (folders, depth) => {
                for (let i = 0; i < folders.length; i++) {
                    const folder = folders[i];
                    // Disallow making a folder a subfolder of itself.
                    if (itemIdToType.has(folder.id)) {
                        continue;
                    }
                    startFolders.push({ key: folder.id, value: folder.id, label: folder.title, indentDepth: depth });
                    if (folder.children)
                        addOptions(folder.children, (depth + 1) < maxDepth ? depth + 1 : maxDepth);
                }
            };
            addOptions(folders, 0);
            comp.setState({
                promptOptions: {
                    label: (0, locale_1._)('Move to notebook:'),
                    inputType: 'dropdown',
                    value: '',
                    autocomplete: startFolders,
                    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
                    onClose: async (answer) => {
                        if (answer) {
                            try {
                                const targetFolderId = answer.value;
                                for (const id of itemIds) {
                                    if (id === targetFolderId) {
                                        continue;
                                    }
                                    const itemType = itemIdToType.get(id);
                                    if (itemType === BaseModel_1.ModelType.Note) {
                                        await Note_1.default.moveToFolder(id, targetFolderId);
                                    }
                                    else if (itemType === BaseModel_1.ModelType.Folder) {
                                        await Folder_1.default.moveToFolder(id, targetFolderId);
                                    }
                                    else {
                                        throw new Error(`Cannot move item with type ${itemType}`);
                                    }
                                }
                            }
                            catch (error) {
                                logger.error('Error moving items', error);
                                void shim_1.default.showMessageBox(`Error: ${error}`);
                            }
                        }
                        comp.setState({ promptOptions: null });
                    },
                },
            });
        },
        enabledCondition: 'someNotesSelected && !noteIsReadOnly',
    };
};
exports.runtime = runtime;
//# sourceMappingURL=moveToFolder.js.map