"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.runtime = exports.declaration = exports.newNoteEnabledConditions = void 0;
const CommandService_1 = require("@joplin/lib/services/CommandService");
const locale_1 = require("@joplin/lib/locale");
const Setting_1 = require("@joplin/lib/models/Setting");
const Note_1 = require("@joplin/lib/models/Note");
exports.newNoteEnabledConditions = 'oneFolderSelected && !inConflictFolder && !folderIsReadOnly && !folderIsTrash';
exports.declaration = {
    name: 'newNote',
    label: () => (0, locale_1._)('New note'),
    iconName: 'fa-file',
};
const runtime = () => {
    return {
        execute: async (_context, body = '', isTodo = false) => {
            const folderId = Setting_1.default.value('activeFolderId');
            if (!folderId)
                return;
            const defaultValues = Note_1.default.previewFieldsWithDefaultValues({ includeTimestamps: false });
            let newNote = Object.assign(Object.assign({}, defaultValues), { parent_id: folderId, is_todo: isTodo ? 1 : 0, body: body });
            newNote = await Note_1.default.save(newNote, { provisional: true });
            void Note_1.default.updateGeolocation(newNote.id);
            CommandService_1.utils.store.dispatch({
                type: 'NOTE_SELECT',
                id: newNote.id,
            });
            // Immediately sort the note list so that the new note is positioned correctly before
            // scrolling to it.
            CommandService_1.utils.store.dispatch({
                type: 'NOTE_SORT',
            });
        },
        enabledCondition: exports.newNoteEnabledConditions,
    };
};
exports.runtime = runtime;
//# sourceMappingURL=newNote.js.map