"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.runtime = exports.declaration = void 0;
const locale_1 = require("@joplin/lib/locale");
const Tag_1 = require("@joplin/lib/models/Tag");
exports.declaration = {
    name: 'setTags',
    label: () => (0, locale_1._)('Tags'),
    iconName: 'icon-tags',
};
// eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
const runtime = (comp) => {
    return {
        execute: async (context, noteIds = null) => {
            noteIds = noteIds || context.state.selectedNoteIds;
            const tags = await Tag_1.default.commonTagsByNoteIds(noteIds);
            const startTags = tags
                // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
                .map((a) => {
                return { value: a.id, label: a.title };
            })
                // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
                .sort((a, b) => {
                // sensitivity accent will treat accented characters as different
                // but treats caps as equal
                return a.label.localeCompare(b.label, undefined, { sensitivity: 'accent' });
            });
            const allTags = await Tag_1.default.allWithNotes();
            // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
            const tagSuggestions = allTags.map((a) => {
                return { value: a.id, label: a.title };
            })
                // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
                .sort((a, b) => {
                // sensitivity accent will treat accented characters as different
                // but treats caps as equal
                return a.label.localeCompare(b.label, undefined, { sensitivity: 'accent' });
            });
            comp.setState({
                promptOptions: {
                    label: (0, locale_1._)('Add or remove tags:'),
                    inputType: 'tags',
                    value: startTags,
                    autocomplete: tagSuggestions,
                    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
                    onClose: async (answer) => {
                        if (answer !== null) {
                            const endTagTitles = answer.map(a => {
                                return a.label.trim();
                            });
                            if (noteIds.length === 1) {
                                await Tag_1.default.setNoteTagsByTitles(noteIds[0], endTagTitles);
                            }
                            else {
                                // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
                                const startTagTitles = startTags.map((a) => { return a.label.trim(); });
                                const addTags = endTagTitles.filter((value) => !startTagTitles.includes(value));
                                const delTags = startTagTitles.filter((value) => !endTagTitles.includes(value));
                                // apply the tag additions and deletions to each selected note
                                for (let i = 0; i < noteIds.length; i++) {
                                    const tags = await Tag_1.default.tagsByNoteId(noteIds[i]);
                                    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
                                    let tagTitles = tags.map((a) => { return a.title; });
                                    tagTitles = tagTitles.concat(addTags);
                                    tagTitles = tagTitles.filter((value) => !delTags.includes(value));
                                    await Tag_1.default.setNoteTagsByTitles(noteIds[i], tagTitles);
                                }
                            }
                        }
                        comp.setState({ promptOptions: null });
                    },
                },
            });
        },
        enabledCondition: 'someNotesSelected && !inTrash',
    };
};
exports.runtime = runtime;
//# sourceMappingURL=setTags.js.map