"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.runtime = exports.declaration = void 0;
const Setting_1 = require("@joplin/lib/models/Setting");
const getActivePluginEditorView_1 = require("@joplin/lib/services/plugins/utils/getActivePluginEditorView");
const Logger_1 = require("@joplin/utils/Logger");
const logger = Logger_1.default.create('showEditorPlugin');
exports.declaration = {
    name: 'showEditorPlugin',
    label: () => 'Show editor plugin',
    iconName: 'fas fa-eye',
};
const runtime = () => {
    return {
        execute: async (context, editorViewId = '', show = true) => {
            logger.info('View:', editorViewId, 'Show:', show);
            const shownEditorViewIds = Setting_1.default.value('plugins.shownEditorViewIds');
            if (!editorViewId) {
                const { editorPlugin, editorView } = (0, getActivePluginEditorView_1.default)(context.state.pluginService.plugins);
                if (!editorPlugin) {
                    logger.warn('No editor plugin to toggle to');
                    return;
                }
                editorViewId = editorView.id;
            }
            const idx = shownEditorViewIds.indexOf(editorViewId);
            if (show) {
                if (idx >= 0) {
                    logger.info(`Editor is already visible: ${editorViewId}`);
                    return;
                }
                shownEditorViewIds.push(editorViewId);
            }
            else {
                if (idx < 0) {
                    logger.info(`Editor is already hidden: ${editorViewId}`);
                    return;
                }
                shownEditorViewIds.splice(idx, 1);
            }
            logger.info('Shown editor IDs:', shownEditorViewIds);
            Setting_1.default.setValue('plugins.shownEditorViewIds', shownEditorViewIds);
        },
    };
};
exports.runtime = runtime;
//# sourceMappingURL=showEditorPlugin.js.map