"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const reducer_1 = require("@joplin/lib/services/plugins/reducer");
const CommandService_1 = require("@joplin/lib/services/CommandService");
const InteropService_1 = require("@joplin/lib/services/interop/InteropService");
const MenuUtils_1 = require("@joplin/lib/services/commands/MenuUtils");
const InteropServiceHelper_1 = require("../../InteropServiceHelper");
const locale_1 = require("@joplin/lib/locale");
const types_1 = require("@joplin/lib/services/plugins/api/types");
const callbackUrlUtils_1 = require("@joplin/lib/callbackUrlUtils");
const bridge_1 = require("../../services/bridge");
const BaseModel_1 = require("@joplin/lib/BaseModel");
const Note_1 = require("@joplin/lib/models/Note");
const Setting_1 = require("@joplin/lib/models/Setting");
const { clipboard } = require('electron');
const Menu = (0, bridge_1.default)().Menu;
const MenuItem = (0, bridge_1.default)().MenuItem;
class NoteListUtils {
    static makeContextMenu(noteIds, props) {
        const cmdService = CommandService_1.default.instance();
        const menuUtils = new MenuUtils_1.default(cmdService);
        const notes = BaseModel_1.default.modelsByIds(props.notes, noteIds);
        const singleNoteId = noteIds.length === 1 ? noteIds[0] : null;
        const includeDeletedNotes = notes.find(n => !!n.deleted_time);
        const includeEncryptedNotes = notes.find(n => !!n.encryption_applied);
        const menu = new Menu();
        if (!includeEncryptedNotes && !includeDeletedNotes) {
            menu.append(
            // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
            new MenuItem(menuUtils.commandToStatefulMenuItem('setTags', noteIds)));
            menu.append(
            // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
            new MenuItem(menuUtils.commandToStatefulMenuItem('moveToFolder', noteIds)));
            menu.append(
            // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
            new MenuItem(menuUtils.commandToStatefulMenuItem('duplicateNote', noteIds)));
            if (singleNoteId) {
                const editInMenu = new Menu();
                const cmd = props.watchedNoteFiles.includes(singleNoteId) ? 'stopExternalEditing' : 'startExternalEditing';
                // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
                editInMenu.append(new MenuItem(menuUtils.commandToStatefulMenuItem(cmd, singleNoteId)));
                editInMenu.append(new MenuItem(menuUtils.commandToStatefulMenuItem('openNoteInNewWindow', singleNoteId)));
                menu.append(new MenuItem({ label: (0, locale_1._)('Edit in...'), submenu: editInMenu }));
            }
            if (noteIds.length <= 1) {
                menu.append(new MenuItem(
                // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
                menuUtils.commandToStatefulMenuItem('toggleNoteType', noteIds)));
            }
            else {
                const switchNoteType = async (noteIds, type) => {
                    for (let i = 0; i < noteIds.length; i++) {
                        const note = await Note_1.default.load(noteIds[i]);
                        const newNote = Note_1.default.changeNoteType(note, type);
                        if (newNote === note)
                            continue;
                        await Note_1.default.save(newNote, { userSideValidation: true });
                    }
                };
                menu.append(new MenuItem({
                    label: (0, locale_1._)('Switch to note type'),
                    click: async () => {
                        await switchNoteType(noteIds, 'note');
                    },
                }));
                menu.append(new MenuItem({
                    label: (0, locale_1._)('Switch to to-do type'),
                    click: async () => {
                        await switchNoteType(noteIds, 'todo');
                    },
                }));
            }
            menu.append(new MenuItem({
                label: (0, locale_1._)('Copy Markdown link'),
                click: async () => {
                    const links = [];
                    for (let i = 0; i < noteIds.length; i++) {
                        const note = await Note_1.default.load(noteIds[i]);
                        links.push(Note_1.default.markdownTag(note));
                    }
                    clipboard.writeText(links.join(' '));
                },
            }));
            if (noteIds.length === 1) {
                menu.append(new MenuItem({
                    label: (0, locale_1._)('Copy external link'),
                    click: () => {
                        clipboard.writeText((0, callbackUrlUtils_1.getNoteCallbackUrl)(noteIds[0]));
                    },
                }));
            }
            if ([9, 10].includes(Setting_1.default.value('sync.target'))) {
                menu.append(new MenuItem(
                // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
                menuUtils.commandToStatefulMenuItem('showShareNoteDialog', noteIds.slice())));
            }
            const exportMenu = new Menu();
            const ioService = InteropService_1.default.instance();
            const ioModules = ioService.modules();
            for (let i = 0; i < ioModules.length; i++) {
                const module = ioModules[i];
                if (module.type !== 'exporter')
                    continue;
                if (noteIds.length > 1 && module.isNoteArchive === false)
                    continue;
                exportMenu.append(new MenuItem({
                    label: module.fullLabel(),
                    click: async () => {
                        await InteropServiceHelper_1.default.export(props.dispatch.bind(this), module, {
                            sourceNoteIds: noteIds,
                            includeConflicts: props.inConflictFolder,
                            plugins: props.plugins,
                            customCss: props.customCss,
                        });
                    },
                }));
            }
            exportMenu.append(new MenuItem(
            // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
            menuUtils.commandToStatefulMenuItem('exportPdf', noteIds)));
            const exportMenuItem = new MenuItem({ label: (0, locale_1._)('Export'), submenu: exportMenu });
            menu.append(exportMenuItem);
        }
        if (includeDeletedNotes) {
            menu.append(new MenuItem(
            // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
            menuUtils.commandToStatefulMenuItem('restoreNote', noteIds)));
            menu.append(new MenuItem(
            // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
            menuUtils.commandToStatefulMenuItem('permanentlyDeleteNote', noteIds)));
        }
        else {
            menu.append(new MenuItem(
            // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
            menuUtils.commandToStatefulMenuItem('deleteNote', noteIds)));
        }
        const pluginViewInfos = reducer_1.utils.viewInfosByType(props.plugins, 'menuItem');
        for (const info of pluginViewInfos) {
            const location = info.view.location;
            if (location !== types_1.MenuItemLocation.Context && location !== types_1.MenuItemLocation.NoteListContextMenu)
                continue;
            if (cmdService.isEnabled(info.view.commandName)) {
                menu.append(
                // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
                new MenuItem(menuUtils.commandToStatefulMenuItem(info.view.commandName, noteIds)));
            }
        }
        return menu;
    }
}
exports.default = NoteListUtils;
//# sourceMappingURL=NoteListUtils.js.map