"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const NoteEditorScreen_1 = require("./NoteEditorScreen");
const activateMainMenuItem_1 = require("../util/activateMainMenuItem");
const Sidebar_1 = require("./Sidebar");
const GoToAnything_1 = require("./GoToAnything");
const setFilePickerResponse_1 = require("../util/setFilePickerResponse");
const NoteList_1 = require("./NoteList");
const test_1 = require("../util/test");
class MainScreen {
    constructor(page) {
        this.page = page;
        this.newNoteButton = page.locator('.new-note-button');
        this.noteList = new NoteList_1.default(page);
        this.sidebar = new Sidebar_1.default(page, this);
        this.dialog = page.locator('.dialog-modal-layer');
        this.noteEditor = new NoteEditorScreen_1.default(page);
        this.goToAnything = new GoToAnything_1.default(page, this);
    }
    async setup() {
        await this.waitFor();
        await this.sidebar.createNewFolder('Test');
        return this;
    }
    async waitFor() {
        await this.newNoteButton.waitFor();
        await this.noteList.waitFor();
    }
    // Follows the steps a user would use to create a new note.
    async createNewNote(title) {
        await this.waitFor();
        // Create the new note. Retry this -- creating new notes can sometimes fail if done just after
        // application startup.
        await test_1.expect.poll(async () => {
            await this.newNoteButton.click();
            await (0, test_1.expect)(this.noteEditor.noteTitleInput).toHaveValue('', { timeout: 4000 });
            await (0, test_1.expect)(this.noteEditor.noteTitleInput).toHaveJSProperty('placeholder', 'Creating new note...', { timeout: 4000 });
            return true;
        }, { timeout: 10000 }).toBe(true);
        // Fill the title
        await this.noteEditor.noteTitleInput.click();
        await this.noteEditor.noteTitleInput.fill(title);
        return this.noteEditor;
    }
    async openSettings(electronApp) {
        // Check both labels so this works on MacOS
        await (0, activateMainMenuItem_1.default)(electronApp, /^(Preferences\.\.\.|Options)$/);
    }
    async search(text) {
        const searchBar = this.page.getByPlaceholder('Search...');
        await searchBar.fill(text);
    }
    async importHtmlDirectory(electronApp, path) {
        await (0, setFilePickerResponse_1.default)(electronApp, [path]);
        await (0, activateMainMenuItem_1.default)(electronApp, 'HTML - HTML document (Directory)', 'Import');
    }
}
exports.default = MainScreen;
//# sourceMappingURL=MainScreen.js.map