"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const test_1 = require("./util/test");
const MainScreen_1 = require("./models/MainScreen");
test_1.test.describe('sidebar', () => {
    (0, test_1.test)('should be able to create new folders', async ({ mainWindow }) => {
        const mainScreen = await new MainScreen_1.default(mainWindow).setup();
        const sidebar = mainScreen.sidebar;
        for (let i = 0; i < 3; i++) {
            const title = `Test folder ${i}`;
            await sidebar.createNewFolder(title);
            await (0, test_1.expect)(sidebar.container.getByText(title)).toBeAttached();
        }
        // The first folder should still be visible
        await (0, test_1.expect)(sidebar.container.getByText('Test folder 0')).toBeAttached();
    });
    (0, test_1.test)('should allow changing the focused folder with the arrow keys', async ({ electronApp, mainWindow }) => {
        const mainScreen = await new MainScreen_1.default(mainWindow).setup();
        const sidebar = mainScreen.sidebar;
        const folderAHeader = await sidebar.createNewFolder('Folder A');
        await (0, test_1.expect)(folderAHeader).toBeVisible();
        const folderBHeader = await sidebar.createNewFolder('Folder B');
        await (0, test_1.expect)(folderBHeader).toBeVisible();
        await folderBHeader.click();
        await sidebar.forceUpdateSorting(electronApp);
        await folderBHeader.click();
        await mainWindow.keyboard.press('ArrowUp');
        await (0, test_1.expect)(mainWindow.locator(':focus')).toHaveText('Folder A');
        await mainWindow.keyboard.press('ArrowDown');
        await (0, test_1.expect)(mainWindow.locator(':focus')).toHaveText('Folder B');
        await mainWindow.keyboard.press('ArrowUp');
        await (0, test_1.expect)(mainWindow.locator(':focus')).toHaveText('Folder A');
        await mainWindow.keyboard.press('ArrowUp');
        await (0, test_1.expect)(mainWindow.locator(':focus')).toHaveText('All notes');
        await mainWindow.keyboard.press('ArrowUp');
        await (0, test_1.expect)(mainWindow.locator(':focus')).toHaveText(/NOTEBOOKS/i);
        await mainWindow.keyboard.press('ArrowDown');
        await (0, test_1.expect)(mainWindow.locator(':focus')).toHaveText('All notes');
    });
    (0, test_1.test)('should allow changing the parent of a folder by drag-and-drop', async ({ electronApp, mainWindow }) => {
        const mainScreen = await new MainScreen_1.default(mainWindow).setup();
        const sidebar = mainScreen.sidebar;
        const parentFolderHeader = await sidebar.createNewFolder('Parent folder');
        await (0, test_1.expect)(parentFolderHeader).toBeVisible();
        const childFolderHeader = await sidebar.createNewFolder('Child folder');
        await (0, test_1.expect)(childFolderHeader).toBeVisible();
        await sidebar.forceUpdateSorting(electronApp);
        await (0, test_1.expect)(childFolderHeader).toBeVisible();
        await childFolderHeader.dragTo(parentFolderHeader);
        // Verify that it's now a child folder -- expand and collapse the parent
        await (0, test_1.expect)(parentFolderHeader).toHaveJSProperty('ariaExpanded', 'true');
        const toggleButton = parentFolderHeader.getByRole('button', { name: /^(Expand|Collapse)/ });
        await toggleButton.click();
        // Should be collapsed
        await (0, test_1.expect)(childFolderHeader).not.toBeAttached();
        await (0, test_1.expect)(parentFolderHeader).toHaveJSProperty('ariaExpanded', 'false');
        await toggleButton.click();
        // Should be possible to move back to the root
        const rootFolderHeader = sidebar.container.getByText('Notebooks');
        await childFolderHeader.dragTo(rootFolderHeader);
        await (0, test_1.expect)(toggleButton).not.toBeVisible();
    });
    (0, test_1.test)('all notes section should list all notes', async ({ electronApp, mainWindow }) => {
        const mainScreen = await new MainScreen_1.default(mainWindow).setup();
        const sidebar = mainScreen.sidebar;
        const testFolderA = await sidebar.createNewFolder('Folder A');
        await (0, test_1.expect)(testFolderA).toBeAttached();
        await sidebar.forceUpdateSorting(electronApp);
        await mainScreen.createNewNote('A note in Folder A');
        await (0, test_1.expect)(mainWindow.getByText('A note in Folder A')).toBeAttached();
        await mainScreen.createNewNote('Another note in Folder A');
        const testFolderB = await sidebar.createNewFolder('Folder B');
        await (0, test_1.expect)(testFolderB).toBeAttached();
        await mainScreen.createNewNote('A note in Folder B');
        const allNotesButton = sidebar.container.getByText('All notes');
        await allNotesButton.click();
        await (0, test_1.expect)(mainWindow.getByText('A note in Folder A')).toBeAttached();
        await (0, test_1.expect)(mainWindow.getByText('Another note in Folder A')).toBeAttached();
        await (0, test_1.expect)(mainWindow.getByText('A note in Folder B')).toBeAttached();
    });
    (0, test_1.test)('double-clicking should collapse/expand folders in the sidebar', async ({ mainWindow }) => {
        const mainScreen = await new MainScreen_1.default(mainWindow).setup();
        const sidebar = mainScreen.sidebar;
        const testFolderA = await sidebar.createNewFolder('Folder A');
        const testFolderB = await sidebar.createNewFolder('Folder B');
        // Convert folder B to a subfolder
        await testFolderB.dragTo(testFolderA);
        await (0, test_1.expect)(testFolderB).toBeVisible();
        // Collapse
        await testFolderA.dblclick();
        await (0, test_1.expect)(testFolderB).not.toBeVisible();
        // Expand
        await testFolderA.dblclick();
        await (0, test_1.expect)(testFolderB).toBeVisible();
    });
});
//# sourceMappingURL=sidebar.spec.js.map