"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const test_1 = require("./util/test");
const MainScreen_1 = require("./models/MainScreen");
const SettingsScreen_1 = require("./models/SettingsScreen");
const playwright_1 = require("@axe-core/playwright");
const createScanner = (page) => {
    return new playwright_1.default({ page })
        .disableRules(['page-has-heading-one'])
        .setLegacyMode(true);
};
// Fade-in transitions can cause color contrast issues if still running
// during a scan.
// See https://github.com/dequelabs/axe-core-npm/issues/952
const waitForAnimationsToEnd = (page) => {
    return page.locator('body').evaluate(element => {
        const animationPromises = element
            .getAnimations({ subtree: true })
            .map(animation => animation.finished);
        return Promise.all(animationPromises);
    });
};
const expectNoViolations = async (page) => {
    await waitForAnimationsToEnd(page);
    const results = await createScanner(page).analyze();
    (0, test_1.expect)(results.violations).toEqual([]);
};
test_1.test.describe('wcag', () => {
    for (const tabName of ['General', 'Plugins']) {
        (0, test_1.test)(`should not detect significant issues in the settings screen ${tabName} tab`, async ({ electronApp, mainWindow }) => {
            const mainScreen = await new MainScreen_1.default(mainWindow).setup();
            await mainScreen.waitFor();
            await mainScreen.openSettings(electronApp);
            // Should be on the settings screen
            const settingsScreen = new SettingsScreen_1.default(mainWindow);
            await settingsScreen.waitFor();
            const tabLocator = settingsScreen.getTabLocator(tabName);
            await tabLocator.click();
            await (0, test_1.expect)(tabLocator).toBeFocused();
            await expectNoViolations(mainWindow);
        });
    }
    (0, test_1.test)('should not detect significant issues in the main screen with an open note', async ({ mainWindow }) => {
        const mainScreen = await new MainScreen_1.default(mainWindow).setup();
        await mainScreen.waitFor();
        await mainScreen.createNewNote('Test');
        // For now, activate all notes to make it active. When inactive, it causes a contrast warning.
        // This seems to be allowed under WCAG 2.2 SC 1.4.3 under the "Incidental" exception.
        await mainScreen.sidebar.allNotes.click();
        // Ensure that `:hover` styling is consistent between tests:
        await mainScreen.noteEditor.noteTitleInput.hover();
        await expectNoViolations(mainWindow);
    });
});
//# sourceMappingURL=wcag.spec.js.map