"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const state_1 = require("@codemirror/state");
const view_1 = require("@codemirror/view");
const activeLineDecoration = view_1.Decoration.line({ class: 'CodeMirror-activeline CodeMirror-activeline-background' });
const optionToExtension = {
    'styleActiveLine': [
        view_1.ViewPlugin.fromClass(class {
            constructor(view) {
                this.updateDecorations(view);
            }
            update(update) {
                this.updateDecorations(update.view);
            }
            updateDecorations(view) {
                const builder = new state_1.RangeSetBuilder();
                let lastLine = -1;
                for (const selection of view.state.selection.ranges) {
                    const startLine = selection.from;
                    const line = view.state.doc.lineAt(startLine);
                    if (line.number !== lastLine) {
                        builder.add(line.from, line.from, activeLineDecoration);
                    }
                    lastLine = line.number;
                }
                this.decorations = builder.finish();
            }
        }, {
            decorations: plugin => plugin.decorations,
        }),
        view_1.EditorView.baseTheme({
            '&dark .CodeMirror-activeline-background': {
                background: '#3304',
                color: 'white',
            },
            '&light .CodeMirror-activeline-background': {
                background: '#7ff4',
                color: 'black',
            },
        }),
    ],
};
// Maps several CM5 options to CM6 extensions
class CodeMirror5BuiltInOptions {
    constructor(editor) {
        this.editor = editor;
        this.activeOptions = [];
        this.extensionCompartment = new state_1.Compartment();
        editor.dispatch({
            effects: state_1.StateEffect.appendConfig.of(this.extensionCompartment.of([])),
        });
    }
    updateExtensions() {
        const extensions = this.activeOptions.map(option => optionToExtension[option]);
        this.editor.dispatch({
            effects: this.extensionCompartment.reconfigure(extensions),
        });
    }
    supportsOption(option) {
        return optionToExtension.hasOwnProperty(option);
    }
    setOption(optionName, value) {
        this.activeOptions = this.activeOptions.filter(other => other !== optionName);
        if (value) {
            this.activeOptions.push(optionName);
        }
        this.updateExtensions();
    }
}
exports.default = CodeMirror5BuiltInOptions;
//# sourceMappingURL=CodeMirror5BuiltInOptions.js.map