"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const view_1 = require("@codemirror/view");
const autocomplete_1 = require("@codemirror/autocomplete");
const types_1 = require("../types");
const theme_1 = require("./theme");
const state_1 = require("@codemirror/state");
const lang_markdown_1 = require("@codemirror/lang-markdown");
const markdown_1 = require("@lezer/markdown");
const markdownMathParser_1 = require("./markdown/markdownMathParser");
const lookUpLanguage_1 = require("./markdown/codeBlockLanguages/lookUpLanguage");
const lang_html_1 = require("@codemirror/lang-html");
const commands_1 = require("@codemirror/commands");
const codemirror_vim_1 = require("@replit/codemirror-vim");
const language_1 = require("@codemirror/language");
const state_2 = require("@codemirror/state");
const configFromSettings = (settings) => {
    const languageExtension = (() => {
        const openingBrackets = '`([{\'"‘“（《「『【〔〖〘〚'.split('');
        const language = settings.language;
        if (language === types_1.EditorLanguageType.Markdown) {
            return [
                (0, lang_markdown_1.markdown)(Object.assign({ extensions: [
                        markdown_1.GFM,
                        // Don't highlight KaTeX if the user disabled it
                        settings.katexEnabled ? markdownMathParser_1.MarkdownMathExtension : [],
                    ], codeLanguages: lookUpLanguage_1.default }, (settings.autocompleteMarkup ? {
                // Most Markup completion is enabled by default
                } : {
                    addKeymap: false,
                    completeHTMLTags: false,
                    htmlTagLanguage: (0, lang_html_1.html)({ matchClosingTags: false, autoCloseTags: false }),
                }))),
                lang_markdown_1.markdownLanguage.data.of({ closeBrackets: { brackets: openingBrackets } }),
            ];
        }
        else if (language === types_1.EditorLanguageType.Html) {
            return (0, lang_html_1.html)({ autoCloseTags: settings.autocompleteMarkup });
        }
        else {
            const exhaustivenessCheck = language;
            return exhaustivenessCheck;
        }
    })();
    const extensions = [
        languageExtension,
        (0, theme_1.default)(settings.themeData),
        view_1.EditorView.contentAttributes.of({
            autocapitalize: 'sentence',
            autocorrect: settings.spellcheckEnabled ? 'true' : 'false',
            spellcheck: settings.spellcheckEnabled ? 'true' : 'false',
            'aria-label': settings.editorLabel,
        }),
        state_1.EditorState.readOnly.of(settings.readOnly),
        language_1.indentUnit.of(settings.indentWithTabs ? '\t' : '    '),
    ];
    if (settings.automatchBraces) {
        extensions.push((0, autocomplete_1.closeBrackets)());
        extensions.push(view_1.keymap.of(autocomplete_1.closeBracketsKeymap));
    }
    if (settings.keymap === types_1.EditorKeymap.Vim) {
        extensions.push(state_2.Prec.high((0, codemirror_vim_1.vim)()));
    }
    else if (settings.keymap === types_1.EditorKeymap.Emacs) {
        extensions.push(view_1.keymap.of(commands_1.emacsStyleKeymap));
    }
    if (!settings.ignoreModifiers) {
        extensions.push(state_2.Prec.low(view_1.keymap.of(commands_1.defaultKeymap)));
    }
    return extensions;
};
exports.default = configFromSettings;
//# sourceMappingURL=configFromSettings.js.map