"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const types_1 = require("../../types");
const commands_1 = require("@codemirror/commands");
const markdownCommands_1 = require("../markdown/markdownCommands");
const duplicateLine_1 = require("./duplicateLine");
const sortSelectedLines_1 = require("./sortSelectedLines");
const search_1 = require("@codemirror/search");
const focusHandler_1 = require("@joplin/lib/utils/focusHandler");
const handleLinkEditRequests_1 = require("../utils/handleLinkEditRequests");
const replaceSelectionCommand = (editor, toInsert) => {
    editor.dispatch(editor.state.replaceSelection(toInsert));
};
const editorCommands = {
    [types_1.EditorCommandType.Undo]: commands_1.undo,
    [types_1.EditorCommandType.Redo]: commands_1.redo,
    [types_1.EditorCommandType.SelectAll]: commands_1.selectAll,
    [types_1.EditorCommandType.Focus]: editor => (0, focusHandler_1.focus)('editorCommands::focus', editor),
    [types_1.EditorCommandType.ToggleBolded]: markdownCommands_1.toggleBolded,
    [types_1.EditorCommandType.ToggleItalicized]: markdownCommands_1.toggleItalicized,
    [types_1.EditorCommandType.ToggleCode]: markdownCommands_1.toggleCode,
    [types_1.EditorCommandType.ToggleMath]: markdownCommands_1.toggleMath,
    [types_1.EditorCommandType.ToggleComment]: commands_1.toggleComment,
    [types_1.EditorCommandType.DuplicateLine]: duplicateLine_1.default,
    [types_1.EditorCommandType.SortSelectedLines]: sortSelectedLines_1.default,
    [types_1.EditorCommandType.ToggleNumberedList]: (0, markdownCommands_1.toggleList)(types_1.ListType.OrderedList),
    [types_1.EditorCommandType.ToggleBulletedList]: (0, markdownCommands_1.toggleList)(types_1.ListType.UnorderedList),
    [types_1.EditorCommandType.ToggleCheckList]: (0, markdownCommands_1.toggleList)(types_1.ListType.CheckList),
    [types_1.EditorCommandType.ToggleHeading]: (0, markdownCommands_1.toggleHeaderLevel)(2),
    [types_1.EditorCommandType.ToggleHeading1]: (0, markdownCommands_1.toggleHeaderLevel)(1),
    [types_1.EditorCommandType.ToggleHeading2]: (0, markdownCommands_1.toggleHeaderLevel)(2),
    [types_1.EditorCommandType.ToggleHeading3]: (0, markdownCommands_1.toggleHeaderLevel)(3),
    [types_1.EditorCommandType.ToggleHeading4]: (0, markdownCommands_1.toggleHeaderLevel)(4),
    [types_1.EditorCommandType.ToggleHeading5]: (0, markdownCommands_1.toggleHeaderLevel)(5),
    [types_1.EditorCommandType.InsertHorizontalRule]: markdownCommands_1.insertHorizontalRule,
    [types_1.EditorCommandType.ScrollSelectionIntoView]: editor => {
        editor.dispatch(editor.state.update({
            scrollIntoView: true,
        }));
    },
    [types_1.EditorCommandType.DeleteToLineEnd]: commands_1.deleteToLineEnd,
    [types_1.EditorCommandType.DeleteToLineStart]: commands_1.deleteToLineStart,
    [types_1.EditorCommandType.DeleteLine]: commands_1.deleteLine,
    [types_1.EditorCommandType.IndentMore]: markdownCommands_1.increaseIndent,
    [types_1.EditorCommandType.IndentLess]: markdownCommands_1.decreaseIndent,
    [types_1.EditorCommandType.IndentAuto]: commands_1.indentSelection,
    [types_1.EditorCommandType.InsertNewlineAndIndent]: commands_1.insertNewlineAndIndent,
    [types_1.EditorCommandType.SwapLineUp]: commands_1.moveLineUp,
    [types_1.EditorCommandType.SwapLineDown]: commands_1.moveLineDown,
    [types_1.EditorCommandType.GoDocEnd]: commands_1.cursorDocEnd,
    [types_1.EditorCommandType.GoDocStart]: commands_1.cursorDocStart,
    [types_1.EditorCommandType.GoLineStart]: commands_1.cursorLineStart,
    [types_1.EditorCommandType.GoLineEnd]: commands_1.cursorLineEnd,
    [types_1.EditorCommandType.GoLineUp]: commands_1.cursorLineUp,
    [types_1.EditorCommandType.GoLineDown]: commands_1.cursorLineDown,
    [types_1.EditorCommandType.GoPageUp]: commands_1.cursorPageUp,
    [types_1.EditorCommandType.GoPageDown]: commands_1.cursorPageDown,
    [types_1.EditorCommandType.GoCharLeft]: commands_1.cursorCharLeft,
    [types_1.EditorCommandType.GoCharRight]: commands_1.cursorCharRight,
    [types_1.EditorCommandType.UndoSelection]: commands_1.undoSelection,
    [types_1.EditorCommandType.RedoSelection]: commands_1.redoSelection,
    [types_1.EditorCommandType.EditLink]: handleLinkEditRequests_1.showLinkEditor,
    [types_1.EditorCommandType.ToggleSearch]: (view) => {
        if ((0, search_1.searchPanelOpen)(view.state)) {
            return (0, search_1.closeSearchPanel)(view);
        }
        else {
            return (0, search_1.openSearchPanel)(view);
        }
    },
    [types_1.EditorCommandType.ShowSearch]: search_1.openSearchPanel,
    [types_1.EditorCommandType.HideSearch]: search_1.closeSearchPanel,
    [types_1.EditorCommandType.FindNext]: search_1.findNext,
    [types_1.EditorCommandType.FindPrevious]: search_1.findPrevious,
    [types_1.EditorCommandType.ReplaceNext]: search_1.replaceNext,
    [types_1.EditorCommandType.ReplaceAll]: search_1.replaceAll,
    // Getter commands
    // Note that these commands aren't strictly CodeMirror 6 commands as they produce
    // output.
    [types_1.EditorCommandType.SelectedText]: editor => {
        const selection = editor.state.selection;
        return editor.state.sliceDoc(selection.main.from, selection.main.to);
    },
    [types_1.EditorCommandType.InsertText]: replaceSelectionCommand,
    [types_1.EditorCommandType.ReplaceSelection]: replaceSelectionCommand,
    [types_1.EditorCommandType.SetText]: (editor, text) => {
        editor.dispatch({
            changes: [{
                    from: 0,
                    to: editor.state.doc.length,
                    insert: text,
                }],
        });
    },
};
exports.default = editorCommands;
//# sourceMappingURL=editorCommands.js.map