"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.enableLanguageDataAutocomplete = exports.editorCompletionSource = void 0;
const state_1 = require("@codemirror/state");
const autocomplete_1 = require("@codemirror/autocomplete");
// CodeMirror 6's built-in autocomplete functionality is difficult to work with
// unless you want to enable languageData-based autocompletion.
//
// This file wraps CodeMirror's built-in `autocompletion` extension and allows plugins
// to provide completions that work regardless of whether languageData-based autocomplete
// is enabled.
//
// See https://discuss.codemirror.net/t/autocompletion-merging-override-in-config/7853
exports.editorCompletionSource = state_1.Facet.define();
exports.enableLanguageDataAutocomplete = state_1.Facet.define();
// Provides languageData OR override autocompletions based on the value of
// the enableLanguageDataAutocomplete facet.
const customEditorCompletion = () => {
    const compartment = new state_1.Compartment();
    return [
        compartment.of([]),
        state_1.EditorState.transactionExtender.of(transaction => {
            var _a, _b;
            const lastCompletions = (_a = transaction.startState.facet(exports.editorCompletionSource)) !== null && _a !== void 0 ? _a : [];
            const completions = (_b = transaction.state.facet(exports.editorCompletionSource)) !== null && _b !== void 0 ? _b : [];
            const currentExtension = compartment.get(transaction.state);
            const missingExtension = currentExtension.length === 0 && completions.length > 0;
            const lastEnableLangDataComplete = transaction.startState.facet(exports.enableLanguageDataAutocomplete)[0];
            const enableLangDataComplete = transaction.state.facet(exports.enableLanguageDataAutocomplete)[0];
            const completionsChanged = lastCompletions.length !== completions.length || completions.some((val, idx) => lastCompletions[idx] !== val);
            const useLangDataChanged = lastEnableLangDataComplete !== enableLangDataComplete;
            // Update the autocompletion extension based on the editorCompletionSource
            // facet.
            if (missingExtension || completionsChanged || useLangDataChanged) {
                if (completions.length > 0 || enableLangDataComplete) {
                    return {
                        effects: compartment.reconfigure([
                            completions.map(completion => state_1.EditorState.languageData.of(() => [{ autocomplete: completion }])),
                            (0, autocomplete_1.autocompletion)(enableLangDataComplete ? {} : {
                                override: [...completions],
                            }),
                        ]),
                    };
                }
                else if (!missingExtension) {
                    return {
                        effects: compartment.reconfigure([]),
                    };
                }
            }
            return null;
        }),
    ];
};
exports.default = customEditorCompletion;
//# sourceMappingURL=customEditorCompletion.js.map