"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const state_1 = require("@codemirror/state");
const createEditorControl_1 = require("../testUtil/createEditorControl");
const autocomplete_1 = require("@codemirror/autocomplete");
const typeText_1 = require("../testUtil/typeText");
const waitForShownCompletionsToContain = (editor, completionLabels) => {
    return new Promise(resolve => {
        let resolved = false;
        const checkCompletions = (state) => {
            if (resolved) {
                return;
            }
            const completions = (0, autocomplete_1.currentCompletions)(state).map(completion => completion.label);
            for (const result of completionLabels) {
                if (!completions.includes(result)) {
                    return;
                }
            }
            resolve(completions);
            resolved = true;
        };
        checkCompletions(editor.state);
        editor.dispatch({
            effects: state_1.StateEffect.appendConfig.of(state_1.EditorState.transactionExtender.of(transaction => {
                checkCompletions(transaction.state);
                return null;
            })),
        });
    });
};
describe('customEditorCompletion', () => {
    test('should not show completions when no completion sources have been registered', async () => {
        const editorControl = (0, createEditorControl_1.default)('');
        const completion = (0, autocomplete_1.completeFromList)(['test']);
        editorControl.addExtension([
            state_1.EditorState.languageData.of(() => [{ autocomplete: completion }]),
        ]);
        const editor = editorControl.editor;
        (0, typeText_1.default)(editor, 'tes');
        // Would be 'pending' or 'active' if trying to autocomplete.
        expect((0, autocomplete_1.completionStatus)(editor.state)).toBe(null);
    });
    test('should show languageData completions when languageData-based autocomplete is enabled', async () => {
        const editorControl = (0, createEditorControl_1.default)('');
        const completion = (0, autocomplete_1.completeFromList)(['function']);
        editorControl.addExtension([
            state_1.EditorState.languageData.of(() => [{ autocomplete: completion }]),
            editorControl.joplinExtensions.enableLanguageDataAutocomplete.of(true),
        ]);
        const editor = editorControl.editor;
        (0, typeText_1.default)(editor, 'fun');
        await waitForShownCompletionsToContain(editor, ['function']);
    });
    test.each([
        { useLanguageData: false },
        { useLanguageData: true },
    ])('should show completions from all registered sources (%j)', async ({ useLanguageData }) => {
        const editorControl = (0, createEditorControl_1.default)('');
        const completion1 = (0, autocomplete_1.completeFromList)(['completion1-test', 'completion1-test2']);
        const completion2 = (0, autocomplete_1.completeFromList)(['completion2-test', 'completion2-test2']);
        const joplinExtensions = editorControl.joplinExtensions;
        editorControl.addExtension([
            joplinExtensions.completionSource(completion1),
            joplinExtensions.completionSource(completion2),
            joplinExtensions.enableLanguageDataAutocomplete.of(useLanguageData),
        ]);
        const editor = editorControl.editor;
        (0, typeText_1.default)(editor, 'com');
        await waitForShownCompletionsToContain(editor, ['completion1-test', 'completion2-test']);
    });
});
//# sourceMappingURL=customEditorCompletion.test.js.map