"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const lang_markdown_1 = require("@codemirror/lang-markdown");
const markdown_1 = require("@lezer/markdown");
const language_1 = require("@codemirror/language");
const state_1 = require("@codemirror/state");
const view_1 = require("@codemirror/view");
const markdownMathParser_1 = require("../markdown/markdownMathParser");
const forceFullParse_1 = require("./forceFullParse");
const loadLanguages_1 = require("./loadLanguages");
// Creates and returns a minimal editor with markdown extensions. Waits to return the editor
// until all syntax tree tags in `expectedSyntaxTreeTags` exist.
const createTestEditor = async (initialText, initialSelection, expectedSyntaxTreeTags, extraExtensions = []) => {
    await (0, loadLanguages_1.default)();
    const editor = new view_1.EditorView({
        doc: initialText,
        selection: state_1.EditorSelection.create([initialSelection]),
        extensions: [
            (0, lang_markdown_1.markdown)({
                extensions: [markdownMathParser_1.MarkdownMathExtension, markdown_1.GFM],
            }),
            language_1.indentUnit.of('\t'),
            state_1.EditorState.tabSize.of(4),
            state_1.EditorState.allowMultipleSelections.of(true),
            extraExtensions,
        ],
    });
    let sawExpectedTagCount = 0;
    while (sawExpectedTagCount < expectedSyntaxTreeTags.length) {
        (0, forceFullParse_1.default)(editor.state);
        sawExpectedTagCount = 0;
        const seenTags = new Set();
        (0, language_1.syntaxTree)(editor.state).iterate({
            from: 0,
            to: editor.state.doc.length,
            enter: (node) => {
                for (const expectedTag of expectedSyntaxTreeTags) {
                    if (node.name === expectedTag) {
                        seenTags.add(node.name);
                        sawExpectedTagCount++;
                        break;
                    }
                }
            },
        });
        if (sawExpectedTagCount < expectedSyntaxTreeTags.length) {
            // const missingTags = expectedSyntaxTreeTags.filter(tagName => {
            // 	return !seenTags.has(tagName);
            // });
            // console.warn(`Didn't find all expected tags. Missing ${missingTags}. Retrying...`);
            await new Promise(resolve => {
                setTimeout(resolve, 500);
            });
        }
    }
    return editor;
};
exports.default = createTestEditor;
//# sourceMappingURL=createTestEditor.js.map