"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const state_1 = require("@codemirror/state");
const view_1 = require("@codemirror/view");
const autoTextDirectionDecoration = view_1.Decoration.line({
    attributes: { dir: 'auto' },
});
const biDirectionalTextExtension = [
    view_1.EditorView.perLineTextDirection.of(true),
    view_1.ViewPlugin.fromClass(class {
        constructor(view) {
            this.decorations = this.buildDecorations(view);
        }
        update(update) {
            if (update.docChanged || update.viewportChanged) {
                this.decorations = this.buildDecorations(update.view);
            }
        }
        buildDecorations(view) {
            const builder = new state_1.RangeSetBuilder();
            for (const { from, to } of view.visibleRanges) {
                const fromLine = view.state.doc.lineAt(from);
                const toLine = view.state.doc.lineAt(to);
                for (let i = fromLine.number; i <= toLine.number; i++) {
                    const line = view.state.doc.line(i);
                    if (line.text) {
                        builder.add(line.from, line.from, autoTextDirectionDecoration);
                    }
                }
            }
            return builder.finish();
        }
    }, { decorations: v => v.decorations }),
];
exports.default = biDirectionalTextExtension;
//# sourceMappingURL=biDirectionalTextExtension.js.map