"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RegionSpec = void 0;
const { pregQuote } = require('@joplin/lib/string-utils-common');
var RegionSpec;
(function (RegionSpec) {
    // Creates a new RegionSpec, given a simplified set of options.
    // If [config.template] is a string, it is used as both the starting and ending
    // templates.
    // Similarly, if [config.matcher] is not given, a matcher is created based on
    // [config.template].
    RegionSpec.of = (config) => {
        var _a;
        let templateStart, templateEnd;
        if (typeof config.template === 'string') {
            templateStart = config.template;
            templateEnd = config.template;
        }
        else {
            templateStart = config.template.start;
            templateEnd = config.template.end;
        }
        const matcher = (_a = config.matcher) !== null && _a !== void 0 ? _a : matcherFromTemplate(templateStart, templateEnd);
        return {
            nodeName: config.nodeName,
            template: { start: templateStart, end: templateEnd },
            matcher,
        };
    };
    const matcherFromTemplate = (start, end) => {
        // See https://stackoverflow.com/a/30851002
        const escapedStart = pregQuote(start);
        const escapedEnd = pregQuote(end);
        return {
            start: new RegExp(escapedStart, 'g'),
            end: new RegExp(escapedEnd, 'g'),
        };
    };
})(RegionSpec || (exports.RegionSpec = RegionSpec = {}));
//# sourceMappingURL=RegionSpec.js.map