"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const state_1 = require("@codemirror/state");
const findInlineMatch_1 = require("./findInlineMatch");
// Toggles whether the given selection matches the inline region specified by [spec].
//
// For example, something similar to toggleSurrounded('**', '**') would surround
// every selection range with asterisks (including the caret).
// If the selection is already surrounded by these characters, they are
// removed.
const toggleInlineRegionSurrounded = (doc, sel, spec) => {
    let content = doc.sliceString(sel.from, sel.to);
    const startMatchLen = (0, findInlineMatch_1.default)(doc, spec, sel, findInlineMatch_1.MatchSide.Start);
    const endMatchLen = (0, findInlineMatch_1.default)(doc, spec, sel, findInlineMatch_1.MatchSide.End);
    const startsWithBefore = startMatchLen >= 0;
    const endsWithAfter = endMatchLen >= 0;
    const changes = [];
    let finalSelStart = sel.from;
    let finalSelEnd = sel.to;
    if (startsWithBefore && endsWithAfter) {
        // Remove the before and after.
        content = content.substring(startMatchLen);
        content = content.substring(0, content.length - endMatchLen);
        finalSelEnd -= startMatchLen + endMatchLen;
        changes.push({
            from: sel.from,
            to: sel.to,
            insert: content,
        });
    }
    else {
        changes.push({
            from: sel.from,
            insert: spec.template.start,
        });
        changes.push({
            from: sel.to,
            insert: spec.template.end,
        });
        // If not a caret,
        if (!sel.empty) {
            // Select the surrounding chars.
            finalSelEnd += spec.template.start.length + spec.template.end.length;
        }
        else {
            // Position the caret within the added content.
            finalSelStart = sel.from + spec.template.start.length;
            finalSelEnd = finalSelStart;
        }
    }
    return {
        changes,
        range: state_1.EditorSelection.range(finalSelStart, finalSelEnd),
    };
};
exports.default = toggleInlineRegionSurrounded;
//# sourceMappingURL=toggleInlineRegionSurrounded.js.map