"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const state_1 = require("@codemirror/state");
const findInlineMatch_1 = require("./findInlineMatch");
const growSelectionToNode_1 = require("../growSelectionToNode");
const toggleInlineRegionSurrounded_1 = require("./toggleInlineRegionSurrounded");
// Returns updated selections: For all selections in the given `EditorState`, toggles
// whether each is contained in an inline region of type [spec].
const toggleInlineSelectionFormat = (state, spec, sel) => {
    const endMatchLen = (0, findInlineMatch_1.default)(state.doc, spec, sel, findInlineMatch_1.MatchSide.End);
    // If at the end of the region, move the
    // caret to the end.
    // E.g.
    //   **foobar|**
    //   **foobar**|
    if (sel.empty && endMatchLen > -1) {
        const newCursorPos = sel.from + endMatchLen;
        return {
            range: state_1.EditorSelection.cursor(newCursorPos),
        };
    }
    // Grow the selection to encompass the entire node.
    const newRange = (0, growSelectionToNode_1.default)(state, sel, spec.nodeName);
    return (0, toggleInlineRegionSurrounded_1.default)(state.doc, newRange, spec);
};
exports.default = toggleInlineSelectionFormat;
//# sourceMappingURL=toggleInlineSelectionFormat.js.map