"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const state_1 = require("@codemirror/state");
const RegionSpec_1 = require("./RegionSpec");
const toggleRegionFormatGlobally_1 = require("./toggleRegionFormatGlobally");
describe('toggleRegionFormatGlobally', () => {
    jest.retryTimes(2);
    const multiLineTestText = `Internal text manipulation
		This is a test...
		of block and inline region toggling.`;
    const codeFenceRegex = /^``````\w*\s*$/;
    const inlineCodeRegionSpec = RegionSpec_1.RegionSpec.of({
        template: '`',
        nodeName: 'InlineCode',
    });
    const blockCodeRegionSpec = {
        template: { start: '``````', end: '``````' },
        matcher: { start: codeFenceRegex, end: codeFenceRegex },
    };
    it('should create an empty inline region around the cursor, if given an empty selection', () => {
        const initialState = state_1.EditorState.create({
            doc: multiLineTestText,
            selection: state_1.EditorSelection.cursor(0),
        });
        const changes = (0, toggleRegionFormatGlobally_1.default)(initialState, inlineCodeRegionSpec, blockCodeRegionSpec);
        const newState = initialState.update(changes).state;
        expect(newState.doc.toString()).toEqual(`\`\`${multiLineTestText}`);
    });
    it('should wrap multiple selected lines in block formatting', () => {
        const initialState = state_1.EditorState.create({
            doc: multiLineTestText,
            selection: state_1.EditorSelection.range(0, multiLineTestText.length),
        });
        const changes = (0, toggleRegionFormatGlobally_1.default)(initialState, inlineCodeRegionSpec, blockCodeRegionSpec);
        const newState = initialState.update(changes).state;
        const editorText = newState.doc.toString();
        expect(editorText).toBe(`\`\`\`\`\`\`\n${multiLineTestText}\n\`\`\`\`\`\``);
        expect(newState.selection.main.from).toBe(0);
        expect(newState.selection.main.to).toBe(editorText.length);
    });
});
//# sourceMappingURL=toggleRegionFormatGlobally.test.js.map