"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const language_1 = require("@codemirror/language");
const state_1 = require("@codemirror/state");
// Expands and returns a copy of [sel] to the smallest container node with name in [nodeNames].
const growSelectionToNode = (state, sel, nodeNames) => {
    if (!nodeNames) {
        return sel;
    }
    const isAcceptableNode = (name) => {
        if (typeof nodeNames === 'string') {
            return name === nodeNames;
        }
        for (const otherName of nodeNames) {
            if (otherName === name) {
                return true;
            }
        }
        return false;
    };
    let newFrom = null;
    let newTo = null;
    let smallestLen = Infinity;
    // Find the smallest range.
    (0, language_1.syntaxTree)(state).iterate({
        from: sel.from, to: sel.to,
        enter: node => {
            if (isAcceptableNode(node.name)) {
                if (node.to - node.from < smallestLen) {
                    newFrom = node.from;
                    newTo = node.to;
                    smallestLen = newTo - newFrom;
                }
            }
        },
    });
    // If it's in such a node,
    if (newFrom !== null && newTo !== null) {
        return state_1.EditorSelection.range(newFrom, newTo);
    }
    else {
        return sel;
    }
};
exports.default = growSelectionToNode;
//# sourceMappingURL=growSelectionToNode.js.map