"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const view_1 = require("@codemirror/view");
// CodeMirror's built-in keyboard event handlers trigger on key down, rather than on
// key up. In some cases, it's useful to trigger keyboard shortcuts on key up instead.
// For example, if a shortcut should only activate if it isn't pressed at the same time
// as other keys.
const keyUpHandlerExtension = (isTargetKey, onKeyUp) => {
    return view_1.ViewPlugin.fromClass(class {
        constructor(view) {
            this.view = view;
            this.otherKeysWerePressed = false;
            this.targetKeyDown = false;
            this.onKeyDown = (event) => {
                if (isTargetKey(event)) {
                    this.targetKeyDown = true;
                }
                else if (this.targetKeyDown) {
                    this.otherKeysWerePressed = true;
                }
            };
            this.onKeyUp = (event) => {
                if (isTargetKey(event)) {
                    if (this.targetKeyDown && !event.defaultPrevented) {
                        const handled = onKeyUp({
                            domEvent: event,
                            view: this.view,
                            otherKeysWerePressed: this.otherKeysWerePressed,
                        });
                        if (handled) {
                            event.preventDefault();
                        }
                    }
                    this.targetKeyDown = false;
                    this.otherKeysWerePressed = false;
                }
                else if (this.targetKeyDown) {
                    this.otherKeysWerePressed = true;
                }
            };
            view.contentDOM.addEventListener('keydown', this.onKeyDown);
            view.contentDOM.addEventListener('keyup', this.onKeyUp);
        }
        destroy() {
            var _a, _b;
            (_b = (_a = this.view) === null || _a === void 0 ? void 0 : _a.contentDOM) === null || _b === void 0 ? void 0 : _b.removeEventListener('keyup', this.onKeyUp);
        }
    });
};
exports.default = keyUpHandlerExtension;
//# sourceMappingURL=keyUpHandlerExtension.js.map