"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.overwriteModeEnabled = exports.toggleOverwrite = void 0;
const view_1 = require("@codemirror/view");
const state_1 = require("@codemirror/state");
const keyUpHandlerExtension_1 = require("./keyUpHandlerExtension");
const overwriteModeFacet = state_1.Facet.define({
    combine: values => { var _a; return (_a = values[0]) !== null && _a !== void 0 ? _a : false; },
    enables: facet => [
        view_1.EditorView.inputHandler.of((view, _from, _to, text, insert) => {
            if (view.composing || view.compositionStarted || view.state.readOnly) {
                return false;
            }
            if (view.state.facet(facet) && text) {
                const originalTransaction = insert();
                const newState1 = originalTransaction.state;
                const emptySelections1 = newState1.selection.ranges.filter(range => range.empty);
                view.dispatch([
                    originalTransaction,
                    newState1.update({
                        changes: emptySelections1.map(range => {
                            const line = newState1.doc.lineAt(range.to);
                            return {
                                from: range.to,
                                to: Math.min(line.to, range.to + text.length),
                                insert: '',
                            };
                        }).filter(change => change.from !== change.to || change.insert),
                    }),
                ]);
                return true;
            }
            return false;
        }),
        view_1.EditorView.theme({
            '&.-overwrite .cm-cursor': {
                borderLeftWidth: '0.5em',
            },
        }),
    ],
});
exports.toggleOverwrite = state_1.StateEffect.define();
const overwriteModeState = state_1.StateField.define({
    create: () => false,
    update: (oldValue, tr) => {
        for (const e of tr.effects) {
            if (e.is(exports.toggleOverwrite)) {
                return e.value;
            }
        }
        return oldValue;
    },
    provide: (field) => [
        overwriteModeFacet.from(field),
        view_1.EditorView.editorAttributes.from(field, on => ({
            class: on ? '-overwrite' : '',
        })),
    ],
});
const overwriteModeEnabled = (view) => {
    return view.state.field(overwriteModeState);
};
exports.overwriteModeEnabled = overwriteModeEnabled;
const setOverwriteModeEnabled = (enabled, view) => {
    view.dispatch({
        effects: [
            exports.toggleOverwrite.of(enabled),
            view_1.EditorView.announce.of(
            // TODO: Localize:
            enabled ? 'Overwrite mode enabled' : 'Overwrite mode disabled'),
        ],
    });
};
const overwriteModeExtension = [
    overwriteModeState,
    view_1.keymap.of([
        {
            // The <escape> keyboard shortcut may be more easily discoverable for users
            // who enter overwrite mode unintentionally.
            key: 'Escape',
            run: (view) => {
                if ((0, exports.overwriteModeEnabled)(view)) {
                    setOverwriteModeEnabled(false, view);
                    return true;
                }
                return false;
            },
        },
    ]),
    (0, keyUpHandlerExtension_1.default)((event) => (event.code === 'Insert' && !event.shiftKey && !event.altKey && !event.metaKey && !event.ctrlKey), ({ view, otherKeysWerePressed }) => {
        if (otherKeysWerePressed) {
            return false;
        }
        setOverwriteModeEnabled(!(0, exports.overwriteModeEnabled)(view), view);
        return true;
    }),
];
exports.default = overwriteModeExtension;
//# sourceMappingURL=overwriteModeExtension.js.map