"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ListType = exports.EditorKeymap = exports.EditorLanguageType = exports.UserEventSource = exports.EditorCommandType = void 0;
// Editor commands. For compatibility, the string values of these commands
// should correspond with the CodeMirror 5 commands:
// https://codemirror.net/5/doc/manual.html#commands
var EditorCommandType;
(function (EditorCommandType) {
    EditorCommandType["Undo"] = "undo";
    EditorCommandType["Redo"] = "redo";
    EditorCommandType["SelectAll"] = "selectAll";
    EditorCommandType["Focus"] = "focus";
    // Formatting editor commands
    EditorCommandType["ToggleBolded"] = "textBold";
    EditorCommandType["ToggleItalicized"] = "textItalic";
    EditorCommandType["ToggleCode"] = "textCode";
    EditorCommandType["ToggleMath"] = "textMath";
    EditorCommandType["ToggleComment"] = "toggleComment";
    EditorCommandType["DuplicateLine"] = "duplicateLine";
    EditorCommandType["SortSelectedLines"] = "sortSelectedLines";
    EditorCommandType["ToggleNumberedList"] = "textNumberedList";
    EditorCommandType["ToggleBulletedList"] = "textBulletedList";
    EditorCommandType["ToggleCheckList"] = "textCheckbox";
    EditorCommandType["ToggleHeading"] = "textHeading";
    EditorCommandType["ToggleHeading1"] = "textHeading1";
    EditorCommandType["ToggleHeading2"] = "textHeading2";
    EditorCommandType["ToggleHeading3"] = "textHeading3";
    EditorCommandType["ToggleHeading4"] = "textHeading4";
    EditorCommandType["ToggleHeading5"] = "textHeading5";
    EditorCommandType["InsertHorizontalRule"] = "textHorizontalRule";
    // Find commands
    EditorCommandType["ToggleSearch"] = "textSearch";
    EditorCommandType["ShowSearch"] = "find";
    EditorCommandType["HideSearch"] = "hideSearchDialog";
    EditorCommandType["FindNext"] = "findNext";
    EditorCommandType["FindPrevious"] = "findPrev";
    EditorCommandType["ReplaceNext"] = "replace";
    EditorCommandType["ReplaceAll"] = "replaceAll";
    EditorCommandType["EditLink"] = "textLink";
    // Editing and navigation commands
    EditorCommandType["ScrollSelectionIntoView"] = "scrollSelectionIntoView";
    EditorCommandType["DeleteLine"] = "deleteLine";
    EditorCommandType["DeleteToLineEnd"] = "killLine";
    EditorCommandType["DeleteToLineStart"] = "delLineLeft";
    EditorCommandType["IndentMore"] = "indentMore";
    EditorCommandType["IndentLess"] = "indentLess";
    EditorCommandType["IndentAuto"] = "indentAuto";
    EditorCommandType["InsertNewlineAndIndent"] = "newlineAndIndent";
    EditorCommandType["SwapLineUp"] = "swapLineUp";
    EditorCommandType["SwapLineDown"] = "swapLineDown";
    EditorCommandType["GoDocEnd"] = "goDocEnd";
    EditorCommandType["GoDocStart"] = "goDocStart";
    EditorCommandType["GoLineStart"] = "goLineStart";
    EditorCommandType["GoLineEnd"] = "goLineEnd";
    EditorCommandType["GoLineUp"] = "goLineUp";
    EditorCommandType["GoLineDown"] = "goLineDown";
    EditorCommandType["GoPageUp"] = "goPageUp";
    EditorCommandType["GoPageDown"] = "goPageDown";
    EditorCommandType["GoCharLeft"] = "goCharLeft";
    EditorCommandType["GoCharRight"] = "goCharRight";
    EditorCommandType["UndoSelection"] = "undoSelection";
    EditorCommandType["RedoSelection"] = "redoSelection";
    // Getters and multi-argument commands. These correspond to global Joplin
    // commands.
    EditorCommandType["SelectedText"] = "selectedText";
    EditorCommandType["InsertText"] = "insertText";
    EditorCommandType["ReplaceSelection"] = "replaceSelection";
    EditorCommandType["SetText"] = "setText";
})(EditorCommandType || (exports.EditorCommandType = EditorCommandType = {}));
// Intended to correspond with https://codemirror.net/docs/ref/#state.Transaction%5EuserEvent
var UserEventSource;
(function (UserEventSource) {
    UserEventSource["Paste"] = "input.paste";
    UserEventSource["Drop"] = "input.drop";
})(UserEventSource || (exports.UserEventSource = UserEventSource = {}));
var EditorLanguageType;
(function (EditorLanguageType) {
    EditorLanguageType["Markdown"] = "markdown";
    EditorLanguageType["Html"] = "html";
})(EditorLanguageType || (exports.EditorLanguageType = EditorLanguageType = {}));
var EditorKeymap;
(function (EditorKeymap) {
    EditorKeymap["Default"] = "default";
    EditorKeymap["Vim"] = "vim";
    EditorKeymap["Emacs"] = "emacs";
})(EditorKeymap || (exports.EditorKeymap = EditorKeymap = {}));
// Possible types of lists in the editor
var ListType;
(function (ListType) {
    ListType[ListType["CheckList"] = 0] = "CheckList";
    ListType[ListType["OrderedList"] = 1] = "OrderedList";
    ListType[ListType["UnorderedList"] = 2] = "UnorderedList";
})(ListType || (exports.ListType = ListType = {}));
//# sourceMappingURL=types.js.map