"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const markdownUtils_1 = require("./markdownUtils");
describe('Should detect list items', () => {
    test('should detect `- lorem ipsum` as list item ', () => {
        expect(markdownUtils_1.default.isListItem('- lorem ipsum')).toBe(true);
    });
    test('should detect `+ lorem ipsum` as list item ', () => {
        expect(markdownUtils_1.default.isListItem('+ lorem ipsum')).toBe(true);
    });
    test('should detect `* lorem ipsum` as list item ', () => {
        expect(markdownUtils_1.default.isListItem('* lorem ipsum')).toBe(true);
    });
    // ordered list
    test('should detect `1. lorem ipsum` as list item ', () => {
        expect(markdownUtils_1.default.isListItem('1. lorem ipsum')).toBe(true);
    });
    test('should detect `1) lorem ipsum` as list item ', () => {
        expect(markdownUtils_1.default.isListItem('1) lorem ipsum')).toBe(true);
    });
    // checkbox list
    test('should detect `+ [x] lorem ipsum` as list item ', () => {
        expect(markdownUtils_1.default.isListItem('+ [x] lorem ipsum')).toBe(true);
    });
    // ordered list
    test('should NOT detect `-lorem ipsum` as list item ', () => {
        expect(markdownUtils_1.default.isListItem('-lorem ipsum')).toBe(false);
    });
    test('should NOT detect `+lorem ipsum` as list item ', () => {
        expect(markdownUtils_1.default.isListItem('+lorem ipsum')).toBe(false);
    });
    test('should NOT detect `*lorem ipsum` as list item ', () => {
        expect(markdownUtils_1.default.isListItem('*lorem ipsum')).toBe(false);
    });
    // ordered list
    test('should NOT detect `1.lorem ipsum` as list item ', () => {
        expect(markdownUtils_1.default.isListItem('1.lorem ipsum')).toBe(false);
    });
    test('should NOT detect `1)lorem ipsum` as list item ', () => {
        expect(markdownUtils_1.default.isListItem('1)lorem ipsum')).toBe(false);
    });
    test('should NOT detect `+[x]lorem ipsum` as list item ', () => {
        expect(markdownUtils_1.default.isListItem('+[x]lorem ipsum')).toBe(false);
    });
    // Empty list detection
    test('should detect `- ` as empty list item ', () => {
        expect(markdownUtils_1.default.isEmptyListItem('- ')).toBe(true);
    });
    test('should detect `+ ` as empty list item ', () => {
        expect(markdownUtils_1.default.isEmptyListItem('+ ')).toBe(true);
    });
    test('should detect `* ` as empty list item ', () => {
        expect(markdownUtils_1.default.isEmptyListItem('* ')).toBe(true);
    });
    // ordered list
    test('should detect `1. ` as empty list item ', () => {
        expect(markdownUtils_1.default.isEmptyListItem('1. ')).toBe(true);
    });
    test('should detect `1) ` as empty list item ', () => {
        expect(markdownUtils_1.default.isEmptyListItem('1) ')).toBe(true);
    });
    // checkbox list
    test('should detect `+ [x] ` as empty list item ', () => {
        expect(markdownUtils_1.default.isEmptyListItem('+ [x] ')).toBe(true);
    });
    // unordered list
    test('should NOT detect `-` as empty list item ', () => {
        expect(markdownUtils_1.default.isEmptyListItem('-')).toBe(false);
    });
    test('should NOT detect `+` as empty list item ', () => {
        expect(markdownUtils_1.default.isEmptyListItem('+')).toBe(false);
    });
    test('should NOT detect `*` as empty list item ', () => {
        expect(markdownUtils_1.default.isEmptyListItem('*')).toBe(false);
    });
    // ordered list
    test('should NOT detect `1.` as empty list item ', () => {
        expect(markdownUtils_1.default.isEmptyListItem('1.')).toBe(false);
    });
    test('should NOT detect `1)` as empty list item ', () => {
        expect(markdownUtils_1.default.isEmptyListItem('1)')).toBe(false);
    });
    // checkbox list
    test('should NOT detect `+ [x]` as empty list item ', () => {
        expect(markdownUtils_1.default.isEmptyListItem('+ [x]')).toBe(false);
    });
});
//# sourceMappingURL=markdownUtils.test.js.map