"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const execa = require("execa");
const commandToString_1 = require("./commandToString");
const splitCommandString_1 = require("./splitCommandString");
const process_1 = require("process");
exports.default = async (command, options = null) => {
    options = Object.assign({ showInput: true, showStdout: true, showStderr: true, quiet: false, env: {} }, options);
    if (options.quiet) {
        options.showInput = false;
        options.showStdout = false;
        options.showStderr = false;
    }
    if (options.showInput) {
        if (typeof command === 'string') {
            process_1.stdout.write(`> ${command}\n`);
        }
        else {
            process_1.stdout.write(`> ${(0, commandToString_1.default)(command[0], command.slice(1))}\n`);
        }
    }
    const args = typeof command === 'string' ? (0, splitCommandString_1.default)(command) : command;
    const executableName = args[0];
    args.splice(0, 1);
    const promise = execa(executableName, args, { env: options.env });
    if (options.showStdout && promise.stdout)
        promise.stdout.pipe(process.stdout);
    if (options.showStderr && promise.stderr)
        promise.stderr.pipe(process.stderr);
    const result = await promise;
    return result.stdout.trim();
};
//# sourceMappingURL=execCommand.js.map