"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ContainerType = void 0;
const ViewController_1 = require("./ViewController");
const shim_1 = require("../../shim");
const { toSystemSlashes } = require('../../path-utils');
const PostMessageService_1 = require("../PostMessageService");
const reducer_1 = require("../../reducer");
const Logger_1 = require("@joplin/utils/Logger");
const CommandService_1 = require("../CommandService");
const logger = Logger_1.default.create('WebviewController');
var ContainerType;
(function (ContainerType) {
    ContainerType["Panel"] = "panel";
    ContainerType["Dialog"] = "dialog";
    ContainerType["Editor"] = "editor";
})(ContainerType || (exports.ContainerType = ContainerType = {}));
// TODO: Copied from:
// packages/app-desktop/gui/ResizableLayout/utils/findItemByKey.ts
// eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
function findItemByKey(layout, key) {
    if (!layout)
        throw new Error('Layout cannot be null');
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    function recurseFind(item) {
        if (item.key === key)
            return item;
        if (item.children) {
            for (const child of item.children) {
                const found = recurseFind(child);
                if (found)
                    return found;
            }
        }
        return null;
    }
    return recurseFind(layout);
}
class WebviewController extends ViewController_1.default {
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    constructor(handle, pluginId, store, baseDir, containerType) {
        super(handle, pluginId, store);
        // eslint-disable-next-line @typescript-eslint/ban-types -- Old code before rule was applied
        this.messageListener_ = null;
        this.updateListener_ = null;
        this.closeResponse_ = null;
        this.containerType_ = null;
        this.baseDir_ = toSystemSlashes(baseDir, 'linux');
        this.containerType_ = containerType;
        const view = {
            id: this.handle,
            type: this.type,
            containerType: containerType,
            html: '',
            scripts: [],
            // Opened is used for dialogs and mobile panels (which are shown
            // like dialogs):
            opened: containerType === ContainerType.Panel,
            buttons: null,
            fitToContent: true,
        };
        this.store.dispatch({
            type: 'PLUGIN_VIEW_ADD',
            pluginId: pluginId,
            view,
        });
    }
    get type() {
        return 'webview';
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    setStoreProp(name, value) {
        this.store.dispatch({
            type: 'PLUGIN_VIEW_PROP_SET',
            pluginId: this.pluginId,
            id: this.handle,
            name: name,
            value: value,
        });
    }
    get html() {
        return this.storeView.html;
    }
    set html(html) {
        this.setStoreProp('html', html);
    }
    get containerType() {
        return this.storeView.containerType;
    }
    async addScript(path) {
        const fullPath = toSystemSlashes(shim_1.default.fsDriver().resolve(`${this.baseDir_}/${path}`), 'linux');
        if (fullPath.indexOf(this.baseDir_) !== 0)
            throw new Error(`Script appears to be outside of plugin base directory: ${fullPath} (Base dir: ${this.baseDir_})`);
        this.store.dispatch({
            type: 'PLUGIN_VIEW_PROP_PUSH',
            pluginId: this.pluginId,
            id: this.handle,
            name: 'scripts',
            value: fullPath,
        });
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    postMessage(message) {
        const messageId = `plugin_${Date.now()}${Math.random()}`;
        void PostMessageService_1.default.instance().postMessage({
            pluginId: this.pluginId,
            viewId: this.handle,
            windowId: reducer_1.defaultWindowId,
            contentScriptId: null,
            from: PostMessageService_1.MessageParticipant.Plugin,
            to: PostMessageService_1.MessageParticipant.UserWebview,
            id: messageId,
            content: message,
        });
    }
    async emitMessage(event) {
        if (!this.messageListener_)
            return;
        if (this.containerType_ === ContainerType.Editor && !this.isActive()) {
            logger.info('emitMessage: Not emitting message because editor is disabled:', this.pluginId, this.handle);
            return;
        }
        return this.messageListener_(event.message);
    }
    emitUpdate() {
        if (!this.updateListener_)
            return;
        if (this.containerType_ === ContainerType.Editor && (!this.isActive() || !this.isVisible())) {
            logger.info('emitMessage: Not emitting update because editor is disabled or hidden:', this.pluginId, this.handle, this.isActive(), this.isVisible());
            return;
        }
        this.updateListener_();
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    onMessage(callback) {
        this.messageListener_ = callback;
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    onUpdate(callback) {
        this.updateListener_ = callback;
    }
    // ---------------------------------------------
    // Specific to panels
    // ---------------------------------------------
    showWithAppLayout() {
        return this.containerType === ContainerType.Panel && !!this.store.getState().mainLayout;
    }
    async show(show = true) {
        if (this.showWithAppLayout()) {
            this.store.dispatch({
                type: 'MAIN_LAYOUT_SET_ITEM_PROP',
                itemKey: this.handle,
                propName: 'visible',
                propValue: show,
            });
        }
        else {
            this.setStoreProp('opened', show);
        }
    }
    async hide() {
        return this.show(false);
    }
    get visible() {
        const appState = this.store.getState();
        // Mobile: There is no appState.mainLayout
        if (!this.showWithAppLayout()) {
            return this.storeView.opened;
        }
        const mainLayout = appState.mainLayout;
        const item = findItemByKey(mainLayout, this.handle);
        return item ? item.visible : false;
    }
    // ---------------------------------------------
    // Specific to dialogs
    // ---------------------------------------------
    async open() {
        if (this.closeResponse_) {
            this.closeResponse_.resolve(null);
            this.closeResponse_ = null;
        }
        this.store.dispatch({
            type: 'VISIBLE_DIALOGS_ADD',
            name: this.handle,
        });
        this.setStoreProp('opened', true);
        // eslint-disable-next-line @typescript-eslint/ban-types -- Old code before rule was applied
        return new Promise((resolve, reject) => {
            this.closeResponse_ = { resolve, reject };
        });
    }
    close() {
        this.store.dispatch({
            type: 'VISIBLE_DIALOGS_REMOVE',
            name: this.handle,
        });
        this.setStoreProp('opened', false);
    }
    closeWithResponse(result) {
        this.close();
        this.closeResponse_.resolve(result);
        this.closeResponse_ = null;
    }
    get buttons() {
        return this.storeView.buttons;
    }
    set buttons(buttons) {
        this.setStoreProp('buttons', buttons);
    }
    get fitToContent() {
        return this.storeView.fitToContent;
    }
    set fitToContent(fitToContent) {
        this.setStoreProp('fitToContent', fitToContent);
    }
    // ---------------------------------------------
    // Specific to editors
    // ---------------------------------------------
    setActive(active) {
        this.setStoreProp('opened', active);
    }
    isActive() {
        return this.storeView.opened;
    }
    async isVisible() {
        if (!this.storeView.opened)
            return false;
        const shownEditorViewIds = this.store.getState().settings['plugins.shownEditorViewIds'];
        return shownEditorViewIds.includes(this.handle);
    }
    async setVisible(visible) {
        await CommandService_1.default.instance().execute('showEditorPlugin', this.handle, visible);
    }
}
exports.default = WebviewController;
//# sourceMappingURL=WebviewController.js.map