"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const bridge_1 = require("../bridge");
const Setting_1 = require("@joplin/lib/models/Setting");
const registry_1 = require("@joplin/lib/registry");
const BasePlatformImplementation_1 = require("@joplin/lib/services/plugins/BasePlatformImplementation");
const shim_1 = require("@joplin/lib/shim");
const path_1 = require("path");
const uuid_1 = require("@joplin/lib/uuid");
const url_1 = require("@joplin/utils/url");
const path_2 = require("@joplin/utils/path");
const { clipboard, nativeImage } = require('electron');
const packageInfo = require('../../packageInfo');
// PlatformImplementation provides access to platform specific dependencies,
// such as the clipboard, message dialog, etc. It allows having the same plugin
// API for all platforms, but with different implementations.
class PlatformImplementation extends BasePlatformImplementation_1.default {
    static instance() {
        if (!this.instance_)
            this.instance_ = new PlatformImplementation();
        return this.instance_;
    }
    get versionInfo() {
        return {
            version: packageInfo.version,
            syncVersion: Setting_1.default.value('syncVersion'),
            profileVersion: registry_1.reg.db().version(),
            platform: 'desktop',
        };
    }
    get clipboard() {
        return clipboard;
    }
    get nativeImage() {
        return nativeImage;
    }
    constructor() {
        super();
        this.components_ = {};
        this.joplin_ = {
            views: {
                dialogs: {
                    showMessageBox: async (message) => {
                        return (0, bridge_1.default)().showMessageBox(message);
                    },
                    showOpenDialog: async (options) => {
                        return (0, bridge_1.default)().showOpenDialog(options);
                    },
                },
            },
        };
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    registerComponent(name, component) {
        this.components_[name] = component;
    }
    unregisterComponent(name) {
        delete this.components_[name];
    }
    get joplin() {
        return this.joplin_;
    }
    get imaging() {
        const createFromPdf = async (path, options) => {
            const tempDir = (0, path_1.join)(Setting_1.default.value('tempDir'), uuid_1.default.createNano());
            await shim_1.default.fsDriver().mkdir(tempDir);
            try {
                const paths = await shim_1.default.pdfToImages(path, tempDir, options);
                return paths.map(path => nativeImage.createFromPath(path));
            }
            finally {
                await shim_1.default.fsDriver().remove(tempDir);
            }
        };
        return {
            createFromPath: async (path) => {
                let pathToProcess = path;
                let ext = (0, path_2.fileExtension)(path).toLowerCase();
                if ((0, url_1.hasProtocol)(path, ['http', 'https', 'file'])) {
                    ext = (0, path_2.fileExtension)((new URL(path)).pathname);
                    const tempFilePath = `${Setting_1.default.value('tempDir')}/${(0, uuid_1.uuidgen)()}${ext ? `.${ext}` : ''}`;
                    await shim_1.default.fetchBlob(path, { path: tempFilePath });
                    pathToProcess = tempFilePath;
                }
                if (ext === 'pdf') {
                    const images = await createFromPdf(pathToProcess, { minPage: 1, maxPage: 1 });
                    if (images.length === 0) {
                        // Match the behavior or Electron's nativeImage when reading an invalid image.
                        return nativeImage.createEmpty();
                    }
                    return images[0];
                }
                else {
                    return nativeImage.createFromPath(pathToProcess);
                }
            },
            createFromPdf,
            getPdfInfo(path) {
                return shim_1.default.pdfInfo(path);
            },
        };
    }
}
exports.default = PlatformImplementation;
//# sourceMappingURL=PlatformImplementation.js.map