"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const react_1 = require("react");
const useViewIsReady_1 = require("./hooks/useViewIsReady");
const useThemeCss_1 = require("./hooks/useThemeCss");
const useContentSize_1 = require("./hooks/useContentSize");
const useSubmitHandler_1 = require("./hooks/useSubmitHandler");
const useHtmlLoader_1 = require("./hooks/useHtmlLoader");
const useWebviewToPluginMessages_1 = require("./hooks/useWebviewToPluginMessages");
const useScriptLoader_1 = require("./hooks/useScriptLoader");
const Logger_1 = require("@joplin/utils/Logger");
const focusHandler_1 = require("@joplin/lib/utils/focusHandler");
const NewWindowOrIFrame_1 = require("../../gui/NewWindowOrIFrame");
const logger = Logger_1.default.create('UserWebview');
// eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
function serializeForm(form) {
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    const output = {};
    const formData = new FormData(form);
    for (const key of formData.keys()) {
        output[key] = formData.get(key);
    }
    return output;
}
// eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
function serializeForms(document) {
    const forms = document.getElementsByTagName('form');
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    const output = {};
    let untitledIndex = 0;
    for (const form of forms) {
        const name = `${form.getAttribute('name')}` || (`form${untitledIndex++}`);
        output[name] = serializeForm(form);
    }
    return output;
}
// eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
function UserWebview(props, ref) {
    const minWidth = props.minWidth ? props.minWidth : 200;
    const minHeight = props.minHeight ? props.minHeight : 20;
    const viewRef = (0, react_1.useRef)(null);
    const isReady = (0, useViewIsReady_1.default)(viewRef);
    const cssFilePath = (0, useThemeCss_1.default)({ pluginId: props.pluginId, themeId: props.themeId });
    (0, react_1.useEffect)(() => {
        if (isReady && props.onReady)
            props.onReady();
        // eslint-disable-next-line @seiyab/react-hooks/exhaustive-deps -- Old code before rule was applied
    }, [isReady]);
    function frameWindow() {
        if (!viewRef.current)
            return null;
        return viewRef.current.contentWindow;
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    function postMessage(name, args = null) {
        const win = frameWindow();
        if (!win)
            return;
        logger.debug('Got message', name, args);
        win.postMessage({ target: 'webview', name, args }, '*');
    }
    (0, react_1.useImperativeHandle)(ref, () => {
        return {
            formData: function () {
                if (viewRef.current) {
                    return serializeForms(frameWindow().document);
                }
                else {
                    return null;
                }
            },
            focus: function () {
                if (viewRef.current)
                    (0, focusHandler_1.focus)('UserWebView::focus', viewRef.current);
            },
        };
    });
    const htmlHash = (0, useHtmlLoader_1.default)(frameWindow(), isReady, postMessage, props.html);
    const contentSize = (0, useContentSize_1.default)(frameWindow(), htmlHash, minWidth, minHeight, props.fitToContent, isReady);
    (0, useSubmitHandler_1.default)(frameWindow(), props.onSubmit, props.onDismiss, htmlHash);
    const windowId = (0, react_1.useContext)(NewWindowOrIFrame_1.WindowIdContext);
    (0, useWebviewToPluginMessages_1.default)(frameWindow(), isReady, props.pluginId, props.viewId, windowId, postMessage);
    (0, useScriptLoader_1.default)(postMessage, isReady, props.scripts, cssFilePath);
    const style = (0, react_1.useMemo)(() => ({
        '--content-width': `${contentSize.width}px`,
        '--content-height': `${contentSize.height}px`,
    }), [contentSize.width, contentSize.height]);
    return React.createElement("iframe", { id: props.viewId, style: style, className: `plugin-user-webview ${props.fitToContent ? '-fit-to-content' : ''} ${props.borderBottom ? '-border-bottom' : ''}`, ref: viewRef, src: "services/plugins/UserWebviewIndex.html" });
}
exports.default = (0, react_1.forwardRef)(UserWebview);
//# sourceMappingURL=UserWebview.js.map