"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const react_1 = require("react");
const PluginService_1 = require("@joplin/lib/services/plugins/PluginService");
const UserWebview_1 = require("./UserWebview");
const UserWebviewDialogButtonBar_1 = require("./UserWebviewDialogButtonBar");
const focusHandler_1 = require("@joplin/lib/utils/focusHandler");
const Dialog_1 = require("../../gui/Dialog");
function defaultButtons() {
    return [
        {
            id: 'ok',
        },
        {
            id: 'cancel',
        },
    ];
}
function findSubmitButton(buttons) {
    return buttons.find((b) => {
        return ['ok', 'yes', 'confirm', 'submit'].includes(b.id);
    });
}
function findDismissButton(buttons) {
    return buttons.find((b) => {
        return ['cancel', 'no', 'reject'].includes(b.id);
    });
}
function UserWebviewDialog(props) {
    const webviewRef = (0, react_1.useRef)(null);
    function viewController() {
        return PluginService_1.default.instance().pluginById(props.pluginId).viewController(props.viewId);
    }
    const buttons = (props.buttons ? props.buttons : defaultButtons()).map((b) => {
        return Object.assign(Object.assign({}, b), { onClick: () => {
                const response = { id: b.id };
                const formData = webviewRef.current.formData();
                if (formData && Object.keys(formData).length)
                    response.formData = formData;
                viewController().closeWithResponse(response);
            } });
    });
    const onSubmit = (0, react_1.useCallback)(() => {
        const submitButton = findSubmitButton(buttons);
        if (submitButton) {
            submitButton.onClick();
        }
    }, [buttons]);
    const onDismiss = (0, react_1.useCallback)(() => {
        const dismissButton = findDismissButton(buttons);
        if (dismissButton) {
            dismissButton.onClick();
        }
    }, [buttons]);
    const onReady = (0, react_1.useCallback)(() => {
        // We focus the dialog once it's ready to make sure that the ESC/Enter
        // keyboard shortcuts are working.
        // https://github.com/laurent22/joplin/issues/4474
        if (webviewRef.current)
            (0, focusHandler_1.focus)('UserWebviewDialog', webviewRef.current);
    }, []);
    return (React.createElement(Dialog_1.default, { className: `user-webview-dialog ${props.fitToContent ? '-fit' : ''}` },
        React.createElement("div", { className: 'user-dialog-wrapper' },
            React.createElement(UserWebview_1.default, { ref: webviewRef, html: props.html, scripts: props.scripts, pluginId: props.pluginId, viewId: props.viewId, themeId: props.themeId, borderBottom: false, fitToContent: props.fitToContent, onSubmit: onSubmit, onDismiss: onDismiss, onReady: onReady })),
        React.createElement(UserWebviewDialogButtonBar_1.default, { buttons: buttons })));
}
exports.default = UserWebviewDialog;
//# sourceMappingURL=UserWebviewDialog.js.map