"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.setNotesSortOrder = exports.notesSortOrderNextField = exports.notesSortOrderFieldArray = void 0;
const Setting_1 = require("@joplin/lib/models/Setting");
let fields = null;
let perFieldReverse = null;
const notesSortOrderFieldArray = () => {
    // The order of the fields is strictly determinate.
    if (fields === null) {
        fields = Setting_1.default.enumOptionValues('notes.sortOrder.field').sort().reverse();
    }
    return fields;
};
exports.notesSortOrderFieldArray = notesSortOrderFieldArray;
const notesSortOrderNextField = (currentField) => {
    const fields = (0, exports.notesSortOrderFieldArray)();
    const index = fields.indexOf(currentField);
    if (index < 0) {
        return currentField;
    }
    else {
        return fields[(index + 1) % fields.length];
    }
};
exports.notesSortOrderNextField = notesSortOrderNextField;
const setNotesSortOrder = (field, reverse) => {
    // field: Sort order's field. undefined means changing a field cyclically.
    // reverse: whether the sort order is reversed or not. undefined means toggling.
    let nextField = field;
    let nextReverse = reverse;
    const currentField = Setting_1.default.value('notes.sortOrder.field');
    const currentReverse = Setting_1.default.value('notes.sortOrder.reverse');
    const enabled = Setting_1.default.value('notes.perFieldReversalEnabled');
    if (enabled) {
        if (perFieldReverse === null) {
            perFieldReverse = Object.assign({}, Setting_1.default.value('notes.perFieldReverse'));
        }
    }
    if (typeof field === 'undefined') {
        if (typeof reverse === 'undefined') {
            // If both arguments are undefined, the next field is selected.
            nextField = (0, exports.notesSortOrderNextField)(currentField);
        }
        else {
            nextField = currentField;
        }
    }
    if (typeof reverse === 'undefined') {
        if (enabled && perFieldReverse.hasOwnProperty(nextField)) {
            nextReverse = !!perFieldReverse[nextField];
        }
        else {
            nextReverse = currentReverse;
        }
    }
    if (currentField !== nextField) {
        Setting_1.default.setValue('notes.sortOrder.field', nextField);
    }
    if (currentReverse !== nextReverse) {
        Setting_1.default.setValue('notes.sortOrder.reverse', nextReverse);
    }
    if (enabled) {
        // nextField is sane here.
        nextReverse = !!nextReverse;
        if (perFieldReverse[nextField] !== nextReverse) {
            perFieldReverse[nextField] = nextReverse;
            // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Partial refactor of old code before rule was applied
            Setting_1.default.setValue('notes.perFieldReverse', Object.assign({}, perFieldReverse));
        }
    }
};
exports.setNotesSortOrder = setNotesSortOrder;
//# sourceMappingURL=notesSortOrderUtils.js.map