"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateLatestArm64Yml = void 0;
const fs = require("fs");
const path = require("path");
const crypto = require("crypto");
const calculateHash = (filePath) => {
    const fileBuffer = fs.readFileSync(filePath);
    const hashSum = crypto.createHash('sha512');
    hashSum.update(fileBuffer);
    return hashSum.digest('base64');
};
const getFileSize = (filePath) => {
    return fs.statSync(filePath).size;
};
const generateLatestArm64Yml = (info, destinationPath) => {
    if (!fs.existsSync(info.dmgPath) || !fs.existsSync(info.zipPath)) {
        throw new Error(`One or both executable files do not exist: ${info.dmgPath}, ${info.zipPath}`);
    }
    if (!info.version) {
        throw new Error('Version is empty');
    }
    if (!destinationPath) {
        throw new Error('Destination path is empty');
    }
    console.info('Calculating hash of files...');
    const dmgHash = calculateHash(info.dmgPath);
    const zipHash = calculateHash(info.zipPath);
    console.info('Calculating size of files...');
    const dmgSize = getFileSize(info.dmgPath);
    const zipSize = getFileSize(info.zipPath);
    console.info('Generating content of latest-mac-arm64.yml file...');
    if (!fs.existsSync(destinationPath)) {
        fs.mkdirSync(destinationPath);
    }
    const yamlFilePath = path.join(destinationPath, 'latest-mac-arm64.yml');
    const yamlContent = `version: ${info.version}
files:
  - url: ${path.basename(info.zipPath)}
    sha512: ${zipHash}
    size: ${zipSize}
  - url: ${path.basename(info.dmgPath)}
    sha512: ${dmgHash}
    size: ${dmgSize}
path: ${path.basename(info.zipPath)}
sha512: ${zipHash}
releaseDate: '${info.releaseDate}'
`;
    fs.writeFileSync(yamlFilePath, yamlContent);
    console.log(`YML file for version ${info.version} was generated successfully at ${destinationPath} for arm64.`);
    const fileContent = fs.readFileSync(yamlFilePath, 'utf8');
    console.log('Generated YML Content:\n', fileContent);
    return yamlFilePath;
};
exports.generateLatestArm64Yml = generateLatestArm64Yml;
//# sourceMappingURL=generateLatestArm64Yml.js.map